﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;

namespace Revision.Models
{
    /// <summary>
    /// One of several options in a multiple-choice question
    /// </summary>
    public class CheckboxAnswer : BaseQA, IPossibleAnswer
    {
        /// <summary>
        /// The possible answer
        /// </summary>
        public string AnswerText { get; set; }

        /// <summary>
        /// Does the answer contain source code?
        /// (which must be formatted as-is)
        /// </summary>
        public bool IsSourceCode { get; set; }

        /// <summary>
        /// Is this one of the user's chosen answers to the multiple choice question? Variable.
        /// </summary>
        public bool ChosenAnswer { get; set; }

        /// <summary>
        /// Text answer (not relevant for Markable questions like this one)
        /// </summary>
        [AllowHtml]
        public string ChosenText { get; set; }

        ////////////////////

        /// <summary>
        /// Is this one of the correct answers? Fixed, unchangeable.
        /// </summary>
        public bool IsCorrect { get; set; }

        /// <summary>
        /// Can this answer be marked? Yes
        /// </summary>
        public virtual bool IsMarkable { get { return true; } }

        /// <summary>
        /// Did the user get this answer correct? 
        /// </summary>
        public bool IsAnsweredCorrectly 
        {
            get { return ChosenAnswer == IsCorrect; }
        }

        /// <summary>
        /// Did the user answer the question, at all?
        /// </summary>
        public bool IsAnswered 
        {
            get { return ChosenAnswer; }
        }


        /// <summary>
        /// Explanation for this answer being right/wrong
        /// </summary>
        public string Explanation { get; set; }

        /// <summary>
        /// Resets the answer to this question
        /// </summary>
        public void Clear()
        {
            ChosenAnswer = false;
        }
    }
}