﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace Revision.Models
{
    /// <summary>
    /// Exam Ref 70-487: Developing Windows Azure and Web Services
    /// </summary>
    public class ExamAzure : Exam
    {
        /// <summary>
        /// Exam Ref 70-487: Developing Windows Azure and Web Services
        /// </summary>
        public ExamAzure(bool isTextRedacted) 
            : base(title: "Developing Windows Azure and Web Services (70-487)",
                   examCode: "70-487",
                   passMark: 70)
        {
            IsTextRedacted = isTextRedacted;

            ///////////////////////// Chapter 1 ///////////////////////

            // Section 1.1
            CurrentBackground = @"Accessing data - Choose data access technologies";

            ET("1.1", 
               @"DataSet or DataReader?", 
               @"Although ADO.NET is versatile, powerful, and easy to use, it’s the simplest of the choices
available. When studying for the exam, you won't have to focus on learning every nuance
of every minor member of the System.Data or System.Data.SqlClient namespace. This is a
Technical Specialist exam, so it serves to verify that you are familiar with the technology
and can implement solutions using it. Although this particular objective is titled “Choose
data access technology,” you should focus on how you’d accomplish any given task and
what benefits each of the items brings to the table. Trying to memorize every item in each
namespace is certainly one way to approach this section, but focusing instead on “How do
I populate a DataSet containing two related tables using a DataAdapter?” would probably
be a much more fruitful endeavor.");

            ET("1.1",
                @"ObjectContext entities",
                @"As noted earlier, you need to pay attention to the derived class that the given context
is derived from because it is likely to be included in the exam. The same is the case with
entities that derive from the EntityObject base class. These entities have several attributes
decorating the class definition that you should be familiar with. There are also several similar
distinctions surrounding properties of each entity class (or attributes, as they are called
in the conceptual model).");

            MC("1.1 (1)", 
               @"You are building an ADO.NET EF application. You need to add a property of type
NVARCHAR(50) named Notes to the underlying database table named Customer and
have it reflected in the Customer entity. What should you do?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Drop the Customer table and re-create it with the new field; then choose the
Generate Database From Model option.",
                        IsCorrect = true,
                        Explanation = @"This is technically correct, but dropping the table means that all the data
goes with it. That could work in some development environments, but in production
it is a terrible idea. Because the environment wasn’t mentioned, this is still a
correct answer."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add the field to the underlying table, add a Scalar property of type String to the
Customer entity, and update the mapping.",
                        IsCorrect = true,
                        Explanation = @"This would update the entity and the data source correctly."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Run the Update Model Wizard.",
                        IsCorrect = true,
                        Explanation = @"This would update the entity and the data source correctly."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Run the EDM Generator.",
                        IsCorrect = true,
                        Explanation = @"This would update the entity and the data source correctly."
                    }
                });

            MC("1.1 (2)", 
               @"You have been asked to choose a data access technology to retrieve data from a SQL
Server database and have it exposed over the web. The application will be consumed
exclusively by external parties that have no access to the internal database. Which data
access technologies should you recommend?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"LINQ-to-SQL",
                        IsCorrect = false,
                        Explanation = @"LINQ-to-SQL will facilitate data access, but does nothing to expose the
data to the web."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"ADO.NET Entity Framework",
                        IsCorrect = false,
                        Explanation = @"The EF will facilitate data access, but does nothing to expose the data to
the web."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"WCF Data Services",
                        IsCorrect = true,
                        Explanation = @"This would enables you to meet all the data requirements as well as the
transmission requirements."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"ADO.NET Data Services",
                        IsCorrect = false,
                        Explanation = @"This technology no longer exists."
                    }
                });

            SC("1.1 (3)", 
               @"You are working with an EDM and suspect there’s a problem with the mappings. Which
file should you look at to see the underlying code?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"CSDL file",
                        IsCorrect = false,
                        Explanation = @"This file technically no longer exists, so wouldn’t be appropriate."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"SSDL file",
                        IsCorrect = false,
                        Explanation = @"This file technically no longer exists, so wouldn’t be appropriate."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"MSL file",
                        IsCorrect = false,
                        Explanation = @"This file technically no longer exists, so wouldn’t be appropriate."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"EDMX file",
                        IsCorrect = true,
                        Explanation = @"Each of the preexisting files, including the mapping files, is included here."
                    }
                });

            MT("1.1",
               @"Dealing with legacy issues",
               @"Contoso has several applications that currently use ADO.NET to interact with the
database. Each application used to run locally inside the corporate network. Now,
there’s tremendous pressure to expose many of them to the outside world. You’re
tasked with deciding whether ADO.NET should continue to be used or whether
a move should be made to another technology. If a move away from ADO.NET is
made, you need to decide which other technology should be used.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"Should you continue building applications with ADO.NET?",
                       Explanation = @"If you continue to build using ADO.NET, meeting application requirements will be increasingly
difficult. It would provide consistency, but any functionality that needs to be
exposed to the outside world will take more and more effort (arguably wasted effort
to accomplish)."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Does using the EF provide any benefit toward the stated goal that ADO.NET
doesn’t?",
                       Explanation = @"In and of itself, the EF doesn’t get you closer to the goal, but it would enable easier
interoperability with WCF Data Services, for instance. It would set the groundwork for
moving to OData."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Would WCF Data Services provide any benefit toward the stated goal that ADO.
NET doesn’t?",
                       Explanation = @"Moving legacy applications to WCF Data Services would not be trivial. They are already
built and tested. However, moving them so they could be consumed externally will
require changes, no matter how it is accomplished. WCF Data Services would allow for
all the required features that don’t currently exist."
                   }
               });



            // Section 1.2
            CurrentBackground = @"Accessing data - Implement caching";

            ET("1.2", 
               @"Using the HttpContext.Cache", 
               @"When taking the exam, you might see questions that specifically reference the HttpContext.Cache (or Page.Cache or some other similar form) directly. Such a question might be: “Datum Corporation has an application that’s currently caching several collections of data in the HttpContext.Cache object. You need to _____. How should you do it?” In such cases, you can and should make the assumption that the application is an ASP.NET application or a WCF Data Service hosted inside of ASP.NET. Everything else that makes reference to caches (from determining what mechanism should be used the whole way to specific implementations) should be assumed to be referencing the ObjectCache unless specifically stated otherwise.");

            MC("1.2 (1)", 
               @"Which of the following are provided for use by the .NET Framework as ChangeMonitors when defining a CacheItemPolicy?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"CacheEntryChangeMonitor",
                        IsCorrect = true,
                        Explanation = @"CacheEntryChangeMonitor is a valid option."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"FileChangeMonitor",
                        IsCorrect = true,
                        Explanation = @"FileChangeMonitor is a valid option."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"MsmqChangeMonitor",
                        IsCorrect = false,
                        Explanation = @"There is no such thing as an MsmqChangeMonitor."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"SqlChangeMonitor",
                        IsCorrect = true,
                        Explanation = @"SqlChangeMonitor is a valid option."
                    }
                });

            MC("1.2 (2)", 
               @"Which values are valid choices when defining the Priority property of the CacheItemPolicy using the System.Runtime.Caching version of the Cache?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Normal",
                        IsCorrect = false,
                        Explanation = @"The System.Runtime.Caching.CacheItemPriority enum does not have
a Normal option. It does exist, however, in the System.Web.Caching.CacheItemPriority enum."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"High",
                        IsCorrect = false,
                        Explanation = @"The System.Runtime.Caching.CacheItemPriority enum does not
have a High option. It does exist, however, in the System.Web.Caching.CacheItemPriority enum."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"NotRemovable",
                        IsCorrect = true,
                        Explanation = @"There is a NotRemovable option."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Low",
                        IsCorrect = false,
                        Explanation = @"The System.Runtime.Caching.CacheItemPriority enum does
not have a Low option. It does exist, however, in the System.Web.Caching.CacheItemPriority enum."
                    }
                });

            MCC("1.2 (3)", 
               @"You have set up an ObjectCache instance using the following code:

List<String> fileList = new List<String>();
fileList.Add(@""C:\SomeDirectory\SampleFile.txt"");
ObjectCache cacheInstance = MemoryCache.Default;
CacheItemPolicy accountPolicy = new CacheItemPolicy();
accountPolicy.Priority = CacheItemPriority.Default;
accountPolicy.AbsoluteExpiration = DateTime.Now.AddMinutes(60);
accountPolicy.ChangeMonitors.Add(new HostFileChangeMonitor(fileList));
CacheItem exampleItem1 = new CacheItem(""ExampleItemId"", ""Example Item Value"", ""AccountObjects"");

Which of the following items add an item to the cache with a key named “ExampleItemId,” a value of “Example Item Value,” a region named “AccountObjects,” and a CacheItemPolicy with a Default CacheItemPriority?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"cacheInstance.Add(exampleItem1, accountPolicy);",
                        IsCorrect = true,
                        Explanation = @"This would accomplish the requirements."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"cacheInstance.Add(exampleItem1, accountPolicy.Priority.Default);",
                        IsCorrect = false,
                        Explanation = @"The second parameter cannot be a CacheItemPriority. It will not compile."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"cacheInstance.Add(“ExampleItemId”, “Example Item Value”, accountPolicy);",
                        IsCorrect = false,
                        Explanation = @"This does not set the Region name."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"cacheInstance.Add(“ExampleItemId”, “Example Item Value”, accountPolicy, “AccountObjects”);",
                        IsCorrect = true,
                        Explanation = @"This would accomplish the requirements."
                    }
                });

            MT("1.2",
               @"Develop a caching strategy",
               @"Contoso has determined that its primary database is experiencing tremendous traffic
volumes. Upon analysis, it determines that much of the traffic is around identical
requests.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"What type of data would you consider as a candidate for caching?",
                       Explanation = @"The more static and less volatile it is, the better the candidate is for caching."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What factors would you consider when determining a caching strategy?",
                       Explanation = @"How frequent repeat trips are to the source coupled with the size of it. At some point,
reducing trips to the database can be offset by size of the data. Local storage or cache
storage sizes can be burdensome, so caching isn’t a magic bullet."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What would you contemplate regarding data changes?",
                       Explanation = @"The more frequently data changes, the less likely it is to be a viable candidate for caching.
Monitoring is not inexpensive, and although very helpful in some cases, it would
offset the benefit of caching. In the same respect, if the expiration policy is very small,
the benefit of caching is diminished. If monitoring needs to happen in most cases,
any benefit from caching will be greatly reduced, and a scenario could arise in which
things are much worse because of it. Such scenarios are atypical and usually arise from
overuse, but they do happen."
                   }
               });


            // Section 1.3
            CurrentBackground = @"Accessing data - Implement transactions";

            ET("1.3", 
               @"Specifying a transaction isolation level", 
               @"The IsolationLevel enumeration values have not changed since the enumeration was introduced initially. Table 1-6 covers each of the values and what they do, but you would be well advised to learn each of these and understand them. The official documentation for each behavior is available on MSDN at the following URL: http://msdn.microsoft.com/en-us/library/system.data.isolationlevel.aspx. Because questions regarding IsolationLevel are very likely to appear on the exam, by understanding what each level does, you’ll be able to distinguish the correct answer based on requirements mentioned in the question. You’ll likely see something in a question stub indicating that you need to allow or prevent exclusive range locks or you need to ensure that users are prevented from reading data locked by other transactions. Such verbiage is a dead giveaway to which isolation level is correct.");

            MC("1.3 (1)", 
               @"You are developing an ASP.NET application that reads data from and writes data to a
SQL Server database. You should avoid nonrepeatable reads and not concern yourself
with phantom data. Which isolation level should you use?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"ReadUncommitted",
                        IsCorrect = false,
                        Explanation = @"ReadUncommitted would result in nonrepeatable reads."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"RepeatableRead",
                        IsCorrect = true,
                        Explanation = @"RepeatableRead would be the loosest IsolationLevel that would meet
these requirements."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Chaos",
                        IsCorrect = false,
                        Explanation = @"Chaos does not work with SQL Server."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Snapshot",
                        IsCorrect = true,
                        Explanation = @"Snapshot would meet these requirements but might be overkill."
                    }
                });

            SC("1.3 (2)", 
               @"Which items would benefit from being made transactional?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Long-running queries that span multiple tables",
                        IsCorrect = false,
                        Explanation = @"Long running operations that span several tables may necessitate transactional
support, but the duration and complexity would generally pose serious
locking and contention issues."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Quick-running queries that involve only one table",
                        IsCorrect = true,
                        Explanation = @"Small fast operations are generally the best candidates for transactions."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Queries that make use of the file system on a Windows operating system.",
                        IsCorrect = false,
                        Explanation = @"Operations that involve the file system would generally be problematic
and would not benefit from transactions."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Queries that make use of the file system of a non-Windows operating system.",
                        IsCorrect = false,
                        Explanation = @"A non-Windows operating system would present many problems and
would not be a good candidate for transactional support."
                    }
                });

            MT("1.3",
               @"Working with transactions",
               @"Contoso has several mission-critical processes that are starting to cause problems.
Users are reporting increasingly frequent processing errors. Additionally, there are
several problems with partial or incomplete records being created. For instance,
users are receiving Order Processing Failure messages and then getting tracking
numbers for their order.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"What would make an operation a good candidate for a transaction?",
                       Explanation = @"Clearly defined boundaries are the main consideration. The more items affected in a
transaction and the longer an operation takes, the more problems you’ll see related to
making it transactional."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Why not make every operation transactional?",
                       Explanation = @"There are several costs associated with implementing transactions, and these costs can
add up quickly. If every operation were transactional, it would add tremendous cost
without necessary benefit."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What considerations would guide the choice of transaction implementation?",
                       Explanation = @"The importance of the data is one main consideration. How long-running the process
is would be another consideration. The number of resources affected would be
another one."
                   }
               });


            // Section 1.4
            CurrentBackground = @"Accessing data - Implement data storage in Windows Azure";

            SC("1.4 (1)", 
               @"You are developing an ASP.NET application that hosts an online chat room. What Windows Azure Storage options should you consider for storing the contents of these chat rooms?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Table storage",
                        IsCorrect = true,
                        Explanation = @"Chatroom data is mostly nonrelational conversations that have structure
to them. This is a perfect candidate for Table storage."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Queue storage",
                        IsCorrect = false,
                        Explanation = @"Queues are not a good mechanism at all for this type of data because
there is no way to query and fetch data based on those results."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Blob storage",
                        IsCorrect = false,
                        Explanation = @"Blob storage is a terrible option for this type of data because it would
lose any structure the data had and would be on a slow medium."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"CDN storage",
                        IsCorrect = false,
                        Explanation = @"The CDN would not work for this type of data."
                    }
                });

            MC("1.4 (2)", 
               @"Benefits of using Windows Azure Content Delivery Network include",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Increased performance for users geographically close to the content source.",
                        IsCorrect = false,
                        Explanation = @"CDN allows manipulation of nodes to reduce intermediate traffic. The
whole idea is to have nodes closer to the traffic source. Little or no benefit will be
realized if the traffic source is already close."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Increased performance for users geographically far from a content source.",
                        IsCorrect = true,
                        Explanation = @"By specifying nodes closer to the traffic source, increased performance
can be realized."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Large distributed scale to better handle instanteously increased loads.",
                        IsCorrect = true,
                        Explanation = @"CDN helps distributed scale issues and is particularly well suited to traffic
spikes and surges (such as ones associated with product launches)."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Improved security.",
                        IsCorrect = false,
                        Explanation = @"CDN doesn’t offer any security advantages that aren’t already present
in Azure offerings that do not take advantage of it."
                    }
                });

            SC("1.4 (3)", 
               @"Which of the following queries take advantage of the Content Delivery network?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"http://atlanta.vo.mysite.net/documents",
                        IsCorrect = true,
                        Explanation = @"An Azure CDN URL is in the format http://<identifier>.vo.mscend.net/."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"http://myaccount.blob.core.windows.net/documents",
                        IsCorrect = false,
                        Explanation = @"This is a standard Azure storage URL."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"http://myaccount.atlanta.blob.core.windows.net/documents",
                        IsCorrect = false,
                        Explanation = @"This is a standard Azure storage URI with a misplaced identifier."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"http://atlanta.mysite.net/documents",
                        IsCorrect = false,
                        Explanation = @"This is a completely malformed Azure storage URI."
                    }
                });

            MT("1.4",
               @"Choosing a Windows Azure strategy",
               @"Your company is looking to move its core applications to the cloud. You have two
primary applications: one that hosts video training files and another that provides
advanced statistics about user interaction patterns. All the consumers of the application
are currently clustered around three major metropolitan U.S. cities.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"What storage options would you consider for each of the two scenarios?",
                       Explanation = @"Blog storage for the video and table storage for the statistical information would probably
be best here."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Would one storage option suffice or would you want more than one?",
                       Explanation = @"It might be possible to use just one approach. However, although you could use Blob
storage to store the statistical information, you couldn’t use table storage for the media
files in any practical sense."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Would this be a good candidate for the CDN? Why or why not?",
                       Explanation = @"If consumption is all clustered in well-defined areas, using CDN could be very
beneficial."
                   }
               });


            // Section 1.5
            CurrentBackground = @"Accessing data - Create and implement a WCF Data Services service";

            ET("1.5", 
               @"Creating a query", 
               @"Most developers and even computer science students who are not just starting their
programs are familiar with SQL. SQL is easy to learn, and it’s so common that most people
have come across it. Although I can run through every single query and feature available
through OData and toying with the URIs, understanding how different URIs translate to
SQL might be helpful to drive home your understanding. I’ve seen quite a few cases in
which people have missed questions on this exam that they unquestionably knew the
SQL for, but got confused on the OData portion. If you create a service, turn on Profiler
in SQL Server and watch the results as you change different URIs; you’ll see exactly what
it’s being translated to behind the scenes. Watching what’s going on in Profiler is a very
helpful learning tool in many cases, not just this one. It does add a slight resource load to
the SQL Server, so clear it with your DBA or use an instance that you know isn’t being used
by anyone for anything important at the time. I highly recommend that you do a quick
runthrough, watching what happens in Profiler just to drive the point home if you have any
extra time to do so.");

            ET("1.5",
                @"Creating a query",
                @"In the exam, you’ll probably be presented with some definition of an Entity Model (even if it’s just a small piece of it) and then ,told that you need to “create a query that… .” You’ll then be presented with a list of possible queries that would return the desired results. Understanding the items in Table 1-12 and how they affect the resulting query is indispensable for answering such questions correctly. $orderby, $select, $top, and $inlinecount are obvious in what they do and how they are used, and there’s not much to them. That leaves $filter, $skip, and $expand to study.");

            MC("1.5 (1)", 
               @"You need to create an interceptor that runs each time you try to return Topic items. Which items should you use?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"ChangeInterceptor",
                        IsCorrect = false,
                        Explanation = @"This would be right only if the data were being changed. It’s just being
queried."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"QueryInterceptor",
                        IsCorrect = true,
                        Explanation = @"This is the only one that can handle items when they are queried."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"DataServiceProtocolVersion",
                        IsCorrect = false,
                        Explanation = @"This simply would not work for the task at hand or even come close
to it."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"SetEntitySetAccessRule",
                        IsCorrect = false,
                        Explanation = @"This simply would not work for the task at hand or even come close
to it."
                    }
                });

            SC("1.5 (2)", 
               @"You need to return all questions that have an Id greater than 5. Which query should you use?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"http://servicehost/ExamPrepService.svc/Questions?$filter=Id gt 5",
                        IsCorrect = true,
                        Explanation = @"The entity name, followed by a filter condition using gt for greater than 5
is what’s required here."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"http://servicehost/ExamPrepService.svc/Questions?$filter(ID> 5)",
                        IsCorrect = false,
                        Explanation = @"The query and filter syntax is incorrect."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"http://servicehost/ExamPrepService.svc/Questions(>5)",
                        IsCorrect = false,
                        Explanation = @"No mention is made of a filter here, and the syntax is incorrect."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"http://servicehost/ExamPrepService.svc/Questions?$select(Id) gt 5",
                        IsCorrect = false,
                        Explanation = @"The id field and operator are correct, but the syntax is incorrect."
                    }
                });

            SCC("1.5 (3)", 
               @"Assume you have the following C# code snippet.

var selectedQuestions = from q in context.Questions
where q.QuestionNumber > 30
orderby q.QuestionId descending
select q;

Which of the following URI queries is the equivalent?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"http://Service/Question.svc/Questions?Orderby=QuestionId&?$QuestionNumber ( gt 30)",
                        IsCorrect = false,
                        Explanation = @"The filter and query syntax are incorrect."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"http://Service/Question.svc/Questions?$Orderby=QuestionId&?$QuestionNumber gt 30",
                        IsCorrect = false,
                        Explanation = @"The filter and query syntax are incorrect."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"http://Service/Question.svc/Questions?Orderby=QuestionId&?filter=(QuestionNumber > 30)",
                        IsCorrect = false,
                        Explanation = @"The filter syntax is incorrect."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"http://Service/Question.svc/Questions?$Orderby=QuestionId desc&?$filter=QuestionNumber gt 30",
                        IsCorrect = true,
                        Explanation = @"The OrderBy field is specified correctly, as is the filter field."
                    }
                });

            MT("1.5",
               @"Querying data services",
               @"Your company has an application that currently is consumed by in-house applications.
There is a need to make it accessible to outside parties that use this in-house
application but extend it. As such, the capability to consume and manipulate the
data is critical. Enabling users to accomplish this through URI-based semantics
seems like the optimal solution.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"Would it be possible to provide access through URI-based semantics using Data
Services?",
                       Explanation = @"Yes, in fact, that’s the primary benefit. Items can be accessed using traditional code or
using URI-based semantics, but in this case, URI semantics would be particularly useful."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What operations are easily supported?",
                       Explanation = @"All Create, Retrieve, Update, and Delete (CRUD) operations."
                   }
               });


            // Section 1.6
            CurrentBackground = @"Accessing data - Manipulate XML data structures";

            SC("1.6 (1)", 
               @"You need to use an ampersand in the name of an element you create with an XElement item. Which of the following allows a correctly created element name?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Just pass in the string to the XElement constructor; it handles it automatically by
calling the XmlConvert class transparently.",
                        IsCorrect = false,
                        Explanation = @"Technically correct, but other answers are all correct, so choice D is the
only one."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Manually escape the character sequence.",
                        IsCorrect = false,
                        Explanation = @"Technically correct, but other answers are all correct, so choice D is the
only one."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the XmlConvert.Encode method to create the string to pass into the
constructor.",
                        IsCorrect = false,
                        Explanation = @"Technically correct, but other answers are all correct, so choice D is the
only one."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"All of the above.",
                        IsCorrect = true,
                        Explanation = @"Because each of the other answers is correct, this one is the right answer."
                    }
                });

            SC("1.6 (2)", 
               @"Which of the following are requirements for a valid and well-formed XML document?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Every element must have a corresponding closing element.",
                        IsCorrect = true,
                        Explanation = @"An element can self-close, but each opening element must be closed."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Every element must have at least one attribute.",
                        IsCorrect = false,
                        Explanation = @"Elements might have attributes, but there is no requirement for an element
to have an attribute."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Every attribute must have a corresponding closing attribute.",
                        IsCorrect = false,
                        Explanation = @"Attributes don’t require matching closure tags."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Elements and attributes can be used interchangeably as long as they have open and closing tags.",
                        IsCorrect = false,
                        Explanation = @"Elements can be nested and structured in a way that accomplishes the
same goal as attributes, but attributes can’t contain elements."
                    }
                });

            MT("1.6",
               @"Creating an XML manipulation strategy",
               @"You are building an application that makes extensive use of XML. The document
structures follow very consistent patterns, and sometimes large applications to
manipulate these structures are similar.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"Would your application benefit from using LINQ-to-XML?",
                       Explanation = @"The more consistent the structures are, the more any approach will be simplified. LINQ
semantics allow much simpler interaction with the documents, although any of the
existing .NET XML libraries can work."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What could be done to simplify consumption of the documents?",
                       Explanation = @"The more consistent the data, the more one approach can be reused. Because the
documents are all similar, the hard part, so to speak, is largely addressed. You might
consider doing an intermediate transform on the documents, though, to render them
as one very consistent format that might make it even simpler."
                   }
               });


            ///////////////////////// Chapter 2 ///////////////////////

            // Section 2.1
            CurrentBackground = @"Query and manipulate data by using the Entity Framework";

            SCC("2.1 (1)", 
               @"You create a LINQ query as follows. Which of the statements is correct?

var query = (from acct in context.Accounts
where acct.AccountAlias == ""Primary""
select Acct).FirstOrDefault();",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"A foreach loop is needed before this query executes against the database.",
                        IsCorrect = false,
                        Explanation = @"Because of the FirstOrDefault, it is executed immediately."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"NavigationProperty needs to be referenced before this query executes against the database.",
                        IsCorrect = false,
                        Explanation = @"With the FirstOrDefault, it is executed immediately and does not need
to be iterated."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The query returns results immediately.",
                        IsCorrect = true,
                        Explanation = @"Because the value is returned immediately, it works without any iteration."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"It depends on whether the LazyLoadingEnabled property is set.",
                        IsCorrect = false,
                        Explanation = @"Lazy loading is important, but not here."
                    }
                });

            MC("2.1 (2)", 
               @"Assume you call the Add method of a context on an entity that already exists in the database. Which of the following is true?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"A duplicate entry is created in the database if there is no key violation.",
                        IsCorrect = true,
                        Explanation = @"If there is no key constraint violation, a second copy of the entity will be
created in the database."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"If there is a key violation, an exception is thrown.",
                        IsCorrect = true,
                        Explanation = @"If there is a key constraint violation, an exception will be thrown."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The values are merged using a first-in wins approach.",
                        IsCorrect = false,
                        Explanation = @"The values will not be merged"
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The values are merged using a last-in wins approach.",
                        IsCorrect = false,
                        Explanation = @"The values will not be merged."
                    }
                });

            SC("2.1 (3)", 
               @"What happens if you attempt to Attach an entity to a context when it already exists in
the context with an EntityState of unchanged?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"A copy of the entity is added to the context with an EntityState of unchanged.",
                        IsCorrect = false,
                        Explanation = @"A copy is not added with an EntityState of Unchanged."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"A copy of the entity is added to the context with an EntityState of Added.",
                        IsCorrect = false,
                        Explanation = @"A copy is not added with an EntityState of Added."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Nothing happens and the call is ignored.",
                        IsCorrect = true,
                        Explanation = @"There isn’t an operation because it already exists."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The original entity is updated with the values from the new entity, but a copy is not
made. The entity has an EntityState of Unchanged.",
                        IsCorrect = false,
                        Explanation = @"The original entity is not updated or merged."
                    }
                });

            MT("2.1",
               @"Determining when to use lazy loading",
               @"You have an object model that has a relational hierarchy three levels deep. In ad-dition to one chain that nests three levels deep (parent, child, and grandchild), you have four other child relations. In total, you have a parent and five related children, and the first child has two children of its own.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"Should you pull back the entire chain and all the data, or pull back only a portion and then make roundtrips for the additional data as needed?",
                       Explanation = @"The answer depends on how the data is used. Chances are that only a portion of the
items in the chain will be referenced frequently. You might have an application that
needs all the data or most of it for every typical use case, however. It depends on the
nature of the application. It should be tested extensively, and logging and tracing
should be used to see how frequently the nonparent data is referenced."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"If you pull back all the data, what problems might you face?",
                       Explanation = @"If you pull back all the data, you might have performance or responsiveness issues,
especially if you have network constraints. If each of the children or grandchildren has
only a few records and is small in size, this will be minimized. However, if there are
substantially sized records or high record counts on any of the descendants, it might
take a lot longer to query and update and could cause serious issues with application
responsiveness."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"If you choose to lazy load, what problems might you face?",
                       Explanation = @"If you chose to lazy load the data, and it is necessary for the majority of the use cases,
you will cause several roundtrips to the server. Over a constrained network or a lowpowered
device, these frequent roundtrips could easily cause wait times that greatly
exceed the overhead of pulling all the data back at once."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"How should you go about determining which approach should be used?",
                       Explanation = @"Several factors should go into your decision. First, the number of records should be
considered. If you have only one or two records for each of the child relations, and the
record sizes are small, pulling it all down at once will probably be beneficial. But even if
the record count is small or the record size is small, if you don’t need the data, there’s
no use of pulling it down. As such, the typical use cases should be tested extensively,
and you should have logging in place to help build frequency distributions of usage.
This necessitates a good bit more work up front, both in testing and analyzing, but
will pay huge dividends down the road because getting it wrong is very expensive to
correct."
                   }
               });


            // Section 2.2
            CurrentBackground = @"Query and manipulate data - by using Data Provider for Entity Framework";

            MC("2.2 (1)", 
               @"You need to execute a query against a SQL Server database and populate a set of objects. The retrieval operation is time-consuming, and users are complaining about sluggishness. You need to make sure the application doesn’t seem to “hang” while the operation is taking place. Which of the following would accomplish that task?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the ExecuteReader method of the EntityCommand class.",
                        IsCorrect = false,
                        Explanation = @"Using the ExecuteReader method, the EntityCommand would carry as
much, and likely more, overhead than the ADO.NET equivalent. There is no benefit
in processing or on the client side."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the ExecuteScalarAsync method of the EntityCommand class.",
                        IsCorrect = false,
                        Explanation = @"Using the async call can keep the operation from blocking, but the type
here is incorrect."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the ExecuteReaderAsync method of the SqlCommand class.",
                        IsCorrect = true,
                        Explanation = @"Using the ExecuteReaderAsync in conjunction with the SqlCommand
class would accomplish this goal. Overall processing wouldn’t speed up, but the
blocking on the client would be reduced, giving the impression of a much more
responsive UI."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the ExecuteReaderAsync method of the System.Data.OleDb.Command class.",
                        IsCorrect = false,
                        Explanation = @"Provider-specific libraries are always the fastest. Because this database
is SQL Server, the SqlClient namespace can be used, and it would operate more
quickly and with more features than the more generic OleDb equivalent."
                    }
                });

            SC("2.2 (2)", 
               @"The application you’re working on uses the EF to generate a specific DbContext instance. The application enables users to edit several items in a grid and then submit the changes. Because the database is overloaded, you frequently experience situations that result in only partial updates. What should you do?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Call the SaveChanges method of the DbContext instance, specifying the UpdateBehavior.All enumeration value for the Update behavior.",
                        IsCorrect = false,
                        Explanation = @"SaveChanges can help the scenario, but only if done in the context of
another transaction. Without one, it is of no value in this case."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use a TransactionScope class to wrap the call to update on the DbContext instance.",
                        IsCorrect = false,
                        Explanation = @"A TransactionScope would be a good choice, but the DbContext doesn’t
have an Update method, so it is irrelevant."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a Transaction instance by calling the BeginTransaction method of the DbContext Database property. Call the SaveChanges method; if it is successful, call the Commit method of the transaction; if not, call the Rollback method.",
                        IsCorrect = false,
                        Explanation = @"This approach could work, but the transaction would need to be established
on the connection, not on the context."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use a TransactionScope class to wrap the call to SaveChanges on the DbContext. Call Complete if SaveChanges completes without an exception.",
                        IsCorrect = true,
                        Explanation = @"Calling SaveChanges inside of a TransactionScope and selecting the Complete
property if things are successful accomplishes this goal perfectly."
                    }
                });

            MC("2.2 (3)", 
               @"The application you’re working on uses the EF. You need to ensure that operations can be rolled back if there is a failure and you need to specify an isolation level. Which of the following would allow you to accomplish this?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"A EntityTransaction",
                        IsCorrect = true,
                        Explanation = @"An EntityTransaction would accomplish each goal as it provides rollback
functionality and setting of an isolation level."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"A TransactionScope",
                        IsCorrect = true,
                        Explanation = @"A TransactionScope would accomplish each goal as it provides rollback
functionality and setting of an isolation level."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"A SqlTransaction",
                        IsCorrect = false,
                        Explanation = @"A SqlTransaction would work, but not in this context."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"A DatabaseTransaction",
                        IsCorrect = false,
                        Explanation = @"There is not a valid class called DatabaseTransaction."
                    }
                });

            MT("2.2",
               @"When should you use Data Providers?",
               @"You have several applications that have been built using older versions of ADO.NET.
Your development team is not very familiar with the EF and is still in the learning
process. Your CTO has decided that everything should be ported to the newest version
of the .NET Framework and all data access should use the EF.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"Would using the Entity providers help facilitate the transition?",
                       Explanation = @"The syntax is much closer to old ADO.NET, and it would make some sense to take
an incremental approach. By using similar objects, testing is minimized and there is
a higher likelihood that no new bugs will be introduced. You can make the converse
argument that completely revamping everything saves some time and effort, but using
an incremental approach is probably best in this case."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Is the decision to move all code to the EF a sound decision?",
                       Explanation = @"“Sound” is somewhat relative, and if existing applications are already in place, tested,
and working, any changes introduce the possibility of new bugs. On the other hand,
the EF is clearly the choice that Microsoft has made for data access, and if it is not
done, two skill sets are ultimately needed. It really depends on how important consistency
is and how much legacy code is in place."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Should you employ any existing operations as asynchronous?",
                       Explanation = @"Asynchronous operations have been available since the inception of .NET. If the operations
should have been asynchronous initially and were implemented statically, this
provides an opportunity to fix it. On the other hand, asynchronous programming is
much more complicated, and bugs are often much harder to find and diagnose. Under
the circumstances, it is prudent to replace the existing provider code with entity data
provider code as closely as possible, unless big mistakes were made originally by overlooking
asynchronous operations."
                   }
               });


            // Section 2.3
            CurrentBackground = @"Query and manipulate data by using the Entity Framework - LINQ to Entities";

            ET("2.3", 
               @"Querying data using LINQ operators", 
               @"You can use the var keyword, but an exam question might call it out with either IQueryable
or IEnumerable. Note, however, that this doesn’t mean one or the other is being used; both
support either/both interface types.");

            MC("2.3 (1)", 
               @"You are executing a query against a LINQ to EDM that contains an Account entity, among others. Which of the following are valid choices for storing the data?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"IListCollection<Account>",
                        IsCorrect = false,
                        Explanation = @"IListCollection is not a valid interface and would not work."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"ObjectQuery<Account>",
                        IsCorrect = true,
                        Explanation = @"ObjectQuery implements IOrderedQueryable, IQueryable, IEnumerable,
IListSource, and Account is a valid type."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"ObjectContext.Account",
                        IsCorrect = false,
                        Explanation = @"The ObjectContext base class does not have or use a custom Account
entity."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"IQueryable<Account>",
                        IsCorrect = true,
                        Explanation = @"IQueryable passing in the Account type will definitely work."
                    }
                });

            //**bug**

            MC("2.3 (3)", 
               @"Which interfaces must be implemented in order for something to be queried by LINQ?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"IEnumerable",
                        IsCorrect = true,
                        Explanation = @"The item must be either IEnumerable or IQueryable."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"IQueryable",
                        IsCorrect = true,
                        Explanation = @"The item must be IEnumerable or IQueryable."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"IEntityItem",
                        IsCorrect = false,
                        Explanation = @"IEntityItem will not work and has no value here."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"IDbContextItem",
                        IsCorrect = false,
                        Explanation = @"IDbContextItem will not work and has no value here outside of the current
zone."
                    }
                });

            MT("2.3",
               @"What can you do to examine the SQL statements being
generated to determine if they are the problem?",
               @"You’re using LINQ to Entities and are experiencing serious performance problems.
Your database administrators (DBAs) insist that your queries are the problem, but
your developers tell you the opposite.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"How can you determine the problem source?",
                       Explanation = @"Obviously, logging is needed to determine what SQL is being sent back to the database.
It is possible that the problem is the queries, the database, or both. The only way
to know is to rule out one thing at a time—in this case, start with the entity logging."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What features should you implement?",
                       Explanation = @"Depending on the source, the LINQ queries might need restructuring. Lazy loading
and caching might be in order. Moving the queries to stored procedures might help
(not because stored procedures are inherently faster, but because they would yield the
same queries each time, which in turn might help with cached execution plans)."
                   },
               });


            // Section 2.4
            CurrentBackground = @"Query and manipulate data by using the Entity Framework - ADO.NET";

            MCC("2.4 (1)", 
               @"You need to execute the following SELECT query using the COUNT aggregate to determine the total record count in a table:

SELECT COUNT(*) FROM ExamQuestions;

Which of the following enables you to do that?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"SqlDataAdapter in conjunction with a SqlDataReader",
                        IsCorrect = false,
                        Explanation = @"A SqlDataAdapter cannot be used with a SqlDataReader together at all."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"SqlDataAdapter in conjunction with a DataTable",
                        IsCorrect = true,
                        Explanation = @"A SqlDataAdapter and a DataSet will work for this (use the Rows property
of the primary DataTable)."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"SqlDataAdapter in conjunction with a DataSet",
                        IsCorrect = true,
                        Explanation = @"A SqlDataAdapter and a DataTable will work for this (use the Rows property
of the DataTable)."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The SqlCommand ExecuteScalar method",
                        IsCorrect = true,
                        Explanation = @"ExecuteScalar would work assuming that the COUNT aggregate function
was used, which it was."
                    }
                });

            MC("2.4 (2)", 
               @"Which of the following are true about SqlConnection objects?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"They should be opened at the last possible moment and closed at the earliest
opportunity.",
                        IsCorrect = true,
                        Explanation = @"Leaving connections open when they aren’t being used is wasteful and
can cause serious performance problems. Accordingly, they should always be
opened at the last possible moment before they are needed and closed immediately
when they are no longer needed."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"They should be opened as early as possible and closed as early as possible.",
                        IsCorrect = false,
                        Explanation = @"Opening a connection before it is needed is wasteful and could have
serious performance implications."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"They should be opened as late as possible and closed as late as possible.",
                        IsCorrect = false,
                        Explanation = @"Leaving a connection open when it is no longer needed is wasteful and
could cause serious performance problems."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"As long as the Close statement is called in a catch block, the object will be closed
and disposed of correctly.",
                        IsCorrect = false,
                        Explanation = @"Calling Close in a Finally block, or wrapping it in a using statement
would ensure that it is closed properly. If Close is called inside a Catch block, it is
entirely possible that another exception could be thrown there, leading to the connection
dangling unclosed."
                    }
                });

            SC("2.4 (3)", 
               @"What are the advantages of a compiled query?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"None. Every time you run the query you will hit the database.",
                        IsCorrect = false,
                        Explanation = @"It’s true that you will hit the database each time you execute the query.
But there is still a performance gain because a compiled query caches the steps
required to convert your query to SQL."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The results of the query are cached making the querying a lot faster.",
                        IsCorrect = false,
                        Explanation = @"It’s not the results of the query that are cached. Instead the generated
SQL is cached. This can be parameterized to get specific data from the database."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The translation of your query into SQL is cached.",
                        IsCorrect = true,
                        Explanation = @"By caching the generated SQL a compiled query can execute faster."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"None. You can’t change any of the parameters you use in a query, rendering the
compiled query useless.",
                        IsCorrect = false,
                        Explanation = @"The generated SQL is cached but it’s also parameterized so you can
specify new values for your parameters on each execution."
                    }
                });

            MT("2.4",
               @"Performance problems",
               @"You’ve made extensive use of DataAdapters, and your application has been performing
well. Lately, though, it has started to slow down dramatically, and users
report that the longer they use the application, the more sluggish it becomes.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"One manager is sure that it is a memory leak or something related to not closing
connections, How can you determine the validity of such a statement?",
                       Explanation = @"If you’re opening the adapters, there’s a possibility they aren’t being closed and
disposed of properly. However, if you’re relying on the adapter to open and close the
connection, it is unlikely that it is the problem. The more likely issue is that you are
caching too much data locally, and it is being held on to well past its useful life. Less
caching and more round trips to the database might actually help in this case, depending
on the exact nature of the problem, but each area should be looked at."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What might you do to speed things up?",
                       Explanation = @"Overuse of a DataAdapter could be a problem. Remember that an adapter always uses
a DataReader. There might be times when individual values are being queried using
an adapter that could be replaced with a DataReader. You might be able to determine
counts and aggregates from the returned data instead of executing aggregate queries.
You might be able to rely less on joins and use DataRelations locally to enforce constraints.
You could also consider disabling and reenabling constraints before and after
load operations."
                   },
               });


            // Section 2.5
            CurrentBackground = @"Query and manipulate data - Create an Entity Framework data model";

            MC("2.5 (1)", 
               @"Which of the following are necessary components of an EDM?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"SSDL",
                        IsCorrect = true,
                        Explanation = @"The SSDL file is necessary to define the database schema information."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"MSL",
                        IsCorrect = true,
                        Explanation = @"The MSL file is necessary to define the mappings between the other two
files."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"CSDL",
                        IsCorrect = true,
                        Explanation = @"The CSDL file is necessary to define the entity layer of the application."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @".edmx",
                        IsCorrect = true,
                        Explanation = @"The MSL, CSDL, and SSDL files are all necessary and are rolled into the
.edmx file."
                    }
                });

            MC("2.5 (2)", 
               @"You have been tasked with completing a coworker’s data model. You are told that
there are some specific issues in the .ssdl file and the .msl file, but when you look at the
project, you don’t see either of them. What is the problem?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The actual .csdl file no longer exists because it has been consolidated in the .edmx
files. Its logical purpose is still contained and present, however.",
                        IsCorrect = true,
                        Explanation = @"The CSDL file doesn’t physically exist in later versions of EF. However, the
core of what they do is still needed; they just take a different form and are consolidated
inside the .edmx model."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The actual .msl file no longer exists because it has been consolidated in the .edmx
files. Its logical purpose is still contained and present, however.",
                        IsCorrect = true,
                        Explanation = @"The MSL file doesn’t physically exist in later versions of EF. However, the
core of what they do is still needed; they just take a different form and are consolidated
inside the .edmx model."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The actual .csdl file no longer exists because it has been consolidated in the .edmx
files. Because it is no longer needed, there is no need to reference it or for anything
to try to replace it.",
                        IsCorrect = false,
                        Explanation = @"The CSDL file contents might be incorporated into the .edmx file, but
the service it provides is still necessary. Without an equivalent functionality, the
model will not work."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The actual .msl file no longer exists because it has been consolidated in the .edmx
files. Because it is no longer needed, there is no need to reference it or for anything
to try to replace it.",
                        IsCorrect = false,
                        Explanation = @"The MSL file contents might be incorporated into the .edmx file, but the
service it provides is still necessary. Without an equivalent functionality, the model
will not work."
                    }
                });

            MC("2.5 (3)", 
               @"You are building an ADO.NET Entity Framework application. You need to validate each
of the models. Which Entity Data Model tool can you use to accomplish this?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The EDM Generator (EdmGen.exe)",
                        IsCorrect = true,
                        Explanation = @"The EdmGen.exe tool can be used with the ValidateArtifacts switch to
validate a model."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"ADO.NET Entity Data Model Designer",
                        IsCorrect = true,
                        Explanation = @"The ADO.NET entity model designer can be used to validate models."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Entity Data Model Wizard",
                        IsCorrect = false,
                        Explanation = @"The entity data model wizard may validate a model during creation but
will not perform the validation after the fact."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Update Model Wizard",
                        IsCorrect = false,
                        Explanation = @"The Update Model Wizard is intended to let you update an .edmx file
after changes have been made to the underlying database. It is not intended to
perform validation."
                    }
                });

            MT("2.5",
               @"Entity model issues",
               @"You have a huge set of legacy code that employs traditional ADO.NET. You have
been charged with moving it to the EF.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"Should you use POCOs or use entities exclusively?",
                       Explanation = @"POCOs might make sense; they might not. The more you’re moving to Entities, the less
relevant POCOs will generally be, although that’s a general statement and there are
plenty of exceptions. If you have the POCOs already, it would help make the case to
continue their use. If not, and you’re using DataSets and DataTables exclusively, moving
to Entities completely would probably make more sense."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Would it make sense to use EF to populate your existing business objects and
leave everything else in place?",
                       Explanation = @"One could make the case that just changing the plumbing, so to speak, would be a
good intermediate strategy. But generally it is not worth the effort. If there’s a ton
of code with a lot of complex UI behavior, it could provide some benefit because
you wouldn’t need to regression test as much. On the other hand, for smaller-sized
projects, it is probably just a waste of extra effort with very little value proposition."
                   },
               });



            ///////////////////////// Chapter 3 ///////////////////////


            // Section 3.1
            CurrentBackground = @"Create a WCF service";

            ET("3.1", 
               @"Creating contracts", 
               @"On the exam, if you see a custom data type, remember that you must be able to serialize it. As such, the class definition should be decorated with a DataContract attribute or another serializer type. Custom exception types can also be presented. These should be decorated with a FaultContract attribute.");

            ET("3.1", 
               @"Endpoints", 
               @"Although custom bindings are something you need to be familiar with if you are planning
on working with WCF, anything more complicated than their simple existence is unlikely to
appear on the exam.");

            ET("3.1", 
               @"DataMember", 
               @"On the exam, EmitDefaultValue might be used in a question about how the size of the serialized message must be kept to an absolute minimum or about it might be mentioned as part of interoperability issues and the serialization format specifically.");

            SCC("3.1 (1)", 
               @"You’ve implemented a method named GetQuestionText that can throw an IndexOutOfRangeException a SqlException. Which of the following items would correctly expose these exceptions?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"[FaultContract(typeof(IndexOutOfRangeException))]
[FaultContract(typeof(SqlException))]
[OperationContract]
String GetQuestionText(Int32 questionNumber);",
                        IsCorrect = true,
                        Explanation = @"A FaultContract is specified using each of the two types that need to be
supported."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"[FaultMapping(new FaultMappingCollection[]{IndexOutOfRangeException,SqlException})]
[OperationContract]
String GetQuestionText(Int32 questionNumber);",
                        IsCorrect = false,
                        Explanation = @"There is no relevant FaultMapping attribute."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"[FaultMapping(new FaultMappingCollection[]{typeof(IndexOutOfRangeException), typeof(SqlException)})]
[OperationContract]
String GetQuestionText(Int32 questionNumber);",
                        IsCorrect = false,
                        Explanation = @"There is no relevant FaultMapping attribute."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"[FaultContract(IndexOutOfRangeException)]
[FaultContract(SqlException)]
[OperationContract]
String GetQuestionText(Int32 questionNumber);",
                        IsCorrect = false,
                        Explanation = @"The FaultContract definition attribute is the correct one to use, but the
type needs to be specified in conjunction with the Type."
                    }
                });

            MC("3.1 (2)", 
               @"Which of the following are true regarding WCF Services and their contracts?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Each service must have a corresponding Interface class.",
                        IsCorrect = false,
                        Explanation = @"A service needs a “contract,” and that’s usually handled via an interface,
but it does not necessarily have to be an interface."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The ServiceContract and OperationContract attributes must be applied to the
contract, not the implementation.",
                        IsCorrect = false,
                        Explanation = @"The ServiceContract and OperationContract attributes need to be applied
to either the interface or the class definition."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The ServiceContract and OperationContract attributes can be applied to either the
contract or the implementation.",
                        IsCorrect = true,
                        Explanation = @"The ServiceContract attribute and the OperationContract attribute can be
applied to either the interface or the implementation, but not both."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"A complex type does not need to be serializable as long as all its properties are
serializable.",
                        IsCorrect = false,
                        Explanation = @"Any type transferred through the service must be serializable."
                    }
                });

            MC("3.1 (3)", 
               @"You are tasked with adding a method named GetCurrentStockPrice to an existing service.
The Service is named QuoteService and it implements the IQuoteService Contract.
How should you go about adding this new method?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add the method definition to the IQuoteService Contract.",
                        IsCorrect = true,
                        Explanation = @"The method definition should be added to the service contract."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add the OperationContract attribute to the method definition.",
                        IsCorrect = true,
                        Explanation = @"The OperationContract attribute must be added to the method or the
method definition. In this case, it should be the method."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add the OperationBehavior attribute to the method definition.",
                        IsCorrect = false,
                        Explanation = @"Adding an OperationBehavior would be of no benefit here."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a new ServiceContract class and add the GetCurrentStockPrice method
to it.",
                        IsCorrect = false,
                        Explanation = @"Adding a new ServiceContract would not do any good without making
the service implement the contract as well."
                    }
                });

            MT("3.1",
               @"Contract implementation?",
               @"You are building a WCF Service. There are several methods that you do not want to
expose to clients but need to use internally. Several complex types, including enumerations,
will be transferred between the client and server. Although the majority
of your clients are using .NET, you want to make sure that any decision errs on the
side of interoperability.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"How should you handle the methods that you don’t want publicly exposed?",
                       Explanation = @"You have a few options here. One straightforward method would be to mark only the
methods you want publicly exposed with the OperationContract attribute and leave
the other ones undefined. You could alternately move all the private methods to another
interface and have the service implement both the original interface and the new
one."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"How should you handle the enumeration types and the complex types?",
                       Explanation = @"You have several options, but the most straightforward would be to use the
DataContract serializer for both the complex types and the enumeration types. This
would add a layer of consistency while keeping things simple. On the enumeration
values, you would decorate them with the EnumMember attribute. On the members of
the complex type you want exposed, you would decorate them with the DataMember
attribute."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Should anything special be done to deal with exceptions?",
                       Explanation = @"To maximize interoperability, there are several things that can be done including the
choice of bindings (however, that’s not been covered yet). The main thing so far would
be to make sure that you use FaultContracts to wrap exceptions and pass them back to
the client."
                   }
               });



            // Section 3.2
            CurrentBackground = @"Configure WCF services by using configuration settings";

            ET("3.2", 
               @"Config files", 
               @"There are several questions in the exam that show you a section of a configuration file on
one side of the screen, with blank boxes over key elements, and the available choices on
the other side of the screen. You are asked to drag and drop those elements in the correct
places. You need to get all the items correct in order to get the question right.");

            ET("3.2", 
               @"Configuring bindings", 
               @"Remember that, even if you don’t know the answer to a question, if you can eliminate any wrong choices you immediately increase your chances of getting the item correct. On the new versions of the exam, in which you have all the drag-and-drop items, you have to get several things right in order to get the item correct, and you might find a question that has three empty boxes that need answers, but x (where x > 3) available choices. In the previous item, just by reviewing the resulting items that are created from the configuration file, you can know that the answer will have either binding=somebinding or bindingconfiguration=someconfiguration (or both) because they are the only two possible attributes that are available outside of scheme. If the only bindings you saw were netMsmqBinding or new NamedPipeBinding, that is your answer, even if the use case isn’t something that you would typically want to do because no other attributes are allowed to exist there.");

            MC("3.2 (1)", 
               @"What are the minimum steps that must be performed to configure a service using the WCF Service Configuration Editor?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Specify a new service.",
                        IsCorrect = true,
                        Explanation = @"A new service must be specified."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Specify a MetadataExchange endpoint.",
                        IsCorrect = false,
                        Explanation = @"Although useful, particularly for discovery, a MetadataExchange endpoint
does not need to be configured for a service to correctly operate."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Specify a security configuration.",
                        IsCorrect = false,
                        Explanation = @"Not specifying security is ill-advised, but the default settings work if no
security values are configured. Hence, it is not absolutely necessary to operate."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Specify a contract.",
                        IsCorrect = true,
                        Explanation = @"A contract absolutely must be specified, along with defining the service
itself."
                    }
                });

            MC("3.2 (2)", 
               @"Which of the following items can be handled through configuration?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Defining the service’s type.",
                        IsCorrect = true,
                        Explanation = @"A new service can be defined through configuration."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Defining the service’s contract.",
                        IsCorrect = true,
                        Explanation = @"A service’s contract type can be defined through configuration."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Specifying a security configuration.",
                        IsCorrect = true,
                        Explanation = @"A service’s security configuration can be defined through configuration."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Adding metadata exchange.",
                        IsCorrect = true,
                        Explanation = @"Adding metadata exchange can be implemented through configuration."
                    }
                });

            SC("3.2 (3)", 
               @"Which of the following is not a benefit of configuration?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Ability to change Endpoint URIs",
                        IsCorrect = false,
                        Explanation = @"Endpoint URIs can be easily changed through configuration."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Ability to modify security settings",
                        IsCorrect = false,
                        Explanation = @"A service’s security settings can be easily changed through
configuration."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Ability to redefine metadata exchange",
                        IsCorrect = false,
                        Explanation = @"Metadata exchange can be added, removed, or modified through
configuration."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Ability to add new method definitions",
                        IsCorrect = true,
                        Explanation = @"Changes to a method definition or implementation can be made cited
through code."
                    }
                });

            MT("3.2",
               @"What would you configure?",
               @"You are building a WCF Service. You expect a tremendous growth in clients and will
likely have to support and change quite a few different features. Your service will be
hosted internally, so you have full access to it without any real restrictions.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"Would it make sense to use configuration to host custom bindings?",
                       Explanation = @"It would definitely make sense to use configuration for custom bindings (and any others,
for that matter). You could use an imperative approach for items you are absolutely
sure of, but you can do everything discussed here through configuration, so there’s
probably a big benefit to keeping things consistent."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What bindings would you choose to configure?",
                       Explanation = @"You could certainly go ahead and add every binding you can imagine that you’ll need.
Multiple bindings can be supported, although the particular host can limit what is
supported. The best approach, however, is to go ahead and set up the items that you
know for sure you need now. You can easily add new bindings and tweak the configuration
as you go along and find out about new requirements. Configuring everything
up-front has many downsides, and the cost of adding a new configuration binding is
relatively small."
                   },
               });



            // Section 3.3
            CurrentBackground = @"Configure WCF services by using the API";

            MC("3.3 (1)", 
               @"You are creating a CustomBinding that will be consumed only by .NET clients. Performance is the primary concern. Which of the following are true?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You must define a TextMessageEncodingBindingElement to the binding.",
                        IsCorrect = false,
                        Explanation = @"If they were both included, there would be a conflict, and one wouldn’t
take precedence or overwrite the other."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You must define a BinaryMessageEncodingBindingElement to the binding.",
                        IsCorrect = false,
                        Explanation = @"Defining the same binding twice would not cause any compilation
issues."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You must not define either the TextMessageEncodingBindingElement or the MTOMMessageEncodingBindingElement.",
                        IsCorrect = false,
                        Explanation = @"The service would compile, but the conflict would cause a problem
immediately."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You must define the BinaryMessageEncodingBindingElement.",
                        IsCorrect = true,
                        Explanation = @"An InvalidOperationException would be thrown when the application
started."
                    }
                });

            SC("3.3 (2)", 
               @"The ServiceOperation attribute is defined on both the service implementation and the interface definition. Each of the property settings is explicitly defined, and each property matches its counterpart in the other definition. Which of the following is true?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"As long as the property values are identical or don’t conflict with each other, the service will behave as desired.",
                        IsCorrect = false,
                        Explanation = @"Even if they’re defined in an identical manner, there would be a conflict
and resulting exception."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The items defined on the interface will override the ones on the implementation class.",
                        IsCorrect = false,
                        Explanation = @"There would be a conflict; they can’t co-exist without a problem."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The items defined in the implementation class will override the ones on the interface definition.",
                        IsCorrect = false,
                        Explanation = @"There would be a conflict; they can’t co-exist without a problem."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The service will throw an InvalidOperationException because this attribute cannot be defined on both items.",
                        IsCorrect = true,
                        Explanation = @"The service would definitely throw an InvalidOperationException, as
indicated."
                    }
                });

            MC("3.3 (3)", 
               @"Which of the following are valid transport binding elements when defining a custom binding?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"TcpTransportBindingElement",
                        IsCorrect = true,
                        Explanation = @"A TcpTransportBindingElement can be used with a CustomBinding."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"TcpBinding",
                        IsCorrect = false,
                        Explanation = @"A TcpBinding can be used, but it is not a binding element; it is an actual
binding."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HttpTransportBindingElement",
                        IsCorrect = true,
                        Explanation = @"A HttpTransportBindingElement can be used with a CustomBinding."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"MTOMTransportBindingElement",
                        IsCorrect = false,
                        Explanation = @"While MTOM can be used, it is not a specific BindingElement that can
be used with a CustomBinding."
                    }
                });

            MT("3.3",
               @"Configuration versus using the API",
               @"You are building a WCF Service, and you have all the requirements for the service’s
foreseeable future clearly defined. You have all your options open with respect to
implementation.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"Should you lean toward a configuration-based approach, try to define everything
imperatively, or use a mixed approach?",
                       Explanation = @"There are good reasons to use every one of the mentioned approaches. The more
well-defined the application requirements, the less risk there is to using a completely
imperative approach. The less clear the requirements, the more likely you are to benefit
from using configuration. Using a hybrid approach can enable you to have the best
of both worlds, and you don’t have to make a definitive choice one way or the other.
If you’re hosting the service internally and have full access to it, configuration carries
little cost or risk. In some Windows Azure-based hosting scenarios, you have to use a
more imperative approach, although you can still use a hybrid approach here."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What downside would you experience if you defined everything imperatively?",
                       Explanation = @""
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What downside would you experience if you defined everything in
configuration?",
                       Explanation = @""
                   }
               });


            // Section 3.4
            CurrentBackground = @"Secure a WCF service";

            MC("3.4 (1)", 
               @"You are using IIS to host a WCF Service. Which of the following would help implement transport security over http (https) without removing previously existing functionality?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The new Https protocol mapping feature",
                        IsCorrect = true,
                        Explanation = @"Using protocol mapping will help in this situation."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use of a wsHttpsBinding instead of a wsHttpBinding",
                        IsCorrect = true,
                        Explanation = @"Using a secure binding will help in this situation."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Enabling https for the virtual directory that contained the WCF Service items",
                        IsCorrect = true,
                        Explanation = @"Enabling https on the virtual directory will help in this situation."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Ensuring that a metadata is enabled for the service",
                        IsCorrect = false,
                        Explanation = @"Enabling metadata will do nothing to help facilitate transport security."
                    }
                });

            MC("3.4 (2)", 
               @"Which of the following are true regarding WCF security?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You can implement message level security",
                        IsCorrect = true,
                        Explanation = @"You can implement message level security in WCF."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You can implement transport level security",
                        IsCorrect = true,
                        Explanation = @"Transport level security can be implemented."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You can implement both transport level and message level security",
                        IsCorrect = true,
                        Explanation = @"Message and transport security can be implemented together."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Transport security can be enabled, and message security can be enabled, but they cannot both be enabled at the same time",
                        IsCorrect = false,
                        Explanation = @"Both can be implemented together and often are; they are not in conflict
with each other."
                    }
                });

            MC("3.4 (3)", 
               @"You are developing a WCF Service. The service must implement transport security, NTLM authentication, and NetTcpBindings. Which of the following do you need?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"binding value of netTcpBinding",
                        IsCorrect = true,
                        Explanation = @"A netTcpBinding must be defined."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"clientCredentialType of netTcpBinding",
                        IsCorrect = true,
                        Explanation = @"A clientCredentialType of netTcpBinding must be defined."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"clientCredentialType of Transport",
                        IsCorrect = false,
                        Explanation = @"A clientCredentialType of Transport is not needed."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"security mode value of Transport",
                        IsCorrect = true,
                        Explanation = @"A SecurityMode value of Transport must be specified."
                    }
                });

            MT("3.4",
               @"How much should you secure?",
               @"You are building several WCF Services, and some will employ sensitive information.
Others transfer information that is completely public and are intended to be as easy
to use as possible. Your service will be hosted internally, so you have full access to it
without any real restrictions.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"For the secure information, would you choose message level security, transport
level security, or both?",
                       Explanation = @"Message and transport security aren’t mutually exclusive; they are complementary, in
fact. There is additional overhead with more security implemented, but that overhead
can be worked around. Getting it wrong can destroy your company and cause many
problems for your clients, and estimating the full scope of damage is difficult to do.
Generally speaking, people are much more tolerant of a slight performance loss (especially
if they understand why) than they are of a data breach."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"For the less-sensitive information, would you still want to implement any
security?",
                       Explanation = @"You can go overboard with anything, but there is little inconvenience to implementing
either approach. Moreover, even if something is intended to be public, you still would
have an incentive to keep some level of control over it. Many people are inherently
concerned about any insecure communication. You have complete control here, so
there’s quite a bit of flexibility in terms of what you want to identify."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Would certificate-based security be a good fit for either scenario?",
                       Explanation = @"Certificate-based security has many upsides and it’s desirable in many circumstances,
but it does add a level of implementation complexity that many find inconvenient and
cumbersome. If it’s not necessary, it’s probably more hassle than benefit."
                   }
               });


            // Section 3.5
            CurrentBackground = @"Consume WCF services";

            ET("3.5", 
               @"Generating proxies by creating a service reference", 
               @"For test questions, the only things that you might be asked about on this portion are whether to use the Go feature coupled with a URI, the Discover option, or about specifying a specific namespace.");

            MCC("3.5 (1)", 
               @"You need to build a client-side proxy class in VB.NET for a WCF Service named TestService that is hosted by one of your vendors. Which of the following options would accomplish that task?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the Svcutil.exe command-line tool with the following command:
Svcutil.exe http://www.myvendor.com/TestService.svc",
                        IsCorrect = false,
                        Explanation = @"Although the proxy would be successfully generated, absent a specific
switch for Visual Basic, C# code would be generated."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the Svcutil.exe command-line tool with the following command:
Svcutil.exe /l:visualbasic http://www.myvendor.com/TestService.svc.",
                        IsCorrect = true,
                        Explanation = @"Using Svcutil with the l switch and indicating a valid language works."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the Add Service Reference dialog box.",
                        IsCorrect = true,
                        Explanation = @"Using the Add Service Reference dialog box works because it’s designed
to handle this use case in particular."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the ChannelFactory class.",
                        IsCorrect = true,
                        Explanation = @"ChannelFactory works in this case."
                    }
                });

            MC("3.5 (2)", 
               @"You are building a client to consume a WCF Service named TestService, and many of the methods have return types that are generic list collections. You are expecting List<Type> return types for the calls, but the proxy class has implemented each as an array. You want this type implemented specifically as the List<Type>. What should you do?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the ToList<> extension method at the end of each call to the methods that are currently being returned as arrays.",
                        IsCorrect = false,
                        Explanation = @"Calling ToList works, but it’s an extra step and doesn’t directly address
the problem."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Change the Settings in the Add Service Reference dialog box using the Advanced screen. Change the Collection type to List.",
                        IsCorrect = true,
                        Explanation = @"This is the feature designed specifically to handle this situation."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Regenerate the proxy class after changing the settings.",
                        IsCorrect = false,
                        Explanation = @"Regenerating the proxy alone does nothing."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the Add Web Reference dialog box to generate the proxy because it automatically respects the return type definitions.",
                        IsCorrect = false,
                        Explanation = @"The default behavior of the Add Web Reference dialog box is used to
add an older form of the service; it does not handle the conversions correctly."
                    }
                });

            MC("3.5 (3)", 
               @"Svcutil.exe provides which of the following features?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Validates compiled service code.",
                        IsCorrect = true,
                        Explanation = @"Svcutil.exe can be used to validate compiled service code."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Downloads metadata documents from deployed services.",
                        IsCorrect = true,
                        Explanation = @"Downloading metadata documents from deployed services is one of the
features of Svcutil.exe."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Generates serialization code.",
                        IsCorrect = true,
                        Explanation = @"Generating serialization code is one of the features of SvcUtil.exe."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Adds metadata exchange endpoints to deployed services.",
                        IsCorrect = false,
                        Explanation = @"Svcutil.exe does not add metadata exchange to running services."
                    }
                });

            MT("3.5",
               @"Consuming WCF Services",
               @"You are building several WCF Services. You have several different clients on several
different operating system versions.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"Do you recommend the Add Service Reference feature or Svcutil.exe?",
                       Explanation = @"Both can get you to the same place, but the Add Service Reference dialog box has a
few restrictions. First, it necessitates having Visual Studio, which won’t be a problem
for clients using .NET, but might be a problem for others. Similarly, the Add Service
Reference dialog box walks you through the nuances of creating a proxy class, but its
strength is its weakness. Because it is visual, it is a manual process and cannot be easily
called in a build script or through process automation."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Do you recommend that the client proxies support generics or arrays?",
                       Explanation = @"Generics are powerful and make development a lot more attractive. However, only
.NET languages support generics. Using arrays maximizes interoperability. Moreover,
converting an array to a generic is simple to do, particularly if you’re using a .NET
language."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What other considerations might you find?",
                       Explanation = @"Older versions of the .NET Framework did not support generics. LINQ was introduced
in version 3.0 of the Framework as well. You have the power to use Svcutil.exe to generate
several different languages, which is beneficial. Additionally, if you need to work
with a WCF Service as a traditional web service, you can easily have that done."
                   }
               });


            // Section 3.6
            CurrentBackground = @"Version a WCF service";

            MC("3.6 (1)", 
               @"Version 10 of the TestService (which implements the ITestService interface) is about to be updated and deployed. The ITestService interface adds two new methods, both of which are scoped with the public modifier and are decorated with the OperationContract attribute. Which of the following is true?",
                  new CheckboxAnswer[3]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The two additions will be considered breaking changes unless each assembly includes the correct version number.",
                        IsCorrect = true,
                        Explanation = @"Contract changes are a breaking change."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The two additions will not be breaking changes because they leave all previous functionality in place and just add new functionality.",
                        IsCorrect = false,
                        Explanation = @"Additions don’t remove functionality, but are still a breaking change."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"As long as the service and contract pairs are both deployed with their respective version numbers (versions 10, 11), clients will not encounter any problems.",
                        IsCorrect = true,
                        Explanation = @"As long as the old service with its old definitions is still in place, the addition
of a new service will not cause a problem."
                    },
                });

            //SC("3.6 (2)", **bug** **todo** Find answers
            //   @"Version 10 of the TestService assembly (which also includes the ITestService interface and implements it) adds two new methods to the interface definition. The new methods use the Protected modifier and do not currently contain the OperationContract attribute (on either the interface or the implementation). Which of the following is true?",
            //      new CheckboxAnswer[4]
            //      {
            //        new CheckboxAnswer()
            //        { 
            //            AnswerText = @"There will not be breaking changes on any items that statically link to the assembly.",
            //            IsCorrect = false,
            //            Explanation = @""
            //        },
            //        new CheckboxAnswer()
            //        { 
            //            AnswerText = @"There will not be breaking changes because the access modifiers are set to Protected.",
            //            IsCorrect = false,
            //            Explanation = @""
            //        },
            //        new CheckboxAnswer()
            //        { 
            //            AnswerText = @"There will not break changes because they don’t implement the OperationContract attribute.",
            //            IsCorrect = false,
            //            Explanation = @""
            //        },
            //        new CheckboxAnswer()
            //        { 
            //            AnswerText = @"It will be a breaking change to any assembly statically linked to it unless the new methods are also added to the implementation.",
            //            IsCorrect = false,
            //            Explanation = @""
            //        }
            //    });

            MC("3.6 (3)", 
               @"Which versioning strategies are supported through WCF?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Strict Versioning",
                        IsCorrect = true,
                        Explanation = @"Strict versioning can be implemented with WCF."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Lax Versioning",
                        IsCorrect = true,
                        Explanation = @"Lax versioning can be implemented with WCF."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Mixed-Mode Versioning",
                        IsCorrect = false,
                        Explanation = @"Mixed-mode versioning would defeat the purpose or either approach."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Contract Versioning",
                        IsCorrect = true,
                        Explanation = @"Contract versioning is supported and necessary in most cases."
                    }
                });

            MT("3.6",
               @"How should you handle version upgrades?",
               @"You are building several WCF Services. You have to get the service out immediately, but there’s strong reason to believe you’re going to have to quickly implement many new features and additional versions.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"How can you avoid breaking changes?",
                       Explanation = @"Breaking changes are a part of life. There’s often no way to avoid them, particularly
when you have to deploy something immediately that you know will change frequently.
Breaking changes are inconvenient, and many people refuse to move to new
products because of them. Planning and thinking about future features helps reduce
the probability that you’ll run into them, but it’s no guarantee that you won’t have to
implement them. In some cases, it might be advisable to try to consolidate breaking
changes in another service and have the existing service make calls to it. The more you
can consolidate and obscure from clients, the less of a problem they’ll be. You are always
in a position to deal with internal breaking changes more easily than your clients
will be."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"If you have breaking changes that you are forced to implement, how would you handle them?",
                       Explanation = @"Communication is the only way to mitigate problems associated with breaking
changes. An extensive testing plan is needed to avoid breaking changes and the problems
associated with them. Breaking changes are bad enough, but coupled with bugs,
you’ll have angry customers in all likelihood. There are several articles that deal with
the issue, and becoming familiar with the process is absolutely critical. Telling clients
of the breaking changes with as much advance notice as possible is one mitigation
strategy. Making sure that everyone knows about them so no one is surprised will also
help. Precise details about what the changes will be; what will be affected; and, ideally,
suggestions on how best to deal the changes will also help. If you have to deal with a
breaking change, it’s much easier to do with a clear definition of what’s changing and a
six-month planning window. Flip it around and imagine that you find out the changes
have already been made and your users are already dealing with problems. Imagine
that you get an early notice, but you get several updates that change the nature of
what you’ve already changed. In the final analysis, the fewer interruptions, the more
predictability, and the more certainty clients have, the easier it will be for them to deal
with the changes."
                   },
               });


            // Section 3.7
            CurrentBackground = @"Create and configure a WCF service on Windows Azure";

            MC("3.7 (1)", 
               @"Which elements do you need from the management portal when configuring your
host service?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Namespace",
                        IsCorrect = true,
                        Explanation = @"The namespace is required to access your service. You specify the
namespace when creating your bindings and addresses."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"NetTcpRelayBinding",
                        IsCorrect = false,
                        Explanation = @"The binding is specified at the host side for connecting to Azure Service
Bus. This is not something that you configure in the Management Portal."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"ServiceHost",
                        IsCorrect = false,
                        Explanation = @"ServiceHost is required to host your WCF service. It’s not something
that you configure in the Management Portal."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Default Key",
                        IsCorrect = true,
                        Explanation = @"The Default Key is used for authentication both your server and client applications
to the service bus."
                    }
                });

            SCC("3.7 (2)", 
               @"You want to use a SharedSecretToken for authentication. Which method and parameters do you use?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"TokenProvider.CreateSharedSecretTokenProvider(namespace, key)",
                        IsCorrect = false,
                        Explanation = @"You do need the CreateSharedSecretTokenProvider method. However,
you need to pass it the owner and key. The namespace is used in the service URI."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"TokenProvider.CreateSamlTokenProvider(samltoken)",
                        IsCorrect = false,
                        Explanation = @"This method creates a SAML token."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"TokenProvider.CreateSharedSecretTokenProvider(owner, key)",
                        IsCorrect = true,
                        Explanation = @"CreateSharedSecretTokenProvider with the owner and key creates the
correct token."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"TokenProvider.CreateWindowsTokenProvider(uris)",
                        IsCorrect = false,
                        Explanation = @"This creates a Windows token."
                    }
                });

            MC("3.7 (3)", 
               @"You want to connect a client to the service bus. Which elements do you need?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Namespace",
                        IsCorrect = true,
                        Explanation = @"The namespace is required to access your service. You specify the
namespace."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Service path",
                        IsCorrect = true,
                        Explanation = @"The service path is required as a part of your endpoint address."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Owner name",
                        IsCorrect = true,
                        Explanation = @"The owner name is required when creating your SharedSecretToken."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Default Key",
                        IsCorrect = true,
                        Explanation = @"The Default Key is also used for authentication and is required to create
your SharedSecretToken."
                    }
                });

            MT("3.7",
               @"Talking with your customer",
               @"You are consulting for a customer that is thinking about how to expose his private
and confidential data to employees who work remotely. The customer is currently
hosting his data in his own datacenter. Currently, employees can synchronize a copy
of their data only when they are at the office.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"Would you recommend the customer use a cloud or for a hybrid solution? Why?",
                       Explanation = @"Because the customer is dealing with private and confidential data, a cloud-only solution
will probably not appeal to them. By offering a hybrid solution, he can control the
data while having all the benefits of a secure public interface hosted in Azure."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"How can the relaying capabilities of the service bus help your customer?",
                       Explanation = @"The service bus can offer a relaying endpoint to client applications that run on the
devices of the employees The relay then securely forwards messages to the on-premise
datacenter. This creates a secured way of exposing data."
                   },
               });


            // Section 3.8
            CurrentBackground = @"Implement messaging patterns";

            MCC("3.8 (1)",
               @"You have the following method signature. 

Boolean LogMessage(String messsage);

You are told that it should execute as quickly as possible and advised it would be a good candidate for a OneWay method. Which of the following are true:",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"This method is not a valid candidate to execute with the OneWay pattern because it has a return type.",
                        IsCorrect = true,
                        Explanation = @"Having a return type negates the use of the OneWay attribute."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"This method is not a valid candidate to execute with the OneWay pattern because it accepts a parameter.",
                        IsCorrect = false,
                        Explanation = @"OneWay can be used in conjunction with an input parameter."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Simple logging operations that just accept a String parameter and return a Success or Failure message are ideally suited for the OneWay pattern. However, to work a Callback contract would be needed to work properly.",
                        IsCorrect = false,
                        Explanation = @"Simple operations might be good candidates, but not if they return a
message."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"There’s not enough information to know for sure whether this method would be a good candidate to use to implement the OneWay pattern.",
                        IsCorrect = false,
                        Explanation = @"The method has a return type that automatically disqualifies it."
                    }
                });

            MC("3.8 (2)", 
               @"Your service needs to let clients open a channel to it and allow back and forth communication throughout the session. Which is true?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Any binding that supports streaming would be a work here.",
                        IsCorrect = true,
                        Explanation = @"Any binding that enables clients to open a channel with it and have backand-
forth communication would work here."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Only bindings with the IsOneWay property set to true would be valid.",
                        IsCorrect = false,
                        Explanation = @"OneWay goes completely against the desired goal here."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"As long as the service can stream back to the client, this would work.",
                        IsCorrect = true,
                        Explanation = @"As long as the service can call back to the client, it should work."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Duplex communication cannot be implemented with Http because it’s a stateful protocol.",
                        IsCorrect = false,
                        Explanation = @"Http is a stateless protocol, so it can’t be used."
                    }
                });

            SC("3.8 (3)", 
               @"Your service needs to support real-time communication between client and service. Which is true?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Streaming should be enabled.",
                        IsCorrect = false,
                        Explanation = @"Streaming alone would not solve any of the stated problems."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"A Binding that supports duplexing must be implemented.",
                        IsCorrect = true,
                        Explanation = @"Duplexing must be implemented, and as such, a binding that supports
duplexing must be enabled."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"A stateless protocol can be supported but the client must implement threading.",
                        IsCorrect = false,
                        Explanation = @"A stateless protocol by definition would be inadequate here, irrespective
of multi-threaded clients."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Duplexing and streaming must be implemented together.",
                        IsCorrect = false,
                        Explanation = @"Duplexing alone suffices; there is no need for streaming."
                    }
                });

            MT("3.8",
               @"Do message patterns matter?",
               @"You are building several WCF Services. You have several calls that are for logging
purposes only, and the application is known to be chatty. Responsiveness is critical,
and you need real-time responses for much of the application.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"Do logging calls warrant the use of a different messaging pattern than other
calls?",
                       Explanation = @"Logging calls are great candidates for OneWay implementation. The less important the
data is, the better candidate it is for a OneWay call. You lose the ability to have return
types on OneWay calls, but for logging operations, particularly ones that fire frequently,
if a few don’t make it to the server, it will likely have little impact. On the other
hand, the time it takes just to deal with a response type can exacerbate the problems
associated with chatty applications."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Why wouldn’t you just make everything employ duplex communication
patterns?",
                       Explanation = @"Duplex communication is useful and, in some cases, it is the only option you have.
However, it is neither free nor cheap. Although WCF makes duplex communication appear
simple, quite a bit has to happen to make it work right. This happens largely out
of the user’s view, but it still happens. Implementing duplexing when it’s not needed
is the logical inverse of using a OneWay call. It adds overhead with no benefit and can
greatly hinder scalability. Although it’s powerful and useful, it should be used judiciously
and only when necessary."
                   },
               });

            // Section 3.9
            CurrentBackground = @"WCF Services - Host and manage service";

            SC("3.9 (1)", 
               @"You are developing a new WCF Service. Each time a client initiates a request, you need
to create a service instance to support it. What should you do?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Set the InstanceContextMode to Singleton.",
                        IsCorrect = false,
                        Explanation = @"There’s no Singleton enumeration value but even if there was, it would
make no sense here."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Set the ConcurrencyMode to PerCall.",
                        IsCorrect = false,
                        Explanation = @"ConcurrencyMode doesn’t have a PerCall value."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Set the InstanceContextMode to PerCall.",
                        IsCorrect = true,
                        Explanation = @"InstanceContextMode needs to be set to PerCall to work."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Set the ConcurrencyMode’s scope to User and set the InstanceContextMode to PerCall.",
                        IsCorrect = false,
                        Explanation = @"There is no way to set the ConcurrencyMode’s scope to User."
                    }
                });

            MC("3.9 (2)", 
               @"Which of the following would be appropriate as a WCF Service Host?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Hosted workflow service",
                        IsCorrect = true,
                        Explanation = @"A hosted workflow service has an appDomain and an entry point, so it would work."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @".NET console application",
                        IsCorrect = true,
                        Explanation = @"A console application has an entry point and would work fine in this scenario."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Windows Forms application built using Windows Presentation Foundation",
                        IsCorrect = true,
                        Explanation = @"A Managed Windows Service has an entry point and is an ideal candidate as a host."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"IIS",
                        IsCorrect = true,
                        Explanation = @"IIS is well suited to WCF hosting."
                    }
                });

            MC("3.9 (3)", 
               @"You need to develop a WCF Service. The service is supposed to be both self-hosted
and discoverable by any client application. Which of the following are needed to support
this scenario?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"A DiscoveryEndpoint",
                        IsCorrect = true,
                        Explanation = @"A DiscoveryEndpoint would be necessary to meet the discovery
requirement."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"A ServiceHost definition",
                        IsCorrect = true,
                        Explanation = @"Without a servicehost definition, it would be impossible to create this application
as a self-hosted service."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"A ServiceBehavior definition",
                        IsCorrect = false,
                        Explanation = @"A ServiceBehavior definition might be beneficial, but it’s not required as
the item is stated."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"A UpdDiscoveryEndpoint",
                        IsCorrect = false,
                        Explanation = @"While a DiscoveryEndpoint is necessary to meet the application requirements,
there is no need for a UdpDiscoveryEndpoint specifically."
                    }
                });

            MT("3.9",
               @"Where should I host my application?",
               @"You are building several WCF Services. You know that you need to support Http,
TCP, and MSMQ. You expect a large number of clients and heavy loads on your
services. You might need to move the service host from machine to machine on
occasion. Additionally, all your developers are familiar with ASP.NET programming,
but are completely new to Windows Azure and WAS.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"Is a console application or a WPF application implemented via self-hosting a
viable approach?",
                       Explanation = @"Self-hosting with a console or WPF application would most likely be the worst of the
possible approaches. Self-hosted services are easy to move from machine to machine,
and you can support each of the required bindings. However, heavy loads make it
a bad choice. The fact that most self-hosted services have to run under a dedicated
account makes it a bad choice. Someone could accidentally click the mouse over the
close button or erroneously hit an enter button and possibly shut down the entire host."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Is IIS a good candidate here, assuming that there are several instances available
internally?",
                       Explanation = @"IIS would be a great choice except for one major issue: It simply doesn’t support any of
the bindings other than HTTP, so it clearly wouldn’t work here. The fact that developers
are familiar with it and that it scales well is irrelevant because of the bindings issue."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What factors shift the equation to one host choice versus another?",
                       Explanation = @"Windows Azure might be a decent approach, although the fact that the developers
are unfamiliar with it would necessitate them gaining that expertise quickly. That’s not
a huge hurdle, but it is a hurdle nonetheless. WAS supports all the bindings needed.
The services can move from one WAS instance to another with relative ease; in fact,
the move could even be scripted. WAS on Windows Azure could meet all the criteria,
but internally hosted WAS would seem in this case to be the most appropriate hosting
choice."
                   }
               });



            ///////////////////////// Chapter 4 - Creating and consuming Web API-based services ///////////////////////



            // Section 4.1
            CurrentBackground = @"Design a Web API";

            ET("4.1", 
               @"Creating the controller", 
               @"If the query parameters don’t return a matching value, you have several ways to respond, just
as you would any other query. Although you can opt to return a null value, a more elegant
and expressive means is to throw an HttpResponseException. The HttpResponseException
class has two overloads you can use: The first is to specify an HttpStatusCode enumeration
value, and the second is an HttpResponseMessage. The HttpStatusCode enumeration provides
several common and intuitive codes, and in this case, the value of NotFound fits both
logically and mechanically. Make sure that you understand how to use HttpStatusCode for
the exam to return meaningful responses to the client.");

            ET("4.1", 
               @"Creating the controller", 
               @"Experiment with calling your Web API by accessing all action methods through their URL.
Try passing different values for parameters such as accountId. Also pass an invalid value so
you get back an HTTP 404 error (the result of HttpResponseException’s HttpStatusCode).");

            ET("4.1", 
               @"Creating the controller", 
               @"There are HTTP actions defined in two obvious namespaces: System.Web.Http and System.
Web.Mvc. They both have their use and place in the application, but in this sample, the one in
the System.Web.Http namespace is the one that should be used. Because there are some other
classes in the System.Web.Mvc namespace, it is incredibly easy to accidentally use the incorrect
one, so be careful here!

Equally as confusing is the AcceptVerbsAttribute, which also exists in both places. Yet it is
important to use the one in the System.Web.Http namespace. So how do you know when
to use one versus the other? It’s quite simple! Look at your controller and see what the
base class is. If it’s ApiController, you should use the System.Web.Http namespace. If it’s
not, use the System.Web.Mvc namespace for them. It’s certainly possible for both to exist
within the same project!");

            MC("4.1 (1)", 
               @"You have been asked to create an ASP.NET Web API application. The current site enables users to post comments on newly posted articles. There’s been an increasing problem of inappropriate and spam comments. Editors need to be able to edit the inappropriate content after reviewing it and finding it in violation with the terms of service. Editors also need to be able to permanently delete any comments that are considered spam. Which operations accomplish these goals?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HttpGet",
                        IsCorrect = true,
                        Explanation = @"HttpGet requests are necessary to view the data to moderate it."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HttpPost",
                        IsCorrect = false,
                        Explanation = @"HttpPost is used to add new items. Editors don’t need to add new items
(but users do)."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HttpDelete",
                        IsCorrect = true,
                        Explanation = @"Deletions should be done through the HttpDelete operation."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HttpPut",
                        IsCorrect = true,
                        Explanation = @"HttpPut can be used for modifying posts."
                    }
                });

            SC("4.1 (2)", 
               @"You have an ASP.NET MVC Web API that processes new customer inquiries. The client applications are intended to be very focused in scope and centered on just a few operations. A decision was just made that, for now, the API needs to simply allow data collected from a Web Form to be submitted to a SQL Server database. Which of the following is needed to allow form data to be submitted to the server?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HttpGet",
                        IsCorrect = false,
                        Explanation = @"HttpGet should be used only to retrieve data, not to insert it."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HttpHead",
                        IsCorrect = false,
                        Explanation = @"HttpHead can be used to retrieve information about the request, but
not to insert records."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HttpPost",
                        IsCorrect = true,
                        Explanation = @"To insert a new record, the HttpPost operation should be called."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HttpPut",
                        IsCorrect = false,
                        Explanation = @"The HttpPut operation should be used for modifying data. HttpPost is a
better candidate for inserting data."
                    }
                });

            SC("4.1 (3)", 
               @"Your ASP.NET Web API application needs to be able to allow action methods to operate through multiple HTTP Actions (namely HttpPost and HttpPut). How should this be accomplished?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Decorate the method with both the HttpGet and the HttpPost attribute.",
                        IsCorrect = false,
                        Explanation = @"HttpPost and HttpPut need to be implemented here; an HttpGet does
not solve the core requirement. You also need to use the AcceptVerbs attribute to
allow multiple verbs to be used."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Decorate the method with an HttpGet and an HttpPut attribute.",
                        IsCorrect = false,
                        Explanation = @"HttpPost and HttpPut need to be implemented here; an HttpGet does
not solve the core requirement. You also need to use the AcceptVerbs attribute to
allow multiple verbs to be used."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Decorate the method with the ActionNames attribute using the values of HttpPut and HttpPost.",
                        IsCorrect = false,
                        Explanation = @"The operations are correct but need to be implemented with the
AcceptVerbs attribute."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Decorate the method with the AcceptVerbs attributes, including HttpPut and HttpPost.",
                        IsCorrect = true,
                        Explanation = @"The AcceptVerbs attribute should be implemented as indicated here, and
both the HttpPut and HttpPost operation should be supported."
                    }
                });

            MT("4.1",
               @"Understanding Web API",
               @"You are explaining Web API to your colleagues. You think it’s a nice and elegant solution,
but they don’t see the advantages of Web API over Windows Communication
Foundation (WCF).",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"Why should we depend on HTTP verbs instead of calling a method by name?",
                       Explanation = @"By using the HTTP verbs, you use HTTP as the foundation of your service. The HTTP
verbs are supported by browser and by the HttpClient and by all other platforms.
This way, you can use a single URL and use the verb to specify what action you want
to take. This decouples the client code from implementation details such as method
names."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What’s the advantage of content negotiation? Isn’t the service the one that
should declare the return type?",
                       Explanation = @"By using content negotiation, you don’t couple your service to a specific technology.
Instead, you support multiple clients, all with their own requirements. The clients can
then choose the format to work with."
                   },
               });


            // Section 4.2
            CurrentBackground = @"Implement a Web API";

            MC("4.2 (1)", 
               @"Which of the following media types are valid default options for an Accept request
header?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"application/json",
                        IsCorrect = true,
                        Explanation = @"application/json is a supported format type."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"application/xml",
                        IsCorrect = true,
                        Explanation = @"application/xml is a supported format type."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"gzip",
                        IsCorrect = false,
                        Explanation = @"gzip could be supported, but would require special implementation."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"en-us",
                        IsCorrect = false,
                        Explanation = @"en-us is a culture and region formatter, not a media type."
                    }
                });

            MC("4.2 (2)", 
               @"Which of the following request headers can be sent to help perform content negotiation?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Accept",
                        IsCorrect = true,
                        Explanation = @"Accept is a valid request header type."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Accept-Charset",
                        IsCorrect = true,
                        Explanation = @"Accept-Charset is a valid header value."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Accept-Encoding",
                        IsCorrect = true,
                        Explanation = @"Accept-Encoding is a valid encoding."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Accept-Language",
                        IsCorrect = true,
                        Explanation = @"The Accept-Language corresponds to the natural language and is
supported."
                    }
                });

            MC("4.2 (3)", 
               @"You want to create an attribute that executes when an exception happens anywhere in
your application. Which steps do you take?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Inherit from ActionFilterAttribute.",
                        IsCorrect = false,
                        Explanation = @"For exception handling, you shouldn’t inherit from ActionFilterAttribute.
Those filters execute on each request, independently of an exception happening."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Inherit from ExceptionFilterAttribute.",
                        IsCorrect = true,
                        Explanation = @"ExceptionFilterAttribute is the correct base class for filters that should
execute whenever an exception happens."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add the custom attribute to your controllers.",
                        IsCorrect = false,
                        Explanation = @"Adding the attribute to all your controllers is error prone. Instead, you
can apply the attribute globally."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add the custom attribute to HttpConfiguration.Filters.",
                        IsCorrect = true,
                        Explanation = @"By adding the attribute to the global collection of filters, you apply it to
all controllers and action methods in one step."
                    }
                });

            MTC("4.2",
               @"Implementing a Web API",
               @"You are the architect and lead developer on a new project. You are going to build
the back end for a new app. The app will be used by health care services to track
patient records. Depending on the access privileges of the users, they can view and
edit patient status, such as notes and images like x-rays or other scans. To support
the bring-your-own-device policy, the app will be distributed to a multitude of
devices running on Windows, Windows Phone, iOS, and Android, and on the web.
You already have two hospitals with thousands of staff members who are going to
beta test your app.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"Why is Web API well-suited for this scenario?",
                       Explanation = @"All devices are capable of executing web requests. By using Web API, you open your
service to all devices that you want to support."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Are you planning to use any of the following techniques?
 - Content negotiation
 - HttpMessageHandler
 - Dependency injection
 - Action filters
 - Streaming",
                       Explanation = @"The following explains what you should use:
 - Through content-negotiation, all clients can work with data in a format that’s suitable
for the device. It’s one of the foundational features of Web API.
 - You can use an HttpMessageHandler if you have any cross-cutting concerns that
should apply to all requests or to specific routes. At this moment, you have no use
for handlers, but that could change.
 - Dependency injection is an important part of each modern architecture. You can
use it to create a loosely coupled system that can easily be unit tested.
 - You should use the AuthorizeAttribute, which is an implementation of an action filter.
You can also create your own attributes if you want to run code before or after
an action runs.
 - Streaming is important because you are supporting thousands of users, and you
need to transmit large amounts of data for the images."
                   },
               });


            // Section 4.3
            CurrentBackground = @"Secure a Web API";

            ET("4.3", 
               @"Preventing cross-site request forgery", 
               @"If you’re unfamiliar with XSRF, https://www.owasp.org/index.php/Cross-Site_Request_Forgery provides a wonderful and very thorough description of it. A summary description of XSRF is simply when a different website or application (even email links) attempts to access a resource that requires authentication to trigger some action that has some side effect.");

            ET("4.3", 
               @"How to protect from XSRF", 
               @"Of course, it’s not required to memorize all the code involved in checking the antiforgery tokens. You do need to make sure that you understand the ideas behind it. Sending two values, one in a cookie and one in the form data, and making sure these values match is the basic idea for protecting against XSRF. A good practice is to create a simple Web API that is vulnerable to XSRF and then protecting it so you understand all steps involved. An example can be found at http://www.dotnetcurry.com/ShowArticle.aspx?ID=890.");

            SC("4.3 (1)", 
               @"You want to enable HTTPS for your Web API, and you want to make sure that none of
the developers forgets about this requirement. How do you do this?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a custom ApiController base class from which all developers need to inherit.",
                        IsCorrect = false,
                        Explanation = @"Developers can forget to inherit from the custom controller. This is no
safe way to implement the requirement."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create an HttpsRequiredAttribute and add it to each controller.",
                        IsCorrect = false,
                        Explanation = @"If a developer forgets to add the attribute to a controller, the controller
will be accessible through HTTP."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create an HttpsRequiredAttribute and add it to the global collection of attributes.",
                        IsCorrect = true,
                        Explanation = @"If a developer forgets to add the attribute to a controller, the controller
will be accessible through HTTP."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use MakeCert.exe to create an HTTPS certificate.",
                        IsCorrect = false,
                        Explanation = @"MakeCert is used to create a test certificate that you can use on your
development machine. It doesn’t force your application to HTTPS."
                    }
                });

            SC("4.3 (2)", 
               @"You are building a Web API to be used by third-party websites, and you want to reach as many users as possible. You want to protect your service from XSRF attacks. Which requirement do you state for your third parties?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Force them to use ASP.NET MVC so you can easily authenticate requests.",
                        IsCorrect = false,
                        Explanation = @"Forcing your customers to use ASP.NET MVC to develop their websites
doesn’t increase your user base. This is not something you can state as a requirement
in the real world."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Require them to include both a cookie and a form value with a unique token.",
                        IsCorrect = true,
                        Explanation = @"This is the basic idea behind the antiforgery token solution that ASP.NET
MVC offers. By comparing the two values, you can validate the request origin."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the ValidateAntiForgeryToken attribute on your Web API service.",
                        IsCorrect = false,
                        Explanation = @"ValidateAntiForgeryToken can be used only with ASP.NET MVC."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use Windows Authentication.",
                        IsCorrect = false,
                        Explanation = @"Windows Authentication isn’t possible with a consumer-facing
application."
                    }
                });

            SC("4.3 (3)", 
               @"You want to extend the default authentication mechanism of Web API. You want to have an attribute that you can place on a couple of action methods to log nonauthenticated requests. What do you do?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Inherit from IAuthorizationFilter to implement the custom logic.",
                        IsCorrect = false,
                        Explanation = @"IAuthorizationFilter is a very low-level interface. You should inherit from
this interface only if you have any asynchronous work that doesn’t involve the current
user or role."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Inherit from AuthorizeAttribute to implement the custom logic.",
                        IsCorrect = true,
                        Explanation = @"By inheriting from this attribute, you get a fully implemented authorization
attribute that you can easily extend."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Inherit from AuthorizeFilterAttribute to implement the custom logic.",
                        IsCorrect = false,
                        Explanation = @"You should extend this class only to perform synchronous authorization
logic that is not necessarily based on the current user or role."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create an HttpMessageHandler that you attach to specific routes.",
                        IsCorrect = false,
                        Explanation = @"It’s easier to inherit from the AuthorizeAttribute. By directly adding the
attribute to specific methods, you can control when to run the code."
                    }
                });

            MT("4.3",
               @"Securing your Web API",
               @"You are working on a Web API that will store stats for a new game that you will
release as an app for various platforms and as a web application. Users first need to
register before they can play games and share their achievements with other users.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"Can you use Windows Authentication?",
                       Explanation = @"No. Windows Authentication is not suited for consumer-facing applications. It should
be used for intranet applications."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Do you need to protect yourself against XSRF?",
                       Explanation = @"Yes. Because you are storing user credentials in a cookie for the web application, your
application is vulnerable for XSRF attacks."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Is CORS important for your back end?",
                       Explanation = @"Because your back end won’t be called by other websites through AJAX, CORS is not
important. Only when you have AJAX calls coming from other origins should you enable
CORS."
                   }
               });


            // Section 4.4
            CurrentBackground = @"Host and manage a Web API";

            ET("4.4", 
               @"Creating your Web API hosting server", 
               @"Remember to add the default route when self-hosting your Web API. In a web application, this configuration is automatically created for you, but you should add it yourself when self-hosting.");

            ET("4.4", 
               @"Hosting services in a Windows Azure worker role", 
               @"Deploying a Web API to Windows Azure Websites doesn’t require any more extra steps than deploying it to an on-premises IIS.");

            MC("4.4 (1)", 
               @"Which of the following items must be provided to use the self-host option?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"AspnetSelfHostConfiguration",
                        IsCorrect = false,
                        Explanation = @"AspNetSelfHostConfiguration does not exist and is not a valid option."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HttpSelfHostConfiguration",
                        IsCorrect = true,
                        Explanation = @"HttpSelfHostConfiguration is one of the two required classes needed to
self-host."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"AspnetSelfHostServer",
                        IsCorrect = false,
                        Explanation = @"AspNetSelfHostServer does not exist and is not a valid option."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HttpSelfHostServer",
                        IsCorrect = true,
                        Explanation = @"HttpSelfHostServer is the second of the two required classes needed to
self-host."
                    }
                });

            SC("4.4 (2)", 
               @"You are changing the hosting of your Web API to use a Windows Service. Originally, the configuration was automatically generated by the Visual Studio template. What do you need to do to make this work?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Nothing; the existing configuration will work.",
                        IsCorrect = false,
                        Explanation = @"Without specifying a configuration, the host will not operate correctly.
You need the HttpSelfHostConfiguration class to start the host and set up the api
route."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Copy the existing configuration to a new class in the host project.",
                        IsCorrect = false,
                        Explanation = @"As it stands, a copy of the existing configuration does not work because
the HttpSelfHostConfiguration class needs to be used."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add a new class of type HttpSelfHostConfiguration. Use the Initialize method to
call the MapHttpRoute method.",
                        IsCorrect = false,
                        Explanation = @"Although the HttpSelfHostConfiguration should be used, the Routes
property is what you use to access the MapHttpRoute method."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add a new class of type HttpSelfHostConfiguration. Use the Routes property to call
the MapHttpRoute method.",
                        IsCorrect = true,
                        Explanation = @"The Routes property on the HttpSelfHostConfiguration class has the
MapHttpRoute method necessary to configure Routes."
                    }
                });

            SC("4.4 (3)", 
               @"You are working for a large company that has a lot of maintenance engineers on the
road. You are going to develop an app to support their work remotely with publicly
available data. You are looking at your deployment options. You know the app will be
used during office times. Throughout the day, you will have five thousand users of your
service. What deployment option do you choose?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You buy extra servers for your on-premise data center.",
                        IsCorrect = false,
                        Explanation = @"Because your data is publicly available, you don’t face any security issues
when moving your service to the cloud. Because you have a fluctuating user
load, the elasticity of the cloud can help you. Deploying to Azure websites is the
easiest solution."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You use Azure Cloud Services to host your Web API as a Web Role.",
                        IsCorrect = false,
                        Explanation = @"Cloud Services are an option, but because you are only deploying a
Web API service, it’s much easier to use Azure websites."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You use Azure websites.",
                        IsCorrect = true,
                        Explanation = @"Azure websites saves a lot of time and cost upfront. Because you pay
for each minute you use, you can save costs by scaling your service down outside
office times. When you have a high user load, you can automatically add extra
servers to make sure your service performs."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You deploy Azure Virtual Machines to host your Web API.",
                        IsCorrect = false,
                        Explanation = @"Azure virtual machines require you to manage all servers yourself. You
get the hardware from Azure, but it still requires configuring and updating your
operating systems. Azure websites is a much easier solution to host a Web API."
                    }
                });

            MT("4.4",
               @"Choosing your hosting options",
               @"You are planning the deployment of your Web API and you are looking at the vari-ous options you have. You are looking at self-hosting, hosting in IIS on your own server, and hosting on Windows Azure Websites.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"What are the pros and cons for self-hosting?",
                       Explanation = @"Self-hosting is really easy to set up. You can use it for things such as integration tests or
for hosting your Web API on a server that doesn’t have IIS installed. However, ASP.NET
hosting offers you all the mature features that IIS has, which is a big plus."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"If you choose to use IIS, do you want to use an on premise IIS or Windows Azure Websites?",
                       Explanation = @"Windows Azure Websites has the big advantage of completely outsourcing the infrastructure.
You get a robust infrastructure with easy scalability options. You pay only for
what you use. However, an on-premises server is totally under your control. Sometimes
this is a requirement for data security."
                   },
               });


            // Section 4.5
            CurrentBackground = @"Consume Web API web services";

            ET("4.5", 
               @"HttpGet via GetAsync", 
               @"Make sure that you understand how async and await work. Although they’re not explicitly mentioned in the exam objectives, they’re a fundamental part of C# 5. If you haven’t studied async and await yet, see Exam Ref 70-483: Programming in C#, by Wouter de Kort, or start at http://msdn.microsoft.com/en-us/library/vstudio/hh191443.aspx.");

            ET("4.5", 
               @"Sending and receiving requests in different formats", 
               @"To prepare for the exam, you can follow the example at http://www.asp.net/web-api/
overview/formats-and-model-binding/media-formatters. It shows you how to create your
own formatter and how to use it.");

            SC("4.5 (1)", 
               @"The HttpClient class you have implemented frequently receives error codes, but the application is not responding to them. What should you do?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Trap the OnError event of the request.",
                        IsCorrect = false,
                        Explanation = @"There is no OnError event, so trapping it would not work."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Set the EnsureSuccessStatusCode property to true and implement an exception handler in the OnError event.",
                        IsCorrect = false,
                        Explanation = @"The EnsureSuccessStatusCode is a method, not a property. You also
need to catch an exception directly with a try/catch statement."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Call the EnsureSuccessStatusCode method and trap the HttpRequestException exception.",
                        IsCorrect = true,
                        Explanation = @"The EnsureSuccessStatusCode method should be called, and an exception
should be trapped of type HttpRequestException."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Check the ResultStatusCode property and throw an exception if one of the error values is present.",
                        IsCorrect = false,
                        Explanation = @"Checking the status code would work, but you would need to check
each value in the range of 500–599. The EnsureSuccessStatusCode method is a
better way to do this."
                    }
                });

            MC("4.5 (2)", 
               @"You need to create a custom media type for vCards. The media formatter needs to access an XML file to get the format for specific vCards. What should you do?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a formatter using the MediaTypeFormatter class.",
                        IsCorrect = true,
                        Explanation = @"You should inherit from the MediaTypeFormatter class if you want to
execute any I/O or other asynchronous operation."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a formatter using the BufferedMediaTypeFormatter class.",
                        IsCorrect = false,
                        Explanation = @"You should use the MediaTypeFormatter base class so you can implement
asynchronous I/O."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add the formatter to each request at the client.",
                        IsCorrect = false,
                        Explanation = @"The formatter should be added to the server configuration."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add the formatter to the Web API configuration on the server.",
                        IsCorrect = true,
                        Explanation = @"You activate the formatter by adding it to the global Web API configuration
on the server."
                    }
                });

            SC("4.5 (3)", 
               @"You have multiple HttpClient calls that can run independently of each other. You want to execute them as fast as possible. What should you do?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the Result property of each HttpClient call to get the result as fast as possible.",
                        IsCorrect = false,
                        Explanation = @"Using the Result property blocks the asynchronous call until it’s finished.
This is not the fastest way."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use async/await to make sure that the calls run asynchronously.",
                        IsCorrect = false,
                        Explanation = @"Although it runs the requests asynchronously, it doesn’t run them in
parallel."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use Task.WaitAll to execute the tasks in parallel.",
                        IsCorrect = true,
                        Explanation = @"Task.WaitAll can be used to run multiple asynchronous requests in
parallel."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You can’t execute asynchronous tasks in parallel.",
                        IsCorrect = false,
                        Explanation = @"This is not true. Although async/await doesn’t execute requests in
parallel, you can do this by using other features of the Task Parallel Library such as
Task.WaitAll."
                    }
                });

            MT("4.5",
               @"Consuming a Web API",
               @"You are integrating with a third-party Web API service. The service enables you to
execute CRUD operations to work with time records.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"How can you map the HttpClient methods to the standard CRUD operations?",
                       Explanation = @"CRUD stands for Create, Read, Update and Delete. Create can be done with Post, Read
with Get, Update with Put, and Delete with the Delete verb."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Why is it useful to create classes that map to the return values of the service?",
                       Explanation = @"This way, you don’t have to work with raw JSON or XML data. By deserializing the
results to classes, you can easily use them in the rest of your code."
                   },
               });



            ////////////////// Chapter 5 - Deploying web applications and services ///////////////////////


            // Section 5.1
            CurrentBackground = @"Design a deployment strategy";

            ET("5.1", 
               @"Automating a deployment from TFS or Build Server", 
               @"The exam states that you should know how to automate a deployment from TFS or Build
Server. Knowing the conceptual idea behind automatic deployments can help you if you
get any scenarios about it on the exam.");

            ET("5.1", 
               @"Automating a deployment from TFS or Build Server", 
               @"For the exam, it’s good to know that TFS is hosted on-premises on your own server. Team
Foundation Service is hosted in the cloud and is updated every three weeks.");

            ET("5.1", 
               @"Deploying to web farms", 
               @"It’s important to know the difference between scaling up and scaling out for the exam.
Make sure that you understand the advantages and disadvantages of both solutions.");

            SC("5.1 (1)", 
               @"You need to deploy an application that requires some changes to the registry. Which
deployment strategy do you use?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Copy the website.",
                        IsCorrect = false,
                        Explanation = @"The Copy Website option can be used only to deploy the files of an application.
You can’t make any modifications to the registry."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"FTP client.",
                        IsCorrect = false,
                        Explanation = @"With an FTP client, you can copy your files to the server. However, you
can’t make any changes to the registry."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Web Deploy.",
                        IsCorrect = true,
                        Explanation = @"Web Deploy enables you to use a provider that can make changes to the
registry while deploying your application."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"A web farm.",
                        IsCorrect = false,
                        Explanation = @"A web farm is a description of how you will host your application. It’s
not a deployment technique."
                    }
                });

            MC("5.1 (2)", 
               @"You are planning to deploy your application to an on-premises web farm. You use session
state to store some user data. Which session state mode do you use?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"InProc",
                        IsCorrect = false,
                        Explanation = @"The InProc mode does not work in a web farm. Servers store the session
data in memory, which makes it unavailable for other servers."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"StateServer",
                        IsCorrect = true,
                        Explanation = @"Storing your data in a separate StateServer makes it available to all servers
in the web farm."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Off",
                        IsCorrect = false,
                        Explanation = @"Turning session state off is only an option when your application
doesn’t use any session state at all."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"SQL Server",
                        IsCorrect = true,
                        Explanation = @"Storing your data in a separate SQL Server makes it available to all servers
in the web farm."
                    }
                });

            MC("5.1 (3)", 
               @"You want to adopt a continuous deployment strategy. Which elements are important?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Build server",
                        IsCorrect = true,
                        Explanation = @"Having a Build Server that tests your code and then executes a deployment
is required."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Unit tests",
                        IsCorrect = true,
                        Explanation = @"Unit tests are a part of continuous integration. They test your code in an
automatic way so you can find bugs before deploying your code."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Integration tests",
                        IsCorrect = true,
                        Explanation = @"Integration tests test your application in a more elaborate way than unit
tests. Together with unit tests, they are important for automatic testing of your
application."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"A web farm",
                        IsCorrect = false,
                        Explanation = @"A web farm is not required to have a continuous deployment strategy.
Even if you have a single server or are using the cloud, you can automatically
deploy."
                    }
                });

            MT("5.1",
               @"Choosing a deployment strategy",
               @"You are considering a deployment strategy for a web service that will be the back
end of a new app you will deploy worldwide on multiple platforms. Because of your
requirements, you will use WCF to create your back end. You are planning to build
your app in successive iterations that will expand the functionality. You also expect
that your user load will increase when your app gains popularity.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"How can Windows Azure help you with the hosting of your back end?",
                       Explanation = @"A WCF Service can easily be deployed to Windows Azure. Because of the expected
increase in users, Windows Azure is a good choice. Instead of buying big servers upfront
to host the potential user load that you will get, you can start small and pay only
for the resources you need. As your application starts growing, you can increase the
capacity you have inside Windows Azure. This is a cost-effective way of hosting your
application."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Why should you plan for continuous deployment?",
                       Explanation = @"Because you are planning to expand the functionality of your application, it is wise to
have a deployment process in place in which you can quickly update your application.
Setting up a continuous deployment environment takes some time, but will make life
easier when you start releasing updates for your application."
                   },
               });


            // Section 5.2
            CurrentBackground = @"Choose a deployment strategy for a Windows Azure web application";

            SC("5.2 (1)", 
               @"You are deploying a new cloud service with only a web role, and you want to make
sure that you get the maximum guaranteed uptime, even during upgrades. How many
instances do you need?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"1",
                        IsCorrect = false,
                        Explanation = @"The Windows Azure Service Level Agreement says that you must have
two or more role instances in different fault and upgrade domains, so one instance
is not enough."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"2",
                        IsCorrect = true,
                        Explanation = @"The Windows Azure Service Level Agreement says that you must have
two or more role instances in different fault and upgrade domains."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"3",
                        IsCorrect = false,
                        Explanation = @"The Windows Azure Service Level Agreement says that you must have
two or more role instances in different fault and upgrade domains. If you don’t
need three instances for performance, it is not necessary to deploy more than two
instances."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"4",
                        IsCorrect = false,
                        Explanation = @"The Windows Azure Service Level Agreement says that you must have
two or more role instances in different fault and upgrade domains. If you don’t
need four instances for performance, it is not necessary to deploy more than two
instances."
                    }
                });

            SC("5.2 (2)", 
               @"You want to be able to communicate directly from your web role to your worker role,
and you want to make sure that your worker role stays secure by disallowing public
access. What do you do?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add an InputEndpoint to the web role in your ServiceConfiguration.cscfg file.",
                        IsCorrect = false,
                        Explanation = @"You should add an InternalEndPoint to your worker role. By adding
extra endpoints to your web role, you don’t enable communication to your worker
role."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add an InputEndpoint to your worker role in your ServiceConfiguration.cscfg file.",
                        IsCorrect = false,
                        Explanation = @"You shouldn’t add a public InputEndpoint to your worker. This will make
your worker accessible from outside your cloud service. Instead, you should add an
InternalEndPoint."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add an InternalEndPoint to the web role in your ServiceConfiguration.cscfg file.",
                        IsCorrect = false,
                        Explanation = @"You should add an InternalEndPoint to your worker role. By adding
extra endpoints to your web role, you don’t enable communication to your worker
role."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add an InternalEndPoint to your worker role in your ServiceConfiguration.cscfg file.",
                        IsCorrect = true,
                        Explanation = @"An InternalEndPoint to your worker role enables communication from the
web role to the worker role."
                    }
                });

            MC("5.2 (3)", 
               @"You want to follow the recommended best practices for configuring your Windows
Azure Guest OS. Which values do you use?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"osFamily=”3”",
                        IsCorrect = false,
                        Explanation = @"This specifies a fixed version of your OS. You should use an asterisk (*)
to make sure you always get all updates."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"osFamily=”1”",
                        IsCorrect = false,
                        Explanation = @"A value of 1 specifies one of the older Guest OSs."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"osVersion=”*”",
                        IsCorrect = true,
                        Explanation = @"By using *, you don’t depend on a specific version. You get automatic
updates to your Guest OS."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"osVersion=”WA-GUEST-OS-2.12_201208-02”",
                        IsCorrect = false,
                        Explanation = @"You shouldn’t depend on a specific version. This disables automatic
updates."
                    }
                });

            MT("5.2",
               @"Updating your cloud service",
               @"Your company distributes a cloud service with multiple worker and web roles. A new
update will soon be available, and you are considering the different options you
have for updating the cloud service.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"What is the difference between VIP Swap and in-place upgrade?",
                       Explanation = @"With an in-place upgrade, you update your instances according to update domains.
This will give you zero downtime."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Which one do you prefer? Why?",
                       Explanation = @"With a VIP Swap, you promote your staging environment to production (and change
your production environment to staging). A VIP Swap has the advantage of mitigating
the risk of bringing multiple versions of your application into production at the same
time."
                   },
               });

            
            // Section 5.3
            CurrentBackground = @"Configure a web application for deployment";

            ET("5.3", 
               @"Using SetParameters to set up an IIS app pool", 
               @"Make sure that you understand both web.config transformations and the use of parameters.
Remember that transformations are done at compile time, and parameter values are
set during deployment.");

            MC("5.3 (1)", 
               @"You want to deploy your ASP.NET MVC application to your own web server for production.
Which steps do you take?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Change the debug attribute of the web.config to false.",
                        IsCorrect = true,
                        Explanation = @"By setting the debug attribute to false (or removing it), you enable certain
optimizations that you want to use on your production environment."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Build a release configuration.",
                        IsCorrect = true,
                        Explanation = @"A release configuration has better performance and should be used in a
production environment."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Disable the ASP.NET debugger in the Debuggers section of your project
properties.",
                        IsCorrect = false,
                        Explanation = @"The selected debuggers don’t affect performance when there is no
debug information in the assemblies. You don’t have to explicitly turn them off."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Build a debug configuration.",
                        IsCorrect = false,
                        Explanation = @"A debug configuration has a slower performance than a release configuration
and should not be used for a production environment."
                    }
                });

            SCC("5.3 (2)", 
               @"You want to remove your debug element from the web.config file by using web.config transformations. Which syntax do you use?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"<compilation xdt:Transform=”Replace” />",
                        IsCorrect = false,
                        Explanation = @"You don’t want to completely replace the compilation element. If this
element contains other values, you want to remove only the debug attribute."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"<compilation xdt:Transform=”RemoveAttributes(debug)” xdt:Locator=”Condition(@debug=’true’)” />",
                        IsCorrect = false,
                        Explanation = @"You don’t need the locator attribute. Without a locator attribute, Web
Deploy matches your element to the existing element in web.config."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"<compilation xdt:Transform=”RemoveAttributes(debug)” xdt:Locator=”Match(name)” />",
                        IsCorrect = false,
                        Explanation = @"You don’t need the locator attribute. Without a locator attribute, Web
Deploy matches your element to the existing element in web.config. Matching on
name makes no sense because the compilation element doesn’t have a name."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"<compilation xdt:Transform=”RemoveAttributes(debug)” />",
                        IsCorrect = true,
                        Explanation = @"You remove the debug attribute from the compilation element."
                    }
                });

            SC("5.3 (3)", 
               @"You are using parameters for your Web Deploy. You want to automate the creation of
the SetParameters file and make sure that it has the correct values. What do you use?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"XmlPoke with MSDeploy.",
                        IsCorrect = false,
                        Explanation = @"XmlPoke is an MSBuild Task, not an MSDeploy parameter."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"MSDeploy with the setParamFile attribute.",
                        IsCorrect = false,
                        Explanation = @"The setParamFile is used to supply a Parameters.xml file that’s used for
parameterization. You will still have to edit the SetParameters.xml file."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"XmlPoke with MSBuild.",
                        IsCorrect = true,
                        Explanation = @"XmlPoke is an MSBuild task that you can use to edit XML files in an automated
way."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"This is not possible. You need to edit the SetParameters file manually.",
                        IsCorrect = false,
                        Explanation = @"By using MSBuild with the XmlPoke task, you can edit your SetParameters
automatically."
                    }
                });

            MT("5.3",
               @"Preparing for deployment",
               @"You are creating a complex web application that will be deployed to multiple,
on-premises servers. You are working with development, testing, acceptance, and
production environments; and you want to automate your deployment as much as
possible.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"Which configuration do you want to release to each of your different
environments?",
                       Explanation = @"On your development and test environments, you can use a debug configuration. This
way, you can more easily debug your code and find errors. Acceptance and production
want to use the release configuration. If you want more granular control, you can also
add extra configurations for your environments."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Do you want to use Web.config transformations or parameters with Web Deploy
for your deployment pipeline?",
                       Explanation = @"Because you know beforehand to which environments you are going to deploy, it is
easier to use Web.config transformations. Depending on the environment you work in,
you might need to deploy the same DLL that passed quality control to production. This
would require you to use parameterization to account for the differences between the
environments."
                   },
               });


            // Section 5.4
            CurrentBackground = @"Manage packages by using NuGet";

            ET("5.4", 
               @"Installing and updating an existing NuGet package", 
               @"Remember that a NuGet package is not only about adding references to your project. A
NuGet package can contain all kinds of data, such as code or script files and even configuration
changes that can be added to your project.");

            MC("5.4 (1)", 
               @"You want to update jQuery package in your ASP.NET MVC 4 application in the easiest
way possible. What do you do?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Download the newest version from http://jquery.com.",
                        IsCorrect = false,
                        Explanation = @"ASP.NET MVC 4 templates come with package references for all installed
libraries. You can use NuGet to update the package."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Run the command Update-Package jQuery from the Developer command prompt.",
                        IsCorrect = false,
                        Explanation = @"You should run this command from within Visual Studio in the Package
Manager Console."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Run the command Update-Package jQuery from the Package Manager Console.",
                        IsCorrect = true,
                        Explanation = @"This updates your jQuery from the Windows PowerShell command line."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Click the update button in the Package Manager window for the jQuery package.",
                        IsCorrect = true,
                        Explanation = @"You can update the package in through a GUI."
                    }
                });

            SC("5.4 (2)", 
               @"You want to package a custom package that uses the lib, content, and tool folders.
Which command do you use?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"nuget pack MyProject.csproj",
                        IsCorrect = false,
                        Explanation = @"The pack command on a project file converts that project in a package."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"nuget spec MyAssembly.dll",
                        IsCorrect = false,
                        Explanation = @"This creates a nuspec file, not a package."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"nuget pack package.nuspec",
                        IsCorrect = true,
                        Explanation = @"This creates your package with your lib, content, and tools folder."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"nuget push package.nupkg",
                        IsCorrect = false,
                        Explanation = @"The push command is used to publish your package on NuGet."
                    }
                });

            MC("5.4 (3)", 
               @"You want to expose a read-only feed of custom packages on your internal web server
to your colleagues. Which steps do you take?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Set the API key in your appSettings.",
                        IsCorrect = false,
                        Explanation = @"You can leave the API key empty to make sure that your feed is readonly."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Install the NuGet.Server package.",
                        IsCorrect = true,
                        Explanation = @"The NuGet.Server package contains all code for exposing your custom
feed in a web project."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Deploy your application to IIS.",
                        IsCorrect = true,
                        Explanation = @"Your custom feed needs to be hosted in IIS."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a new Empty ASP.NET Web Application.",
                        IsCorrect = true,
                        Explanation = @"A new empty ASP.NET Web Application is the basis of your custom feed."
                    }
                });

            MT("5.4",
               @"Should you use NuGet?",
               @"You are working in a small company that runs a lot of web development projects
each year. Each time you start a new project, you evaluate the current state of
technologies and you try to use the newest technologies available. However, your
colleagues feel that they want control over what gets used in all the projects. You
are thinking of introducing NuGet to them and you are trying to prepare a good
argument.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"Are you still in control over what gets added to your project when using NuGet?
Is that an advantage or a disadvantage?",
                       Explanation = @"You can see all the changes that NuGet made, but you are not totally in control anymore.
NuGet downloads any dependencies for your package automatically. NuGet
can also make changes to your configuration files and add code and other files to your
project. However, this is not a disadvantage because NuGet makes only the changes
that you had to make manually."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"How can NuGet help you stay on the cutting edge of new releases?",
                       Explanation = @"NuGet offers a simple way to update your packages to new releases. You can even use
NuGet to download prerelease packages."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Would it be useful to create your own package repository?",
                       Explanation = @"If you don’t want to create custom packages or keep a local cache of packages, this is
not necessary."
                   }
               });



            // Section 5.5
            CurrentBackground = @"Create, configure, and publish a web package";

            MC("5.5 (1)", 
               @"You want to deploy your website to Windows Azure Websites. What can you do?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a new publish profile and choose the Web Deploy option. Enter the credentials
of your server and publish your website.",
                        IsCorrect = false,
                        Explanation = @"You can download a publish profile from Windows Azure."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Download the publish profile from the Management Portal and import it in Visual
Studio.",
                        IsCorrect = true,
                        Explanation = @"This enables you to update your website directly from Visual Studio."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use FTP publishing to copy your files directly to Windows Azure.",
                        IsCorrect = true,
                        Explanation = @"You can use FTP publishing directly to Windows Azure Websites from
Visual Studio."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a Web Deploy package and upload it to the Management Portal.",
                        IsCorrect = false,
                        Explanation = @"You can’t upload a Web Deploy package for a website in the Management
Portal."
                    }
                });

            SC("5.5 (2)", 
               @"You are using the command line, and you want to see whether your deployment is configured correctly. What do you use?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"postSync",
                        IsCorrect = false,
                        Explanation = @"postSync executes a command after your synchronization has already
run."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"preSync",
                        IsCorrect = false,
                        Explanation = @"preSync executes a command before your synchronization runs."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"verb",
                        IsCorrect = false,
                        Explanation = @"verb is used to specify your required operation."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"whatif",
                        IsCorrect = true,
                        Explanation = @"Specifies that the command should run without making any changes. It
displays an overview of what your deployment will execute, so you can inspect it."
                    }
                });

            SC("5.5 (3)", 
               @"You want to create and deploy a package from the command line. Which command do you use?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"MSBuild “MyProjectName.csproj” /T:Package /P:Configuration=Release",
                        IsCorrect = false,
                        Explanation = @"The package switch only creates a package. It doesn’t deploy it."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"MSBuild “MyProjectName.csproj” /T:Package",
                        IsCorrect = false,
                        Explanation = @"The package switch only creates a package. It doesn’t deploy it. By
default, this also generates a debug configuration."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"MSBuild “MyProjectName.csproj” /T:Package /P:Configuration=Release;DeployOnBuild=True",
                        IsCorrect = true,
                        Explanation = @"This packages and deploys the project with the correct configuration."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"MSBuild “MyProjectName.csproj” /P:Configuration=Release;DeployOnBuild=True",
                        IsCorrect = false,
                        Explanation = @"You are missing the /T:package switch."
                    }
                });

            MT("5.5",
               @"Moving to continuous deployment",
               @"You are planning an internal presentation for your colleagues to show them the
benefits of automating your deployment. Currently, you are doing manual deployments
in which you just copy the required files through FTP to the target servers.
Database updates are always done manually.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"What are the advantages that continuous deployment can give you? Make a list.",
                       Explanation = @"Because continuous deployment decreases your feedback cycle, you get much quicker
feedback on your application. You can respond to customer demand more quickly. In
the current world in which agile is the norm, this is definitely a requirement for most, if
not all, applications."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Which techniques do you need for this?",
                       Explanation = @"Because you are using multiple servers and databases, Web Deploy is the best choice.
By using the command line or Windows PowerShell tools, you can integrate with your
build process and automatically update your servers. If you are using the Entity Framework,
you can use automatic migrations. Otherwise, you can use one of the other
techniques such as dbDacFx or dbFullSql."
                   },
               });


            // Section 5.6
            CurrentBackground = @"Share assemblies between multiple applications and servers";

            ET("5.6", 
               @"Signing assemblies by using a strong name", 
               @"Make sure that you understand the benefits of signing an assembly. Also make sure that you know when to use delayed signing.");

            ET("5.6", 
               @"Implementing assembly versioning", 
               @"The probing option can be used only to point to locations that are relative to the application path. If you want to locate assemblies somewhere else, you have to use the codebase element.");

            SC("5.6 (1)", 
               @"You are building a strong-named assembly and you want to reference a regular assembly to reuse some code you built. What should you do?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You first need to put the assembly in the GAC.",
                        IsCorrect = false,
                        Explanation = @"An assembly in the GAC needs to be strongly named. Your assembly still
won’t be able to reference the unsigned assembly."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Nothing. Referencing another assembly is always possible.",
                        IsCorrect = false,
                        Explanation = @"A strong-named assembly cannot reference a non-strong-named
assembly."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You need to sign the other assembly before using it.",
                        IsCorrect = true,
                        Explanation = @"You need to strongly name the other assembly before you can reference
it."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You need to use the public key token of the other assembly to reference it.",
                        IsCorrect = false,
                        Explanation = @"The public key token is a part of the manifest of a strong-named
assembly. The non-strong-named assembly doesn’t have this key information. It
needs to be strongly named first."
                    }
                });

            MC("5.6 (2)", 
               @"You are building an assembly that will be used by a couple of server applications. You want to make the update process of this assembly as smooth as possible. Which steps should you take?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Run aspnet_intern.",
                        IsCorrect = false,
                        Explanation = @"aspnet_intern is used to intern assemblies, meaning that only one copy
of an assembly will be loaded in memory when it’s referenced by multiple web
applications. This is only for performance reasons. It doesn’t mean you can update
the assembly in one location."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Deploy the assembly to the GAC.",
                        IsCorrect = true,
                        Explanation = @"A shared assembly can be deployed in the GAC. Other applications can
reference it there. When you want to update it, you can do so by deploying the
new version to the GAC. By using configuration files, you can then let other applications
reference your new assembly."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add an assemblyBinding section to each client application that points to the location of the assembly.",
                        IsCorrect = false,
                        Explanation = @"You can use the assemblyBinding configuration element to add extra
search locations for an assembly. It asks for changes to each client application,
however. The GAC is the location in which a shared assembly needs to be
deployed."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Strongly name the assembly.",
                        IsCorrect = true,
                        Explanation = @"Strongly naming an assembly doesn’t make it a shared assembly. Each
application still requires its own copy. But when deploying to GAC, you need strong names."
                    }
                });

            MC("5.6 (3)", 
               @"You want to deploy an assembly to a shared location on the intranet. Which steps should you take?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Strongly name the assembly.",
                        IsCorrect = true,
                        Explanation = @"Strongly naming the assembly is required to be able to reference it on
the intranet."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the codebase configuration element in the applications that use the assembly.",
                        IsCorrect = true,
                        Explanation = @"The codebase configuration element can be used to have local client applications
know they can find an assembly on another location such as the intranet."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Deploy the assembly to the GAC.",
                        IsCorrect = false,
                        Explanation = @"Deploying it to the GAC doesn’t put the assembly on the intranet."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the assemblyBinding configuration element with the probing option.",
                        IsCorrect = false,
                        Explanation = @"The probing option can be used only to give additional locations relative
to the application path. It can’t be used to point to the intranet."
                    }
                });

             MT("5.6",
               @"Choosing your technologies",
               @"You are building a library that will contain some utility code. The idea is that the
library can be shared by multiple web applications that you are developing.",
               new TextboxAnswer[]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"Why should you sign your assembly?",
                       Explanation = @"Signing the assembly protects the assembly against tampering. The .NET Framework
will check that the assembly hasn’t been altered between signing and running. Signing
is also a requirement to be able to use a digital certificate, so users of your application
will know that you are the publisher of the application. A disadvantage could be that
you can no longer reference other unsigned assemblies. If you own these assemblies,
you can sign them yourself. If not, you have to ask their publisher to sign them."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What is the advantage of interning your assembly?",
                       Explanation = @"The advantage of interning your assemblies is that your assembly won’t be loaded
multiple times into memory. Instead, one copy will be loaded that gets used by all web
applications that reference it."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"How you can you do this?",
                       Explanation = @"You can intern your assemblies by running the aspnet_intern tool regularly on your
server."
                   }
               });

       }
    }
}