﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace Revision.Models
{
    /// <summary>
    /// Exam Ref 70-486: Developing ASP.NET MVC 4 Web Applications
    /// </summary>
    public class ExamMVC4 : Exam
    {
        /// <summary>
        /// Exam Ref 70-486: Developing ASP.NET MVC 4 Web Applications
        /// </summary>
        public ExamMVC4(bool isTextRedacted) : base(title: "Developing ASP.NET MVC 4 Web Applications (70-486)",
                                                    examCode: "70-486",
                                                    passMark: 70)
        {
            IsTextRedacted = isTextRedacted;

            // Section 1.1
            CurrentBackground = @"Design the application architecture - Plan the application layers";

            SC("1.1 (1)", 
               @"You are designing an application in which a section of the main page will be populated
by content from a third-party provider. You do not have control over the responsiveness
of the client or how much information will be returned with each request. The call
is to a RESTful service and will return the information formatted in Extensible Markup
Language (XML). What is the best way to implement this application?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Design a model that handles the data call to populate the model. Create a partial
                view containing only this display area and put an asynchronous service call that
                returns this model in the partial view controller.",
                        IsCorrect = true,
                        Explanation = @"Because you do not have control over the responsiveness of the thirdparty
provider and you do not know how much data might be returned from each
call, you should wrap the call in the asynchronous framework. Providing the data in
a strongly-typed model gives it more flexibility than working with the raw XML on
the client side."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Put a synchronous service call into the main page controller.",
                        IsCorrect = false,
                        Explanation = @"You do not know how long the call to the third party will take, and putting
a synchronous call into the main page will not give any response until the call
is completed."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a partial view containing only this display area and put a synchronous service
                call in the partial view controller.",
                        IsCorrect = false,
                        Explanation = @"You do not want to use a synchronous call in this case due to the unknown
response time."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a partial view containing only this display area and put an asynchronous
                service call in the partial view controller.",
                        IsCorrect = false,
                        Explanation = @"Although you can take this approach, it infers that you will manipulate
the third-party response data in either the controller or the view. SoC recommends
that this manipulation occur in a model."
                    }
                });

            SC("1.1 (2)",
               @"You have been given requirements for a dashboard page that will contain summary information from your order processing system in a single display table. However, this summarization needs to be done by combining data requests from the order system, the shipping system, and the accounting system. The dashboard page will be the only place you use this combined data. What is the best way to implement this requirement?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Make the various data requests and compile the information in the controller for display.",
                        IsCorrect = false,
                        Explanation = @"You should not perform any data manipulation in the controller."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create an individual model for each of the data requests, and then create a view-specific model that calls those models and merges the data.",
                        IsCorrect = true,
                        Explanation = @"You will have a better chance of code reuse if you break down the
separate calls into their own models and then create another model to pull them
together and compile them."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a model for the summary data and handle the various data requests within that model as well as the merging of the data.",
                        IsCorrect = false,
                        Explanation = @"Although this would be a plausible way to implement the solution, it is
not the best. If any other work came up that uses any of the calls within this model,
you will either have to refactor the code to extract it at that point or have duplicate
code."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create an individual model for each of the data requests and then merge the data on the client side for display.",
                        IsCorrect = false,
                        Explanation = @"The fact that this data can be merged into a single table display shows
there is some intrinsic business worth to the information in this format. Merging on
the client side goes against SoC considerations."
                    }
                });

            MC("1.1 (3)",
               @"A significant change has been requested in an application maintained within your company. The application is a classic ASP application that uses custom Open Database Connectivity (ODBC) drivers to connect to a relational data repository on a mainframe computer. The CIO decided that the company needs to replace this 30-year-old system. The team that worked on the original project is made up of developers who have never worked with an object-oriented approach before. Which approaches would be the best to use when designing your initial schema in Entity Framework?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create your own custom design because it’s too much work to manage an inexperienced staff.",
                        IsCorrect = false,
                        Explanation = @"You should incorporate your team into the project as soon as possible."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use Code First.",
                        IsCorrect = false,
                        Explanation = @"Because the team has no experience with object-oriented programming,
the Code First approach is unlikely to be the most efficient way to create the
new schema."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use Model First.",
                        IsCorrect = true,
                        Explanation = @"The use of the Entity Designer as an integral component in the Model
First approach will help unfamiliar users to walk through the process."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use Database First.",
                        IsCorrect = true,
                        Explanation = @"There is already a working relational database for the application,
although it is in a system that will be replaced. A port of the design should be
considered."
                    }
                });

            SC("1.1 (4)",
               @"You are designing an application that allows employees to change their human resources (HR) information, such as next of kin and direct deposit information. The requirements state that the application should talk directly to the HR systems’ database. However, at a recent company meeting, the CFO announced that the company will be converting to a new HR system over the next two years. They will take an additional year to move employees to the new system, one department at a time. How will this affect your design?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"It won’t; the requirements state that the application should talk directly to the HR systems’ database.",
                        IsCorrect = false,
                        Explanation = @"It is the designer’s job to ensure that any known enhancements or future
changes are accounted for. Although this approach follows the requirements,
it is not the best long-term solution."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You should ensure your naming convention for the database as clearly as possible so you can rework your data calls with minimal changes.",
                        IsCorrect = false,
                        Explanation = @"This solution does not provide the proper level of abstraction; it requires
either a “one or the other” approach to supporting the HR system, or an
approach in which you have to manage which database you are calling from within
each data call."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You should implement the Repository pattern with the current HR system being
the first repository that is built. When the second system comes online, you implement
that data access using the same pattern.",
                        IsCorrect = true,
                        Explanation = @"Using the Repository pattern will give you a level of abstraction into the
data layer. When you create the second data access component for the new HR
system, you can then differentiate on a user or departmental level which implementation
to use."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You should map the model directly to the database calls, anticipating that you will
have to change the model as the new system rolls out.",
                        IsCorrect = false,
                        Explanation = @"This solution does not provide the proper level of abstraction; it requires
either a “one or the other approach” to supporting the HR system, or an
approach in which you have to manage which database you are calling from within
each data call."
                    }
                });


            MT("1.1",
               @"Implementing a government website",
               @"You are a consultant helping a municipal government bring some of its services onto the Internet. The first application you will work on enables pet licensing over the web. The initial work was done by a volunteer from the local pet shelter. Although it is an attractive website that was very well received by the public, it provides only downloadable forms that must be filled out and returned manually.",
               new TextboxAnswer[3]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"The client received positive feedback on its current website design and wants to
keep it. How would you plan to maintain the look and feel across the new application?
What components need to be included in your architecture?",
                       Explanation = @"By default, most of the HTML design that went into the original site would be put into
the views. Common areas such as navigation or boilerplate text could be put into a
layout page if there is a lot of reuse. Also, any areas that might need independent
functionality could be put into a partial view."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"The department currently files submitted forms in alphabetical order by pet
owner. What could you do with the information so the employees would have
real-time access to it?",
                       Explanation = @"There are several ways that this could be done. The first is to simply put the form
online and email the form results to the department. This gives them minimal advantage
over their current process and is not in real time. The next is to use a data storage
mechanism such as SQL Server to store the information in a database. This enables
them to have reports built as needed as well as giving them real-time access into the
data."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"You realize you would be best served by creating a separate business layer. How
would you manage this layer?",
                       Explanation = @"A typical breakdown of models for this situation could be a model for the user, a
model for the pet, and a model for the license year. The user might have multiple
pets and each pet might have multiple licenses, one for each year. The models would
handle the access into and out of the database. This application is a candidate for using
the Repository pattern because the system used to manage the backend might be
replaced as more departments go online and the municipality might standardize in a
different direction."
                   }
               });


            // Section 1.2
            CurrentBackground = @"Design the application architecture - Design a distributed application";

            MC("1.2 (1)",
               @"You are developing an application. One requirement is that part of your data access
layer needs to be available to a third party, that wants to get this information from a
REST URL in XML. Your company does not have experience with web services, but you
have several websites running ASP.NET MVC 4. How could you design and provide
these new services?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Task an individual on staff to learn WCF, and have this individual develop and deploy
these new services using WCF.",
                        IsCorrect = false,
                        Explanation = @"Although you would eventually be able to get a WCF REST services, it
would not be efficient."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the Web API to create REST services using ApiController in which the serialization
type is defined by the Accepts property of the browser.",
                        IsCorrect = true,
                        Explanation = @"Using the Web API is a straightforward way to present REST services."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Build a basic ASP.NET MVC 4 project in which the view simply passes through the
information provided by the controller, and the controller manages the code for
serializing the response.",
                        IsCorrect = true,
                        Explanation = @"Using ASP.NET MVC is another way to create a controller that will return
XML."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create an ASP.NET ASMX services file to get, serialize, and return the data.",
                        IsCorrect = false,
                        Explanation = @"An ASMX web service is SOAP-based, not REST-based."
                    }
                });

            MC("1.2 (2)",
                @"How could you traditionally consume an ASMX web service from your application?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Generate a proxy by selecting Add Reference In Visual Studio.",
                        IsCorrect = false,
                        Explanation = @"Selecting Add Reference does not enable you to create a proxy."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create an HttpService and connect using Get(URL).",
                        IsCorrect = false,
                        Explanation = @"HttpService.Get gets the output of a REST service."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Generate a proxy by selecting Add A Service Reference in Visual Studio.",
                        IsCorrect = true,
                        Explanation = @"Visual Studio creates a proxy for you from the WSDL at the site you select."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a WCF proxy class.",
                        IsCorrect = false,
                        Explanation = @"A WCF proxy class needs endpoints and bindings. REST services do not
use, nor understand, WCF endpoints and bindings."
                    }
                });

            MC("1.2 (3)",
               @"What are examples of hybrid applications using Windows Azure?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"An application where the local network hosts the IIS server while the database is
being run from the corporate IT office",
                        IsCorrect = false,
                        Explanation = @"Both the web application and the database storage are being run from
within the company network. Although this is a distributed design, it is not a hybrid
application."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"An application where Windows Azure is used to host the IIS process, and Windows
Azure SQL is used to store the data",
                        IsCorrect = false,
                        Explanation = @"Both the web instance and the data repository are using Windows
Azure technology. This is not a hybrid app; it is a fully deployed Windows Azure
application."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"An application where the IIS process is run on a local web server, whereas the data
is stored in Windows Azure SQL",
                        IsCorrect = true,
                        Explanation = @"Part of the application is being run in the Windows Azure environment;
the other part is being run in the corporate network environment."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"An application where the web part of the application is run on Windows Azure,
whereas the confidential data is stored in the company’s network",
                        IsCorrect = true,
                        Explanation = @"Part of the application is being run in the Windows Azure environment;
the other part is being run in the corporate network environment."
                    }
                });


            MT("1.2",
               @"Building a geographically distributed application",
               @"You are the new technical architect at a large custom home building firm with offices in North America, South America, and Europe. Your firm is expanding, both by opening new offices and by buying smaller custom home builders. Although each main geographic region stores its own data in its own systems, your CIO wants you to build an application that displays information from each region in different widgets on a dashboard. She does not want SQL queries run from the corporate office where the dashboard will be hosted.",
               new TextboxAnswer[3]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"You will have to deploy some software in the various regions. What will the
software do?",
                       Explanation = @"Yes, because you cannot query the databases directly, you need to deploy some kind
of solution that gives you access to the data. This could be a web services wrapper to
enable you to call the data remotely or an application that will manage aggregating
the data."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What are some primary concerns of building an ASP.NET MVC application that
gathers information from such disparate sources?",
                       Explanation = @"The primary concerns would be the inability to guarantee responsiveness and the need
to manage connection issues."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What is the benefit of adding Windows Azure AppFabric?",
                       Explanation = @"AppFabric acts as a service bus, so it provides a single point of contact/service connector
that would manage the calls out to the remote systems by routing the requests to
the appropriate server."
                   }
               });


            // Section 1.3
            CurrentBackground = @"Design and implement the Windows Azure role life cycle";

            SC("1.3 (1)", 
                @"What occurs if an unhandled error is fired on a startup task?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The startup role consumes the error during its load; if there is no event handler configured, it is set to Handled as the task completes.",
                        IsCorrect = false,
                        Explanation = @"If the startup task fires an unhandled error, the role startup stops in a
failure. The task will not complete successfully."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The startup is cancelled and the role does not start.",
                        IsCorrect = true,
                        Explanation = @"The task will stop processing and return a non-zero value."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The OnStop method automatically runs.",
                        IsCorrect = false,
                        Explanation = @"The task will stop in error. The OnStop process will not run because the
role will not get that far."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The startup task goes to the lowest security setting and continues to run, if possible.",
                        IsCorrect = false,
                        Explanation = @"The task will stop processing. It will not try to continue to run on a
lower security setting."
                    }
                });

            MC("1.3 (2)",
                @"AppCmd.exe is an application that helps you work with which objects?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Site",
                        IsCorrect = true,
                        Explanation = @"AppCmd.exe enables the configuration of virtual sites."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Users",
                        IsCorrect = false,
                        Explanation = @"There is no capability to manage users in AppCmd.exe."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Config",
                        IsCorrect = true,
                        Explanation = @"AppCmd.exe supports the administration of general configuration
sections."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"App",
                        IsCorrect = true,
                        Explanation = @"AppCmd.exe manages the support of applications."
                    }
                });

            MC("1.3 (3)", 
                @"Which of the following are valid reasons for overriding the Run method?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Creating and starting a messaging service that will work in parallel with the Web role to manage queued messages",
                        IsCorrect = true,
                        Explanation = @"Creating and running an application in parallel is what the Run method
was designed to allow."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Creating an always-running service that periodically makes HTTP calls to other websites to determine their availability",
                        IsCorrect = true,
                        Explanation = @"The polling service is a good example of an activity in which the Run
method enables a process to work independently of the main role."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Managing error handling for the application",
                        IsCorrect = false,
                        Explanation = @"The error handling will be managed in the OnError event and will not
involve the overridden Run method."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Starting and supporting a logging application for use by the Worker role",
                        IsCorrect = true,
                        Explanation = @"Creating and running an application in parallel is what the Run method
was designed to allow."
                    }
                });


            MT("1.3",
               @"Investigating Windows Azure",
               @"Your company wants to refactor its mission critical line-of-business (LOB) application to make it more robust and scalable as well as to increase performance. The CIO wants to investigate hosting the item in Windows Azure. Currently, the application has a lot of maintenance processes running in the background, such as processes to ensure that there is no orphaned data by running queries against the database, processes that check website availability with an email sent if a website is non-responsive; and a process that counts how many users logged in during the previous hour and sends an email to the IT support staff.",
               new TextboxAnswer[3]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"Where in the Windows Azure process would it make the most sense to put the orphaned data check?",
                       Explanation = @"When you need something to run the lifetime of the application, putting it into the
override of the Run method is the best solution. It enables you to create a timer that
fires an event every x minutes that you want to run the check."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Would it make more sense to put these processes in a Worker role or within the Web role?",
                       Explanation = @"They both would work. The advantage to putting them in a worker process is that they
can continue to function if the Web role has stopped for some reason. This is particularly
useful if there are other non-web ways of getting information into the database,
and you are still at risk for orphaned data."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Assuming that these processes were all C# console programs, do you think it would be difficult to migrate them for use in the cloud? Why or why not?",
                       Explanation = @"Provided that the processes are console applications, it should be relatively straightforward
to move them into methods that can be called from within startup process."
                   }
               });


            // Section 1.4
            CurrentBackground = @"Configure state management";

            MC("1.4 (1)",
                @"You are designing an ASP.NET MVC 4 application that uses an Oracle database for persistence. What session configuration choices enable you to deploy your application on a web farm?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"InProc",
                        IsCorrect = false,
                        Explanation = @"InProc does not support web farms as session items are stored only in
the individual server’s memory."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"SQLServer",
                        IsCorrect = false,
                        Explanation = @"SQLServer is not available in the application stack. This means that using
the default SQLServer state is not possible."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"StateServer",
                        IsCorrect = true,
                        Explanation = @"Using a shared state server across the web farm is an available option.
Using a state server designates one server to maintain state for all the servers that
connect to it."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Custom session provider",
                        IsCorrect = true,
                        Explanation = @"A custom session provider enables you to maintain state as necessary by
doing the work in your custom code. It is generally used when you try to use a different
RDBMS system or when you do not want to use the default session database
design."
                    }
                });

            MC("1.4 (2)", 
                @"You are creating an ASP.NET MVC 4 web application that will be accessed by a large number of traditional consumers. If you need to be able to access state information on the client side in JavaScript/jQuery, where can you store it?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"localStorage",
                        IsCorrect = false,
                        Explanation = @"localStorage is HTML5 and is not available in all browsers."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"QueryString",
                        IsCorrect = true,
                        Explanation = @"Query string information is available across all browsers and is usable on
both the client and server."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"ViewState",
                        IsCorrect = false,
                        Explanation = @"Although ViewState is available in a form field on the page, it is encrypted
and cannot be used on the client side. It is also not used by many ASP.NET
MVC 4 constructs."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Cookies",
                        IsCorrect = true,
                        Explanation = @"Cookies can be stored for a period of time on the client and be read from
either client- or server-side operations."
                    }
                });

            MC("1.4 (3)",
                @"As you design a sessionless state management system, what do you need to ensure that your application manages?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Access to the state management system, whether it is a database, a web service, or other type of system",
                        IsCorrect = true,
                        Explanation = @"Your application needs to manage whatever information might be required
to access the state management system."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The HTTP headers",
                        IsCorrect = false,
                        Explanation = @"The HTTP headers are usually not used as part of state management."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The session setting within the Web.config file",
                        IsCorrect = false,
                        Explanation = @"Because your application is sessionless, there is no need to manage session
in the Web.config file."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"An identifier used by the server to identify the request",
                        IsCorrect = true,
                        Explanation = @"Your application needs to manage the passing of the identifier between
requests."
                    }
                });



            MT("1.4",
               @"Designing an architecture for a process management system",
               @"You are an architect for a Fortune 1000 company that wants to create an internal task and process management system. The concept of the system is to give visibility to daily tasks that are performed throughout the company. Every employee is expected to use the system and be diligent about inputting tasks and to manage the statuses of the tasks as they move through the process. Your team was instructed to design an architecture that would support hundreds of simultaneous users. You have been given five physical servers and an old load balancer to run the system. You also have several licenses for SQL Server.",
               new TextboxAnswer[3]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"How would you provision these servers for maximum reliability, safety, and performance?",
                       Explanation = @"There are many different ways that you could provision the servers. A typical approach
would be to use two servers for SQL Server, with the data replicated between the
servers. One of the servers would be the primary SQL server while the other would be
the secondary, redundant fallback SQL server. Two other machines could be set up as
a web farm to handle the web requests. A fifth server could be added to the web farm,
or kept in reserve in case of a failure in either of the server blocks."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"How would you manage the state in this situation?",
                       Explanation = @"There does not appear to be any real special cases for state management, so an Out-
Proc solution in which IIS is configured to use SQL Server to manage sessions should
be acceptable. This would enable the application to send requests to any server in the
farm without a loss of state data. Typically, it is best to use the IIS built-in state management
systems where available because it frees your team from having to write code
that might be redundant."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"If two servers shut down at the same time because of hardware issues, and the problem took several days to resolve, how would your application be affected?",
                       Explanation = @"It depends on the two servers that were lost. Using at least two servers for the data tier
and the web tier should give you some contingency for hardware failures because it is
rare that more than one server goes out at a time. However, if two servers are lost at
the same time, the only real risk would be some downtime as you roll the fifth server
in to replace one of the ones that was lost. The only real data loss might be if both
servers in the database tier were lost, in which case it is likely that there will be some
data loss. If you lose one in each tier, or even both web servers in the farm, you can
provision the fifth server as a web server without any loss of data other than in those
requests that the server was processing as it went down."
                   }
               });


            // Section 1.5
            CurrentBackground = @"Design a caching strategy";

            MC("1.5 (1)", 
                @"You are designing a work order management application for a mid-size repair company. The application will be used by repair personnel in the field on their laptops with wireless phone connections. The connections are slow, and the laptops are several years out of date. There will never be more than 15 users at any one time, and rarely more than 2 concurrent users. What kind of caching will give the repair personnel a better user experience?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Page output caching",
                        IsCorrect = true,
                        Explanation = @"Page output caching will cache content at the client side to eliminate
some of the required downloads. It is useful in a limited bandwidth environment. It
can also be used in donut hole and donut caching scenarios for partial client-side
caching."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Application caching",
                        IsCorrect = false,
                        Explanation = @"Application caching is an HTML5 feature, and it is unlikely that the older
laptops will be able to support the feature."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Data caching",
                        IsCorrect = false,
                        Explanation = @"Data caching might decrease some server time, but with the limited
number of users, it is unlikely that the data access would be an issue."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HTTP caching",
                        IsCorrect = true,
                        Explanation = @"HTTP caching will help response time even though there is not much a
developer needs to do to implement the caching."
                    }
                });

            MC("1.5 (2)",
                @"You are creating a solution in which the majority of the application is dynamic, but some areas can be cached for a long time. What kind of approach would you take?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Data caching",
                        IsCorrect = false,
                        Explanation = @"Although data caching can add some support in a highly dynamic situation,
it does not support the capability to have long-term caching."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Donut hole caching",
                        IsCorrect = true,
                        Explanation = @"Donut hole caching provides the ability to cache parts of each page."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Donut caching",
                        IsCorrect = true,
                        Explanation = @"Donut caching is another approach that gives the ability to cache parts of
the application."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Windows AppFabric caching",
                        IsCorrect = false,
                        Explanation = @"AppFabric caching would provide some support in a highly dynamic
situation, but it does not suit the need to store some of the page output."
                    }
                });

            MC("1.5 (3)", 
                @"You are adding a reporting vertical to an enterprise application. Many of the reports will be run every morning by a set of users. Some of the reports will be identical as every member of a team will get the same report sometime in the morning. What kind of caching will provide an improvement in performance?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Data caching",
                        IsCorrect = true,
                        Explanation = @"Data caching with the appropriate timeout will enable the data needed
for the reports to be stored so that the call to the database is not necessary."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Page output caching with a duration of two minutes",
                        IsCorrect = false,
                        Explanation = @"Although a page output caching would be useful, the short time
frame of two minutes means that the cache will likely expire before the next user
requests the page."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Page output caching with a duration of four hours",
                        IsCorrect = true,
                        Explanation = @"A page output caching of four hours caches the output of the report
for the whole morning and should eliminate the need for the report to be run a
second time."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Windows AppFabric caching",
                        IsCorrect = true,
                        Explanation = @"AppFabric caching acts much like data caching to eliminate the need for
additional calls to the database to generate the reports."
                    }
                });



            MT("1.5",
               @"Improving the performance of an inventory management system",
               @"You are working at a company that has a custom inventory management system
that has been used for years. Company personnel recently completed an overhaul
of the web-based application that provides access into the system. Although they
are comfortable with the look, feel, and usability of the application, they are unhappy
with the performance, even after updating all their users to the most recent
hardware equipment and software. They find that every interaction with the server
is taking seconds. At times, this renders the application almost unusable. When you
look into the application, you see that all the models in the MVC structure do calls
into the database whenever they need data, even down to the level of the user’s
name, shift, and building, which are calls into different tables and displayed on
every page the user visits.",
               new TextboxAnswer[2]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"As you look into the system more carefully, you see 15 calls into the database for
every time a page is rendered that is strictly to provide basically static information
to the page. How could you use caching to improve this?",
                       Explanation = @"There are several ways that caching could help this process. The first is to get the information
from the database and store it in localStorage. That way you never need to call
the server again unless the client realizes it does not have the information any more.
You could also use donut caching or donut hole caching, whichever is more appropriate,
to cache that area of the page where the information doesn’t change. If you assign
a duration of 30 minutes, you decrease a lot of redundant database calls."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"After you have eliminated the redundant calls, you see that the application
makes calls into the database for every request to get supporting information
such as colors, product sizes, and box sizes. The application gets the complete
list from the database every time and then gets the necessary size, color, and so
on from that longer listing. How could caching be used to help this process?",
                       Explanation = @"Because the list of colors, sizes, and so on is the same for all users, you could store this
information in a data cache layer in which the system will make only one call into the
database every x minutes and will make that same set of returned information available
to all users of the system. This gives you an immediate performance gain across all
users."
                   }
               });


            // Section 1.6
            CurrentBackground = @"Design and implement a WebSocket strategy";

            SC("1.6 (1)",
                @"What is the technique in which the client sends a request to the server, and the server holds the response until it either times out or has information to send to the client is?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HTTP polling",
                        IsCorrect = false,
                        Explanation = @"In HTTP polling, the client sends a request to the server, and as soon as
the response is returned, it sends a new request."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HTTP long polling",
                        IsCorrect = true,
                        Explanation = @"In HTTP long polling, the client sends a request to the server, and the
server holds it open until it either has something to return to the client or the connection
times out."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"WebSockets",
                        IsCorrect = false,
                        Explanation = @"WebSockets are a way for two-way communication between the client
and the server. The server does not hold onto the response."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HTTP request-response",
                        IsCorrect = false,
                        Explanation = @"The request-response path is a traditional HTTP connection."
                    }
                });

            MC("1.6 (2)",
                @"You are building an application in which you want to display updated information to a website every 15 minutes. What are efficient ways to manage the update?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"WebSockets",
                        IsCorrect = true,
                        Explanation = @"WebSockets can be used to pass information between the client and server."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HTTP polling with 1-minute intervals",
                        IsCorrect = false,
                        Explanation = @"HTTP polling can provide the need, but the 1-minute refresh interval would not be efficient."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HTTP long polling",
                        IsCorrect = false,
                        Explanation = @"HTTP long polling is not a valid strategy. The typical timeout on a
single request is less than 15 minutes, and chaining multiple requests to get the
15-minute timespan is resource intensive."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HTTP polling with 15-minute intervals",
                        IsCorrect = true,
                        Explanation = @"HTTP polling with 15-minute intervals is a valid way to get the information
within the required time frame."
                    }
                });

            SC("1.6 (3)",
                @"What is the first request sent to start HTTP polling?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HTTP DELETE",
                        IsCorrect = false,
                        Explanation = @"HTTP DELETE is not used to start the WebSocket connection; it is instead
used to perform a delete on a discrete item."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HTTP GET",
                        IsCorrect = true,
                        Explanation = @"The first request to open a WebSocket connection is a standard HTTP
GET. After the request is received, the browser sends a separate upgrade request."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HTTP CONNECT",
                        IsCorrect = false,
                        Explanation = @"HTTP CONNECT converts the request connection to a transparent TCP/
IP tunnel."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Upgrade request",
                        IsCorrect = false,
                        Explanation = @"The upgrade request is sent after the server has responded to an HTTP
GET request."
                    }
                });


            MT("1.6",
               @"Using WebSockets for a communications application",
               @"You are a consultant hired by an insurance and financial services firm, to create an
online system for employees to get breaking financial news as well as to communicate
with each other. The company already has a news tracking system that stores
relevant news articles and sends an email with the content of the articles to some
of the fund managers. The company wants to include this information as well as an
instant messenger application and create a simple site for communications for all
employees.",
               new TextboxAnswer[3]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"Although HTML5 WebSockets appears to be a natural fit to fulfill these requirements,
what kind of issues could you run into?",
                       Explanation = @"The most common set of issues you would encounter when creating a solution that
includes WebSockets is the nonuniversal support for HTML5. It is possible aspects of
the company’s business still run on non-HTML5-compliant browsers. Other issues you
could encounter include proxy servers, firewall filters, and other security systems that
might look at nontraditional HTML communications as a threat."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"There appear to be two different communications processes going on. Would
you use a different socket connection for each one or share the same connection?
Explain your answer.",
                       Explanation = @"When following a traditional SoC route, the design should manage each different type
of communication separately, even though it might be on the same page. This would
give them the opportunity to change independently of each other, perhaps by moving
to a different server or even starting to take the news feed from a third-party service
directly."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What kind of server-side services do you have to create?",
                       Explanation = @"You need to create the server-side application that will be notified of news articles
and send the information to the users. You also need to create a server-side application
that will manage the instant messaging part of the application. Theoretically they
could be the same application, but it would be prudent to design them in such a way
that they could scale separately and independently."
                   }
               });


            // Section 1.7
            CurrentBackground = @"Design HTTP modules and handlers";

            SC("1.7 (1)",
                @"In an HTTP module, can you redirect the request to a different handler than is in the routing table? If so, what event would you handle?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Yes, and you handle the PostAuthorizeRequest event.",
                        IsCorrect = false,
                        Explanation = @"The PostAuthorizeRequest event is thrown before the handler is
mapped."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"No, after the request starts into the process, it either continues through to the mapped handler or throws an error.",
                        IsCorrect = false,
                        Explanation = @"You can handle the mapping of the request in the MapRequestHandler."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Yes, and you handle the MapRequestHandler event.",
                        IsCorrect = true,
                        Explanation = @"You handle the mapping of the request in the MapRequestHandler."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Yes, and you handle the ReleaseRequestState event.",
                        IsCorrect = false,
                        Explanation = @"The ReleaseRequestState is thrown after the handler has completed."
                    }
                });

            SC("1.7 (2)",
                @"When you are creating a custom handler, what is the parameter being passed into the ProcessRequest method?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"object, EventArgs",
                        IsCorrect = false,
                        Explanation = @"object, EventArgs are the parameters used for the event handlers
thrown during the startup process. The event handlers are assigned in the Init
method."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HttpApplication",
                        IsCorrect = false,
                        Explanation = @"HttpApplication is the parameter used in the Init method."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"HttpContext",
                        IsCorrect = true,
                        Explanation = @"The ProcessRequest method takes the HttpContext parameter."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Object",
                        IsCorrect = false,
                        Explanation = @"There are no default methods that just accept an object parameter."
                    }
                });

            SC("1.7 (3)",
                @"What is the best way to intercept every request for an image on your site and ensure that a watermark is added to the image?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"An HTTP module handling the AuthorizeRequest event",
                        IsCorrect = false,
                        Explanation = @"A module is not the best way to handle the request because it would
have to deal with every HTTP request rather than just the image calls."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"A custom HTTP handler set to handle .htm and .html pages",
                        IsCorrect = false,
                        Explanation = @"Serving .htm and .html pages will not create watermarks on image files."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"A custom HTTP handler configured to serve .png and .jpg files",
                        IsCorrect = true,
                        Explanation = @"Intercepting every request for .jpg and .png files is the easiest way to
consistently add watermarks to the images."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"An HTTP module handling the PostRequestHandlerExecute event",
                        IsCorrect = false,
                        Explanation = @"A module is not the best way to handle the request because it would
have to deal with every HTTP request rather than just the image calls."
                    }
                });


            MT("1.7",
               @"Using HTTP handlers and HTTP modules as services",
               @"You have been asked to create a set of web services. They will not be standard web
services; they will be based entirely on HTTP modules and HTTP handlers. These
services will be REST-based and need to support authentication.",
               new TextboxAnswer[3]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"What would be the most standard way to use HTTP modules and HTTP handlers
to fill this need?",
                       Explanation = @"Creating an HTTP handler is a relatively simple way to create a customized process
to return XML or JSON return objects. Using it in a RESTful scenario is more complicated
because there is no extension to also map the handler. You have to manage all
requests without an extension and then filter the URL request to see what the appropriate
response would be."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"If you needed to add custom authentication, where would be the best place to
put that functionality?",
                       Explanation = @"The AuthenticateRequest and AuthorizeRequest events are the traditional access points
for authorization and authentication. You add the event handlers in the Init method
and you have access to the entire HTTP Request in the module as it moves through the
application stack."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Do you think that creating web services in handlers and modules would result in
a responsive application, or do you think performance would suffer? Why?",
                       Explanation = @"It would be a relatively responsive application, especially when comparing it to traditional
ASP.NET MVC applications. Because it would use its own custom handler, a lot of
the overhead of MVC would be left out of the process."
                   }
               });



            ///////////////////////// Section 2 ///////////////////////


            // Section 2.1
            CurrentBackground = @"Apply the user interface design for a web application";


            MC("2.1 (1)", @"Layout.cshtml and Site.Master are the two default template pages in ASP.NET MVC. 
Which of the following scenarios would best be solved using a single layout or master template?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Your application has a requirement to display a menu section that changes based on the area of the application the user is visiting.",
                        IsCorrect = true,
                        Explanation = @"Adding logic to the master page to determine the menu design enables
your application to display a menu section that changes based on the area of the
application the user is visiting."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Each content area on your page needs a header that displays the company’s branded color and contains the first 40 characters of the content area’s content followed by an ellipsis.",
                        IsCorrect = false,
                        Explanation = @"The master page does not have the capability to “look” into the content
sections and select the first 40 characters."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You have created a set of styles, each in a different style sheet. The styles need to be available to every page in the application.",
                        IsCorrect = true,
                        Explanation = @"Referencing all style pages in a single place, the layout page, is an
appropriate use."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Your application has three different default page designs: two rows of information, two columns of information, and three columns of information.",
                        IsCorrect = false,
                        Explanation = @"A master page does not control the design of the content being
displayed."
                    }
                });


            SC("2.1 (2)", @"You are designing a web application. You want to create a certain look and feel while
reusing styles across pages as much as possible. How should you handle styles?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use only one or two styles throughout your application to simplify maintenance.",
                        IsCorrect = false,
                        Explanation = @"It is unlikely that one or two styles will enable you to create the look
and feel you want."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use a specific (unique) style for every element.",
                        IsCorrect = false,
                        Explanation = @"Using a specific style for each element will limit your ability for style
reuse."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use general styles for common elements and specific styles for elements that are
unique.",
                        IsCorrect = true,
                        Explanation = @"You should use general styles as much as possible. Achieving a certain
look and feel means that you might have to create some specific styles, however."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use inline styling.",
                        IsCorrect = false,
                        Explanation = @"Although inline styling will give you the most, control over styling,
it removes the possibility of code reuse. Design changes would be difficult to
implement."
                    }
                });


            MC("2.1 (3)", @"What are compelling reasons to switch from static web content to dynamic web content?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The ability to substitute a new image for the company logo",
                        IsCorrect = false,
                        Explanation = @"There is no need to create a dynamic website simply to replace the
company logo."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The ability to display information from a database",
                        IsCorrect = true,
                        Explanation = @"Displaying data from a database is an ideal reason to use a dynamic
website."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The ability to link to other pages outside of your application",
                        IsCorrect = false,
                        Explanation = @"Linking to other pages outside of an application does not require a
dynamic website."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The ability to display information pertaining to the current user",
                        IsCorrect = true,
                        Explanation = @"Personalization, or displaying information about the user on-screen, is an
example of dynamic content."
                    }
                });


            MT("2.1",
               @"Incorporating dynamic content into a website",
               @"Your company wants to update its website, which was built using ASP.NET MVC. The website is a portal for independent musicians to upload their recordings. Both registered and unregistered visitors can listen to music and provide feedback and comments. The main layout was done using tables to position elements within the page.",
               new TextboxAnswer[2]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"How can a webpage be rendered with dynamic content depending on the type of user?",
                       Explanation = @"The decision to use dynamic content can add functionality to the view to check for
user information. The application decides what to display based on user information,
which is what makes the content dynamic."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Some of your potential visitors do not have browsers that support HTML5. How will that affect your design choice?",
                       Explanation = @"Because HTML5 is not yet supported by all browsers, some elements might not be able
to use certain styles, resulting in an experience different from what you expect. If you
use HTML5-specific tags in your application, such as <audio> or <video>, you should
have a fallback for displaying and managing the information."
                   }
               });


            // Section 2.2
            CurrentBackground = @"Design and implement UI behavior";


            SCC("2.2 (1)", @"You are creating an ASP.NET MVC web application. The application must accept user input for a ProductName field. To reduce delays due to invalid entries making round trips between the client and server, user input should be validated on the client before being submitted to the server. Which code segment should you choose?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"<div class=”editor-label”>
    @Html.LabelFor(model => model.ProductName)
</div>
<div class=”editor-field”>
    @Html.EditorFor(model => model.ProductName)
</div>",
                        IsCorrect = false,
                        Explanation = @"@Html.EditorFor only inserts data. The data is verified on the server
side."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"<div class=”editor-label”>
    @Html.LabelFor(model => model.ProductName)
</div>
<div class=”editor-field”>
    @Html.ValidationMessageFor (model => model.ProductName)
</div>",
                        IsCorrect = false,
                        Explanation = @"@Html.ValidationMessageFor displays the validation message. There is
no code included in this answer choice to validate the data."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"<div class=”editor-field”>
    @Html.EditorFor(model => model.ProductName)
    @Html.ValidationMessageFor(model => model.ProductName)
</div>",
                        IsCorrect = true,
                        Explanation = @"@Html.EditorFor, in combination with @Html.ValidationMessageFor, are
used for client-side validation."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"<div class=”editor-label”>
    @Html.LabelFor(model => model.ProductName)
</div>",
                        IsCorrect = false,
                        Explanation = @"@Html.LabelFor only displays labels for the items."
                    }
                });


            SC("2.2 (2)", @"Your team is building an application and you are reviewing the functional specifications.
Your team must include a stockticker in the UI that displays the company’s stock
price every 15 minutes, and include the capability to do partial saves of base objects as
users step through a data entry wizard. You want to use the same approach for both
requirements to make it easier to add functionality and maintain it going forward.
What approach should you use?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use JavaScript to refresh the page every 15 minutes and to manage whether wizard
buttons are enabled or disabled.",
                        IsCorrect = false,
                        Explanation = @"This approach will update the stockticker, but enabling and disabling
the wizard buttons will not ensure that the base objects are saved."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use AJAX to make asynchronous calls to the server on a timer for the stock price
and to automatically save the base objects as the user navigates through the
wizard.",
                        IsCorrect = true,
                        Explanation = @"The best way to solve this issue is to use AJAX to do asynchronous calls to
check the stock price as well as manage the save process through the wizard."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use jQuery to refresh the page every 15 minutes and to manage whether wizard
buttons are enabled or disabled.",
                        IsCorrect = false,
                        Explanation = @"This approach will update the stockticker, but enabling and disabling
wizard buttons will not ensure that the base objects are saved."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use data validation annotations on the model to ensure that the stock price is validated
every 15 minutes and that the client saves the base object information after
every wizard step.",
                        IsCorrect = false,
                        Explanation = @"Data validation annotations will not meet any of the requirements."
                    }
                });


            MC("2.2 (3)", @"In which of the following scenarios should you include validation?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You have an online diary with two form fields, a title, and a large subject box. A
title is not required, and the subject content is stored in a database column with no
maximum size.",
                        IsCorrect = true,
                        Explanation = @"You should build in validation to check that at least one field has been
populated before saving an entry."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The web application you maintain has an area that serves as a pass-through to
another company’s web services. The form contains personal information, such
as address and phone number, and is used to set up a profile on the company’s
retirement partner website. The partner has never given you any instructions as to
what is or is not required to be sent to them.",
                        IsCorrect = true,
                        Explanation = @"Although the partner company has not provided any requirements, your
company is responsible for data entry and therefore should ensure that the data
passing through the application meets some minimum criteria."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Your application is a long wizard that college students use to apply for financial
aid. They do not have access to the application until they are already logged on to
the system so the application knows who they are. Most students will log on many
times to finish the application, so any field might or might not be completed at any
time.",
                        IsCorrect = true,
                        Explanation = @"The application has many entry fields so it is difficult to predict when an
entry hits a valid stage. However, you already know you have constraints on the
data being input because of the size of the database columns in which you will be
storing them. A MaxLength validator on each field would help ensure that there is
no loss of data."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You are developing a simple form that helps home brewers keep track of their
process. The form provides two input fields: Date/time and ph level. The Date/
time box needs to be an ordinary text box because people around the world might
enter the date differently, in a way that is meaningful to them. The ph level can be
either a numeric value or a text description.",
                        IsCorrect = true,
                        Explanation = @"Although either field in the form can be anything, they should have a
maximum length limit imposed so the data does not exceed the size of the database
column they will be stored in. You should also validate that neither field is empty."
                    }
                });


            MTC("2.2",
               @"Modifying a web application for data validation and responsiveness",
               @"You want to update an ASP.NET MVC web application, and you decided to use jQuery to make the website feel more fluid and easier to develop. The application has various fields, including a memo field, an email field, and a date field. The application also uses a set of buttons that makes up the navigation pane. When navigating, the client must requery the server each time, which makes the application feel less responsive.

When thinking of answers to these questions, keep in mind that JavaScript alone might not be able to fix the situation.",
               new TextboxAnswer[3]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"How can you prevent users from entering incorrect dates in a date-only field?",
                       Explanation = @"Use the jQuery date widget."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"How can you preload information and display it as needed?",
                       Explanation = @"You can convert from the list of buttons to tabs, keeping the look and feel the same.
You can then preload the information by using the jQuery load function and place the
information in the appropriate tab that represents what used to be a separate page.
Taking this route preloads the page so that when a user selects a tab, the content has
already been loaded."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"When querying for a product, if the product is not found, how can you update
a portion of the search screen rather than creating a pop-up to notify the end
user?",
                       Explanation = @"Use AJAX or jQuery to update the same page if the result is not found."
                   }
               });


            // Section 2.3
            CurrentBackground = @"Compose the UI layout of an application";


            SCC("2.3 (1)", @"You are creating an ASP.NET MVC web application. Within the application, you have created a partial view for contact email and phone number. Which code segment should you use to display the partial view on the main page?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"<div class=”float-right”>
    <section id=”contact”>
        @Html.ActionLink(“ContactPartial”)
    </section>
</div>",
                        IsCorrect = false,
                        Explanation = @"@Html.ActionLink creates a link but does not load a partial view."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"<div class=”float-right”>
    <section id=”contact”>
        @Html.Partial(“ContactPartial”)
    </section>
</div>",
                        IsCorrect = true,
                        Explanation = @"@Html.Partial loads a partial view."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"<div class=”float-right”>
    <section id=”contact”>
        @RenderPage(“ContactPartial”)
    </section>
</div>",
                        IsCorrect = false,
                        Explanation = @"@RenderPage method inserts one complete page into another This
is not what you are looking to do as you only want the partial view content to be
displayed."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"<div class=”float-right”>
    <section id=”contact “>
        @RenderBody()
    </section>
</div>",
                        IsCorrect = false,
                        Explanation = @"@RenderBody inserts views on master layout pages."
                    }
                });


            SCC("2.3 (2)", @"You are creating an ASP.NET MVC web application. The application accepts phone number input through the application’s form. When viewing the source from a browser, you find the following code:

PhoneNumber: <input id=”phoneNumber” name=”phoneNumber” size=”10”
type=”text” value=”” /><br />

What Razor syntax code segment was used?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"PhoneNumber: <input id=”phoneNumber” name=”phoneNumber” size=”10” type=”text” value=”3125551212” /><br />",
                        IsCorrect = false,
                        Explanation = @"The Value construct sets the display information in the element. In addition,
the field is not bound to the model."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"<div class=”editor-field”>
    @Html.EditorFor(model => model.PhoneNumber)
</div>",
                        IsCorrect = false,
                        Explanation = @"This will not validate or set the size requirement."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"PhoneNumber: @Html.TextBox(“phoneNumber”, Request[“phoneNumber”], new { @placeholder = “3125551212”})<br />",
                        IsCorrect = false,
                        Explanation = @"This will make the input field display with a placeholder."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"PhoneNumber: @Html.TextBox(“phoneNumber”, Request[“phoneNumber”], new { size = 10 })<br />",
                        IsCorrect = true,
                        Explanation = @"This is the proper way to limit the size of a certain field that is being
bound to the model."
                    }
                });


            SCC("2.3 (3)", @"You are modifying an ASP.NET MVC web application and you have created a new master layout page named _Layout.WindowsPhone.cshtml. You want to use that layout in a new view. Which code segment do you use?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"@Html.ActionLink(“_Layout.WindowsPhone.cshtml”);",
                        IsCorrect = false,
                        Explanation = @"@Html.ActionLink creates a clickable link and does not do anything to
set the layout."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Layout=”~/Views/Shared/_Layout.WindowsPhone.cshtml”;",
                        IsCorrect = true,
                        Explanation = @"Layout=”” loads the layout file to be used with that view."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Layout=”Layout.WindowsPhone.cshtml”;",
                        IsCorrect = false,
                        Explanation = @"Layout=”Layout.WindowsPhone.cshtml“ points to an incorrect view
folder."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"@Html.Partial(“_Layout.WindowsPhone.cshtml”);",
                        IsCorrect = false,
                        Explanation = @"@Html.Partial loads. a partial view, but does not manage the layout
being used by the page."
                    }
                });


            MTC("2.3",
                @"Updating a web application for usability",
                @"Your office hosts its internal asset tracking database using an ASP.NET MVC application. To find contact information for an employee from the Inventory screen, a user must exit the Inventory screen and launch a User screen. In addition, the current process takes the user to a separate login screen, but the IT department wants to be able to log in and out of the application quickly. Your IT manager wants you to update the application to improve usability.

When thinking of answers to these questions, keep in mind that JavaScript alone might not be able to fix the situation.",
               new TextboxAnswer[2]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"What can you modify to enable staff to switch screens?",
                       Explanation = @"Use AJAX or jQuery to load a partial view that lets you select users and view their
information."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What can you modify to achieve quick login and logout functionality?",
                       Explanation = @"Insert a login/logout partial view on top of the master layout. This partial view enables
you to log in and log out from every screen in the application that uses that master
layout page."
                   }
               });


            // Section 2.4
            CurrentBackground = @"Enhance application behavior and style based on browser feature detection";

            MC("2.4 (1)", @"You want to support the Internet Explorer, Firefox, and Opera web browsers in your application. Which vendor-specific extensions do you need to include with CSS3 properties?",
                  new CheckboxAnswer[]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"-webkit-",
                        IsCorrect = false,
                        Explanation = @"The -webkit- prefix is used for Google Chrome and Apple Safari."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"-ms-",
                        IsCorrect = true,
                        Explanation = @"The -ms- prefix is used for Internet Explorer."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"-o-",
                        IsCorrect = true,
                        Explanation = @"The -o- prefix is used for the Opera browser"
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"-hp-",
                        IsCorrect = false,
                        Explanation = @"The -hp- prefix is used by Hewlett-Packard."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"-moz-",
                        IsCorrect = true,
                        Explanation = @"The -moz- prefix is used for Mozilla Firefox."
                    }
                });


            MC("2.4 (2)", @"What are common methods for detecting the type of browser running on a client?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use JavaScript to query for the userAgent header.",
                        IsCorrect = true,
                        Explanation = @"Using JavaScript to query the userAgent gives you information about the
type of browser being used by the client."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the window.addEventListener method.",
                        IsCorrect = false,
                        Explanation = @"The window.addEventlistener does not give any information on the
browser being used by the client, but it can be used to see whether a browser is
HTML5-compliant."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the viewport <meta> tag.",
                        IsCorrect = false,
                        Explanation = @"The viewport <meta> tag gives access to the visible area of the device;
it does not tell you anything about the device itself."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the DisplayMode provider.",
                        IsCorrect = true,
                        Explanation = @"The display mode provider performs some of the analysis of the HTTP
request to try and determine what kind of browser made the request."
                    }
                });


            SC("2.4 (3)", @"You are creating a different view for each of several different browsers/devices, such as
Home.iemobile.cshtml and Home.IPad.cshtml. What is the best way to implement it so
your application knows to look for the specific views?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the viewport <meta> tag.",
                        IsCorrect = false,
                        Explanation = @"The viewport <meta> tag does not do any direction to views; it is
strictly a client-side helper."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add a new DisplayModeProvider for each of the special view types you want to
support.",
                        IsCorrect = true,
                        Explanation = @"Adding a DisplayModeProvider for each type of special view, such as
IEMobile or IPad, informs the framework to use those views where the context
condition is fulfilled."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Put logic into each action to select the appropriate view based on information in
the request.",
                        IsCorrect = false,
                        Explanation = @"Putting logic into each action could be done, but it would be timeconsuming
and difficult to maintain. The DisplayModeProvider does the work for
you in the background and eliminates the need for special code."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You don’t have to do anything. The framework automatically handles browser/device
detection by reading the new extensions on the views.",
                        IsCorrect = false,
                        Explanation = @"The framework does not make any assumptions by itself; it only knows
what to do based on configurations that it has been given."
                    }
                });


            MT("2.4",
               @"Enhancing an application based on browser features",
               @"Your team has been hired to develop an ASP.NET MVC application for an insurance agency. The agency has both office staff and sales staff. Office staff members use Microsoft Surface tablets and sales personnel use iPads. When visiting with clients and prospects onsite, sales personnel must be able to use their devices to check in with the main office to place orders and ask questions of the office staff. The office staff needs to be able to respond to questions, but not ask questions. The applica-tion should also maximize screen real estate on both types of devices.",
               new TextboxAnswer[3]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"What should you do to ensure that both types of devices can use the application?",
                       Explanation = @"The main way to manage this requirement is to use display mode providers, one for
the iPads and the other for the Windows tablets. You must create a different set of
views based on each device. Using the @media query is complicated because both
types of devices are tablets with similar screen sizes. It is unlikely the viewport will be
different for each."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What approach should you take to provide different functionality based on device?",
                       Explanation = @"If you use the different views based off the display mode providers, you can design
a completely different experience for users on each device. You can make different
menus, use different colors—basically design completely different experiences."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What kinds of issues could you run into when making functionality issues based on device?",
                       Explanation = @"There are several problems with making the determination of functionality based on
the type of the device being used. The first is that you lock in a user type to a device
type; for example, a salesperson who breaks or loses his iPad could not work until he
has another iPad. The second problem is that this kind of decision should really be by
user type, not by device."
                   }
               });


            // Section 2.5
            CurrentBackground = @"Plan an adaptive UI layout";


            MCC("2.5 (1)", @"You are creating an ASP.NET MVC web application, and you decide to create a new layout for mobile devices with relatively small screens. Which @media query should you create or modify to accomplish the task?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"@media (max-width:768px)",
                        IsCorrect = true,
                        Explanation = @"This allows for devices with a maximum width of 768 pixels."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"@media (max-width:478px) and (orientation:portrait)",
                        IsCorrect = true,
                        Explanation = @"This allows for devices with a maximum width of 478 pixels in portrait
view, such as tablets."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"@media (min-width:768px) and (orientation:portrait)",
                        IsCorrect = false,
                        Explanation = @"The screen resolution is too large for a typical mobile device."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"@media (min-width:1200px)",
                        IsCorrect = false,
                        Explanation = @"The screen resolution is too large for a typical mobile device."
                    }
                });


            MC("2.5 (2)", @"You are modifying an existing ASP.NET MVC web application to incorporate mobile access. What should you do to ensure that mobile devices view mobile-only pages?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Run the Install-Package jQuery.Mobile.MVC command using the Package Manager console.",
                        IsCorrect = true,
                        Explanation = @"This installs all the necessary mobile packages."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a _Layout.Mobile.cshtml master layout page.",
                        IsCorrect = false,
                        Explanation = @"You need the additional views made for mobile calling the master
layout."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create additional views with .Mobile, such as Index.Mobile.cshtml.",
                        IsCorrect = false,
                        Explanation = @"Just the views alone do not work; you need the master layout as well."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create additional views with .Mobile, such as Index.Mobile.cshtml as well as a _Layout.Mobile.cshtml master layout page.",
                        IsCorrect = true,
                        Explanation = @"All mobile views and mobile master layout ensure that the page will load
accordingly."
                    }
                });


            MC("2.5 (3)", @"You are modifying an ASP.NET MVC web application for a client. The client requires that the application must be viewable on Android devices in a UI format native to the device. What should you do?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Run the Install-Package jQuery.Mobile.MVC command using the Package Manager console.",
                        IsCorrect = false,
                        Explanation = @"Simply installing the jQuery Mobile package does not provide support
for Android-specific browsers. It enables the application to use the package."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a viewport and use @media queries to make styles that are specific for Android.",
                        IsCorrect = false,
                        Explanation = @"By using @media queries and viewport, you can create an Androidreadable
website, but it won’t give an Android-specific UI."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create corresponding mobile views for the project targeted toward Android.",
                        IsCorrect = true,
                        Explanation = @"Additional views must be created or ported to fit the smaller layout."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Modify the Web.config file in Shared views in the project to add in support for other browsers.",
                        IsCorrect = false,
                        Explanation = @"You do not have to modify the Web.config file."
                    }
                });


            MT("2.5",
               @"Incorporating mobility and security into a web application",
               @"Your team is modifying an ASP.NET MVC application for an automotive dealership.
The company issued Windows phones to sales staff. Office staff members access the
application via their desktop computers to perform data entry, generate reports,
and handle other administrative functions. The mobile devices will be used to
search inventory and to check cars in and out for test drives. The application must
also be viewable in a native user experience relative to the devices.",
               new TextboxAnswer[2]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"How can you ensure that both types of users have a positive user experience?",
                       Explanation = @"Two separate user groups—admin users and sales users—use two different types of
devices to access the application. You can safely assume that admin users, or users
from any desktop computer should have access to all functionality, whereas mobile
device users need access to only a limited set of functionality. Therefore, you could
start with separate, overridden views because they use different navigation schema.
After you separate mobile users into their own template, you could use jQuery Mobile
to create the UI while working in the mobile section of code."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Would adding two more types of mobile clients, such as Android tablets and
iPads, to the list of supported devices change your approach? Why or why not?",
                       Explanation = @"When developing a mobile app strategy, remember that mobile apps constantly
evolve, and new mobile devices are available constantly. By using a jQuery Mobile
approach without any specific templates for each mobile device, the framework makes
many decisions for you. Adding supported devices should be almost free for the developer,
outside of testing efforts."
                   }
               });


            /////////////// Section 3 - Develop the user experience ///////////////////

            // Section 3.1
            CurrentBackground = @"Plan for search engine optimization and accessibility";


            SC("3.1 (1)", @"You have been asked by the marketing department to help improve your company’s ranking in search engine results. They are particularly concerned about a section of the site that is highly interactive, with extensive mouse-over color, background, and text changes. Without looking at the code, what is the most likely reason for ranking low in search engine results?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Unclosed HTML tags",
                        IsCorrect = false,
                        Explanation = @"Although the pages might contain unclosed HTML tags, the highly
interactive nature of the site indicates that the content could be in script files, and
therefore hidden from the search crawler."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Content hidden in JavaScript tags",
                        IsCorrect = true,
                        Explanation = @"Text revealed by mouse-overs not being picked up by search engine
crawlers indicates that content is being skipped."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Broken links",
                        IsCorrect = false,
                        Explanation = @"Broken links do not slow down a crawler."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Excessive number of images",
                        IsCorrect = false,
                        Explanation = @"The presence of images does not affect the search engine crawler."
                    }
                });



            MC("3.1 (2)", @"You have been asked by a company to evaluate its web application as a user. The company wants you to review the website from outside its network and identify areas that might be problematic. You have been asked to consider SEO and accessibility. Which tools would be helpful for your review?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"IIS SEO Toolkit",
                        IsCorrect = false,
                        Explanation = @"The IIS toolkit requires access to the server, and your charge is to perform
the assessment from outside the client’s network."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Internet Explorer F12 tools",
                        IsCorrect = true,
                        Explanation = @"The Internet Explorer F12 tool enables you view HTML and CSS code,
which can help you detect structural problems or errors that might affect accessibility
or SEO."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"IIS Logging tab",
                        IsCorrect = false,
                        Explanation = @"The IIS Logging tab displays logs related to functioning of a website
and also requires access to the server."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Bing Webmaster Toolkit",
                        IsCorrect = false,
                        Explanation = @"The Bing Webmaster Toolkit requires access to the server to put a
special file on to the file system so that Bing can tie a Webmaster account to a
particular site."
                    }
                });


            MC("3.1 (3)", @"Your company has joined an industry accessibility group and you are a member. As one of your responsibilities, you have been asked to estimate the time it would take to modify your corporate site for accessibility. What should you consider to properly estimate the required time?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The amount of text in the website",
                        IsCorrect = false,
                        Explanation = @"The amount of text shouldn’t affect the amount of accessibility work to
be performed. The work will be the same whether there is one word in the ARIAcompliant
element or 100 words."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The level of current usage of HTML helpers in your views",
                        IsCorrect = true,
                        Explanation = @"Because ASP.NET MVC 3 HTML helpers are not ARIA compliant, you need
to write custom HTML helpers or use a different process to create HTML forms."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The numbers of controllers in the application",
                        IsCorrect = false,
                        Explanation = @"The number of controllers does not affect your need to make the rendered
HTML ARIA compliant."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The complexity of your data entry forms",
                        IsCorrect = true,
                        Explanation = @"Because many of the problems with accessibility are related to giving
context to content, complex forms need a thorough review to ensure that labels
and section context are provided for every element on the page."
                    }
                });


            MTC("3.1",
               @"Improving a website’s SEO and accessibility",
               @"Your team has created a blog site based on ASP.NET MVC. The site is not ranking well in Bing searches.

Consider various tools with overlapping feature sets in case one tool misses an issue.",
               new TextboxAnswer[3]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"What tool can you use to detect major errors in your site and correct them to
improve search engine crawling?",
                       Explanation = @"Run the IIS SEO Toolkit or a similar tool."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"If you implement ARIA in your application, what effect would it have on your
search engine ranking?",
                       Explanation = @"When properly implemented, ARIA should not have any effect on search engine rankings.
All markup occurs in attributes within the HTML elements."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"How can you check which parts of the site are slowing down when downloading?",
                       Explanation = @"The Internet Explorer Developer Toolbar is one of several tools you can use to evaluate
the performance of a website. With the toolbar, you can see every HTTP call made
from the browser and determine how long each call takes."
                   }
               });


            // Section 3.2 
            CurrentBackground = @"Plan and implement globalization and localization";

            MC("3.2 (1)", @"You are globalizing an application and are compiling a list of items that need to be
translated. Which items should be included on the list?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Images that are used as part of the menu structure",
                        IsCorrect = true,
                        Explanation = @"Images that are being used for menus likely have text displayed on them.
That text should be localized, and extra sets of buttons might have to be created."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The company logo",
                        IsCorrect = false,
                        Explanation = @"A company logo should not be changed because it provides brand
awareness."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Server error-logging messages",
                        IsCorrect = false,
                        Explanation = @"Server error-logging messages are not seen by the user and are instead
an internal item. You should keep them in the default language."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Button tooltips",
                        IsCorrect = true,
                        Explanation = @"Tooltips are supposed to give extra contextual help about the item the
mouse hovers over. It is important to get them translated."
                    }
                });


            SCC("3.2 (2)", @"You are creating an ASP.NET MVC website and you want to replace a <h2>hello</h2>
tag within the view with custom resources you have created. What should you use?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"<h2>@mvcapp.Resources.Home.Index.Heading</h2>",
                        IsCorrect = true,
                        Explanation = @"This is the correct way to insert resources into a view."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"<h2 div=”resource”>Hello</h2>",
                        IsCorrect = false,
                        Explanation = @"The <div> tag cannot be used to insert resources into a view."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"<h2 class=”resource”>Hello</h2>",
                        IsCorrect = false,
                        Explanation = @"This attempts to style the element, looking for a style named “resource.”"
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"<h2>@mvcapp.Resources.Home.Index.Heading Hello</h2>",
                        IsCorrect = false,
                        Explanation = @"Although this inserts resources into the view, it also adds Hello to the
end of the header tag."
                    }
                });

            SCC("3.2 (3)", @"You are developing an ASP.NET MVC application and you need to create satellite assemblies from resource files. What code should you use?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"mage.exe /n: /embed:strings.de-DE.resources MyApp.de-DE.resources.dll",
                        IsCorrect = false,
                        Explanation = @"Manifest Generation and Editing Tool is for creation and editing of application
manifests."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"WinRes.exe /t: embed:strings.de-DE.resources /culture:de-DE",
                        IsCorrect = false,
                        Explanation = @"Windows Form Resource Editor is a visual layout tool for the Windows
Forms user interface."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Lc.exe strings.de-DE.resources MyApp.de-DE.resources.dll",
                        IsCorrect = false,
                        Explanation = @"License Compiler reads text license files and compiles them into binary
format."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"al.exe /t:lib /embed:strings.de-DE.resources /culture:de-DE /out:MyApp.de-DE.resources.dll",
                        IsCorrect = true,
                        Explanation = @"Assembly Linker generates modules or resource files."
                    }
                });


            MT("3.2",
               @"Localizing an application",
               @"You are developing an ASP.NET MVC application that monitors and tracks exercise
and diet. The application will be released simultaneously in North America and
Europe. Because of the flexibility of the application, you estimate the application
will have more than 50 data entry and reporting screens. The application will be
released in English, Spanish, French, German, Dutch, Italian, and Danish.",
               new TextboxAnswer[2]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"Would separate views or resource files be the easiest way to implement and
maintain translations?",
                       Explanation = @"Using separate views to maintain translations would be problematic because there are
more than 350 views to support the initial launch. Although translating a view is easier
than translating a resource file, the creation and maintenance of the additional views
would be a lot of additional work because the view gives context to the translator."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"You want to use AJAX UI items in various parts of the application. How does that
affect your application’s design as it relates to globalization?",
                       Explanation = @"Using jQuery in the context of AJAX requires some additional work because you have
to add JavaScript globalization to the mix. Although you can use traditional resource
files for any content created on the server side, you need to include the globalization
package in your solution, have the appropriate translated versions of the culture file
available, and provide enough information to the browser about the client’s culture so
the JavaScript globalization features can determine which translated file to display."
                   }
               });


            // Section 3.3
            CurrentBackground = @"Design and implement MVC controllers and actions";


            SC("3.3 (1)", @"You have a set of requirements that expects a particular action to be accessed only through HTTPS by an authenticated user. What is the best way to meet these requirements?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the Authorize attribute before the RequireHttps filter.",
                        IsCorrect = false,
                        Explanation = @"Although this would solve the requirements, it is not the best way. It is
better to run the RequireHttps filter first because it expends fewer resources than
the Authorization filter."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the RequireHttps attribute before the Authorize filter.",
                        IsCorrect = true,
                        Explanation = @"It filters out those users not on an SSL connection before performing the
more resource-intensive authentication check."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use a custom action filter that combines the check for HTTPS and the check for authentication into a single filter.",
                        IsCorrect = false,
                        Explanation = @"A custom action filter that performs both is more complicated to write
than using the standard filters provided with ASP.NET MVC."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use Authorize alone; you do not need to use RequireHttps separately if you already have Authorize, because using Authorize implies the use of HTTPS.",
                        IsCorrect = false,
                        Explanation = @"A user can be authorized and not be connected over HTTPS. The two
are independent and need to be treated as such."
                    }
                });


            SC("3.3 (2)", @"You have written a shareware application and want to sell it on your personal website. You are considering various approaches for distributing the application after users purchase it. Which of the following is the best approach?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use the FileResult action result to initiate the file download.",
                        IsCorrect = true,
                        Explanation = @"The FileResult property was designed for this need."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Convert the application to an encoded string and provide it for download through the ContentResult.",
                        IsCorrect = false,
                        Explanation = @"The product being distributed is a binary file and does not successfully
transition to a string format."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a view or partial view that contains an action link to a route in which the
user can download the file.",
                        IsCorrect = false,
                        Explanation = @"Creating a link to download the file through the application just
postpones the decision until that link is clicked as it is still going through the
application."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create an email containing a link to a route from which the user can download the
file.",
                        IsCorrect = false,
                        Explanation = @"Creating a link to download the file in an email just postpones the decision
until that link is clicked as it is still going through the application."
                    }
                });


            SC("3.3 (3)", @"You have been tasked with modernizing an application created in ASP 3.0. Part of one
of the pages contains HTML that comes as a string from a third-party application as
part of a service call that populates part of a form. The same form contains input values
for your application. You need to support the same business process as the original
application, but you also want to use some MVC features. What is the approach?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a single model containing information from your local application. Use
strongly-typed binding as much as possible and manually match the rest of the
fields.",
                        IsCorrect = false,
                        Explanation = @"Although this could conceivably work, it has the misfortune of expecting
manual mapping. It also uses the approach of a single model, and because the
provided HTML input forms might change under you, it makes more sense to use
two models so that the information your application controls is not affected."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a single model for the local input fields and the service call input, and use
ToValueProvider to map the entire object.",
                        IsCorrect = false,
                        Explanation = @"Although this would work, it uses the single model approach. Because
the provided HTML input forms can change outside of your own release cycle,
it makes more sense to use two models so that the information your application
controls is not affected."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a single model. Use weakly-typed binding for the form fields, and the HTML
provided by the third-party application.",
                        IsCorrect = false,
                        Explanation = @"You cannot weakly bind to the input fields provided from the third
party because weakly-bound models imply that you have used an HTML helper to
write the information, and you are just giving the helper a hint to the property it
should map to in the model."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a model that contains only your fields and strongly bind the fields to the
model. Create a second model that maps to the fields in the imported HTML and
bind to that model using ToValueProvider.",
                        IsCorrect = true,
                        Explanation = @"The key is separating your data input fields from the provided input
fields. You can strongly-bind to yours because you have full control over the relationship
between your model and your view, and can then use the ToValueProvider
to merge the other model that is tied to the provided input fields."
                    }
                });


            MT("3.3",
               @"Adding role-based permissions",
               @"You are updating an existing MVC blog site. You are told to add role-based permissions
to the site.",
               new TextboxAnswer[3]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"The manager wants to add information about currently logged-in users to the
top of each webpage. Can a nonregistered user and registered user experience
the site the same way? Why or why not?",
                       Explanation = @"Yes, the layout of the site can remain the same. You have to look at two areas: view and
controller. The view needs the capability to display default placeholder information
for nonregistered users. Use the controller to authenticate a user. Pass the information
back to the view and show the required information for registered and logged-in users.
As long as the design is the same with some different options in a similar place for
each type of user, the overall user experience is much the same."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"You want to ensure that your site is viewable by all but allow only authorized users
to edit. How can you accomplish this?",
                       Explanation = @"To allow for unauthenticated users to view, you need to ensure that there are no
Authorize attributes set on the action. Anywhere authentication is needed, use the
Authorize attribute to filter out unauthenticated users. You can even redirect the page
to a login page so that a person with credentials can log in and post the information as
needed."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"You have updated the site but can no longer write new blogs, although you can
edit existing blogs. You are logging in as the Administrator role. What could be
the problem?",
                       Explanation = @"Although your roles are set up and you are correctly authenticated, if the information
being returned misses required information, the model state is no longer valid. In that
case, make sure that your model state is valid by manipulating the data before you
insert the object."
                   }
               });


            // Section 3.4
            CurrentBackground = @"Design and implement routes";


            SCC("3.4 (1)", @"You are developing an ASP.NET MVC application. You have a set of requirements to create a help section for remote users. Your typical help scheme is help/desktop or help/mobile, so logically this section should be help/remote. The change board wants the links in the application to point to the default support site. Which code segment would you use?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"routes.MapRoute(
    name: “Default”,
    url: “{controller}/{action}/{id}”,
    defaults: new { controller = “Home”,
    action = “Index”, id = UrlParameter.Optional });",
                        IsCorrect = false,
                        Explanation = @"This is a default route; it accomplishes nothing."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"routes.MapRoute(
    “remote”,
    “help/remote”,
    new { controller = “support”, action = “Index” }
);",
                        IsCorrect = true,
                        Explanation = @"This is how you add an additional route and point to a different
controller."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"routes.MapRoute(
    “remote”,
    “help”,
    new { controller = “support”, action = “Index” }
);",
                        IsCorrect = false,
                        Explanation = @"The URL portion of the new route does not satisfy the question being
asked."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"routes.MapRoute(
    “remote”,
    “remote/help”,
    new { controller = “support”, action = “Index” }
);",
                        IsCorrect = false,
                        Explanation = @"The URL portion of the new route is reversed."
                    }
                });


            SC("3.4 (2)", @"You are developing an ASP.NET MVC application. You noticed a bug on the DVD controller of the application. You want to ignore the DVD pages until the bug is fixed. What should you do?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add route.SetItem(route.Count, “dvd/”); to the RegisterRoute method.",
                        IsCorrect = false,
                        Explanation = @"SetItem adds an item to the collection. However, adding a route at the
end of the collection does not solve the problem because it is being added as a
route that should be matched."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add route.IgnoreRoute(“dvd/”); to the RegisterRoute method.",
                        IsCorrect = true,
                        Explanation = @"IgnoreRoute is the correct statement to use."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add route.Insert(route.Count,“dvd/”); to the RegisterRoute method.",
                        IsCorrect = false,
                        Explanation = @"Insert adds a new route to the collection for matching, but does not
ignore pages."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add route.Remove(“dvd/”); to the RegisterRoute method.",
                        IsCorrect = false,
                        Explanation = @"Remove removes a route from the table. More than likely, this is an
implied mapping, so removing the DVD route does not stop the application from
mapping the request to the same action."
                    }
                });


            MC("3.4 (3)", @"You are the lead developer on a team that has been working on a large website for months, and several controllers are getting unwieldy. You do not have that problem with views and models because most of your communications are through AJAX to display snippets of information on the UI. What changes can best help you solve your problem?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create partial classes for the controllers that have AJAX functionality to create a set of Controller.Ajax.cs files. Put the AJAX-specific functionality in those files.",
                        IsCorrect = true,
                        Explanation = @"Creating partial classes offers some relief to the problem that you are experiencing,
although not as much as a more-functional separation such as offered
by areas. You do not have to make any UI changes, however."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create an area called AJAX. Move all controllers, models, and views into the area.",
                        IsCorrect = false,
                        Explanation = @"You do not want or need to move any views or models. The code clutter
is due to the small discrete actions that support your AJAX site."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create an area called AJAX. Move the specific AJAX functionality into the Controllers directory and leave the Views and Models directories empty.",
                        IsCorrect = true,
                        Explanation = @"Your best move is to separate the AJAX components into an area. You
would have to change all the calls to it, but you would achieve complete separation
of concern for AJAX calls."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a new ASP.NET MVC 4 application project. Move the AJAX functionality into that so that it will be more responsive.",
                        IsCorrect = false,
                        Explanation = @"This would be too drastic a change. It would require a complete change,
from deployment to testing, throughout the entire application stack."
                    }
                });


            MTC("3.4",
               @"Using routes in various ways",
               @"You are updating an ASP.NET MVC application for a client that sells DVDs and other types of media. The website manager has identified several issues:

- The site includes pages for customers to shop for media as well as pages intended only for sales team members. Currently, everyone can access both types of pages, making the site confusing for sales staff and customers alike.

- When a user enters information about a product into a web browser’s address bar, the browser displays a 404 error if the product is no longer available.

- The manager wants a particular page to be called in multiple ways through different URLs.",
               new TextboxAnswer[3]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"How can you modify the site to make it less confusing to sales team members
and customers?",
                       Explanation = @"You can split the site into areas. A staff area can be authenticated against the company’s
employee database. The customer area can display only products customers can
purchase."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"How can users be redirected to the main page if a product is no longer available
instead of showing them a 404 error?",
                       Explanation = @"If a certain product line is no longer available and still cached in the search engines,
the route can be rewritten so that default controller and actions are called when the
URL fits a certain type of request rather then causing a 404 error."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"How can you modify the site so that when a user types http://myurl/movies, for
example, the DVDs for Sale section is displayed?",
                       Explanation = @"When multiple products can be mapped to the same controller or action, additional
routes can be mapped, and the same controller and/or action can be called for a different
URL request. For example, movies can be mapped to the DVD controller with a
default action."
                   }
               });


            // Section 3.5
            CurrentBackground = @"Control application behavior by using MVC extensibility points";


            SC("3.5 (1)", @"Your application manages the sale of expensive well-drilling equipment. Your IT director wants you to add functionality that sends an email to a customer’s sales account manager whenever someone from your client company logs in to the system. What is an appropriate solution?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Override the AuthorizeAttribute and apply it to the login action. Run the base method first to handle the authentication and then evaluate the status of the request. If the user is a customer, send the email.",
                        IsCorrect = false,
                        Explanation = @"You should not have an AuthorizeAttribute on your login action, because
it ensures that users have to be authenticated before they log in. Users can
never log in to the site."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a custom action filter that is globally defined and overrides the OnActionExecuting method. This action filter evaluates the status of the user and sends the email if it is a customer.",
                        IsCorrect = false,
                        Explanation = @"Because this filter is applied globally, it sends the email every time the
user takes an action, rather than just once per visit."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Override the AuthorizeAttribute that is applied to the login action, check for the user’s authentication status, and determine whether it is a customer prior to sending it to the base authentication method.",
                        IsCorrect = false,
                        Explanation = @"You should not have an AuthorizeAttribute on your login action because
it ensures that users have to be authenticated before they log in. Users can never
log in to the site."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create an action filter that overrides the OnActionExecuted method and apply it to the login action. This action filter evaluates the status of the user and sends the email if it is a customer.",
                        IsCorrect = true,
                        Explanation = @"This is be applied only because the user is leaving the login section of the
application, at which point you also know whether they have been authenticated."
                    }
                });


            MC("3.5 (2)", @"You work for a financial services company that deals with many small brokers. Your executives want to be able to run a report that details all the actions taken by the brokers on the site as a form of auditing and protection. Neither the application nor system currently stores this kind of information. Which of the following are viable solutions?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a globally applied custom action filter that implements the OnActionExecuting method. Have it store the user, the URL, and the forms collection.",
                        IsCorrect = true,
                        Explanation = @"Creating a globally applied action filter enables you to save the state of
every action taken by the user while logged in to your site."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a globally applied custom action filter, ensuring that you set the AllowMultiple parameter to false, that implements the OnActionExecuting method. Have it store the user, the URL, and the forms collection.",
                        IsCorrect = false,
                        Explanation = @"By setting the AllowMultiple to false, you have configured the filter to
be run only once during the lifetime of the application."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a globally applied custom action filter that implements the OnActionExecuted method. Have it store the user, the URL, and the forms collection.",
                        IsCorrect = true,
                        Explanation = @"By creating a globally applied action filter you have ensured that every
action taken by the user will be logged."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Override the AuthorizeAttribute and have it store the user, the URL, and the forms collection.",
                        IsCorrect = false,
                        Explanation = @"Because this is applied only to the AuthorizeAttribute, it logs only those
actions where the user has to be logged in. That enables them to take actions that
might not be logged."
                    }
                });


            MC("3.5 (3)", @"You are adding the capability for users to customize their site’s display colors. You are required to provide a slider that enables users to change each RGB element. What can you do to ensure that this gets treated in the model and stored in the database as a single RGB color?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Strongly bind the sliders to their own individual fields in the model. Ensure that the model unit for color only implements the GET, where you write code that concatenates the values.",
                        IsCorrect = false,
                        Explanation = @"This does not easily enable saving into the database because the color
itself only implements the HTTP Get."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a custom model binder that knows to look for the three values and how to put them together to get the single color.",
                        IsCorrect = true,
                        Explanation = @"This solution works because it combines the three discrete values into a
single object on the model."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add the three different elements to the model and ensure that your update statement to the database correctly joins the elements to get the appropriate color.",
                        IsCorrect = false,
                        Explanation = @"Although this solution gets the value into the database appropriately, it
does not fulfill the requirement that the model be able to use the color as a single
value."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a custom model binder that evaluates the entire color customization process and binds the entire model rather than just managing a subset of the information.",
                        IsCorrect = true,
                        Explanation = @"Although it takes more work, using an approach of creating a model
binder for the entire object, rather than just a part, successfully manages the need
to have it in the model and in the database."
                    }
                });


            MT("3.5",
               @"Applying a theme and logging errors",
               @"You are updating an existing MVC blog site and you want to apply a theme to the
site to create a more modern-looking user interface. You also want to log errors
when a certain page is loaded. Finally, you want to ensure that when a category
such as “recipes” is typed as part of the URL, only content from that category is
displayed.",
               new TextboxAnswer[3]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"How can you create and test the theme without taking down the site or causing
disruptions?",
                       Explanation = @"You can create themes by adding an additional custom view engine for the site and
show the custom view engine on the test site."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What must you modify to ensure that logs are created when a certain action is
executed?",
                       Explanation = @"To execute a certain custom attribute, you need to create a custom action filter and
run additional code within the OnActionExecuting method you have overridden in the
custom action filter."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Currently, the site displays the default list of blog entries, even if you enter a
specific category. You have also modified the action result. What else is missing?",
                       Explanation = @"Because every contingency has been reviewed, and the controller code is written correctly,
you should check the route. Make sure that additional parameter(s) are included
because the code might be checking against parameters."
                   }
               });


            // Section 3.6
            CurrentBackground = @"Reduce network bandwidth";


            SC("3.6 (1)", @"You are working on a AJAX-heavy site, and your script files are separated in your solution by function. You have already implemented minification and bundling, but you are still getting reports of poor performance when users try to access your pages. You cannot replicate the problem locally. What additional steps can you take and still provide the same user experience?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Compress all scripts locally and have the users download the compressed files rather than the uncompressed files.",
                        IsCorrect = false,
                        Explanation = @"Although you might be able to minimize the size of the files being
downloaded, your browser cannot open and use those files."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Turn IIS compression on, turn on the option to compress dynamic pages, and set the minimum file size to 0 so that every file served will be compressed.",
                        IsCorrect = false,
                        Explanation = @"This approach minimizes the size of the files being sent to the client, but
it also has a heavy cost on both server-side and client-side processing because the
server tries to compress every file."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Turn IIS compression on, disable dynamic page compression, and set the minimum file size to the size of your smallest bundled script file.",
                        IsCorrect = true,
                        Explanation = @"This is an appropriate next step to try to compress the script and style
files being sent to the client."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"There is nothing more to do without redesigning the site.",
                        IsCorrect = false,
                        Explanation = @"IIS compression has not yet been tried. It makes more sense to try all
other solutions before approaching a code rewrite."
                    }
                });


            MC("3.6 (2)", @"You want to implement bundling and minification in your site. What are some of the potential problems you need to be aware of?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"None; there is no condition in which this is a poor decision.",
                        IsCorrect = false,
                        Explanation = @"Poor decisions in implementing bundling and minification can hurt
performance rather than help it."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You need to be sure you do not bundle too many scripts together because you cannot take advantage of concurrent downloads if only one or two files are being downloaded.",
                        IsCorrect = true,
                        Explanation = @"Bundling every script into a single large file might decrease the effectiveness
of concurrent downloading."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"You should bundle and minimize scripts and CSS files together for maximum effectiveness.",
                        IsCorrect = false,
                        Explanation = @"You should keep scripts and CSS files separated for maximum effectiveness.
They represent different aspects of the user experience, so a logical separation,
even on the client side, makes sense."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Not bundling logically linked scripts together can have a negative effect on performance.",
                        IsCorrect = true,
                        Explanation = @"Not separating scripts logically might lead to the user downloading
scripts that will never be used. Although the download experience is maximized, it
does not make sense to download unused scripts."
                    }
                });


            MC("3.6 (3)", @"Your U.S.-based company recently opened an office in England. Staff members have been making lots of sales calls, which have generated an increase in visits to the company’s websites. Much of the activity involves downloading sales sheets, product descriptions, and other sales support information currently stored in PDFs. There have been some performance-related complaints from remote sales staff, but no local users have noticed any problems. Which of the following are potential solutions?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Bundle and minify the PDF files to ensure that there is no wasted space.",
                        IsCorrect = false,
                        Explanation = @"Bundling would create unreadable files because it would concatenate
PDF files inappropriately."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Write an action result that takes a file name and returns a compressed version of the file for download.",
                        IsCorrect = true,
                        Explanation = @"This solution causes more work on the client side to open and view the
compressed content, but it provides a more responsive user experience."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Sign on with a CDN with nodes in North America and Europe and use it to serve files and other static content to sales support staff.",
                        IsCorrect = true,
                        Explanation = @"Moving the part of the site most affected by the new usage closer to the
users increases their download speed and thus their perceived performance."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add two additional servers and create a server farm to serve your content.",
                        IsCorrect = false,
                        Explanation = @"Because local users have not noticed a reduction in performance, it is
unlikely that adding more cycles at the server level would improve the responsiveness
of the application for remote users."
                    }
                });


            MT("3.6",
               @"Increasing the responsiveness of a site",
               @"You are troubleshooting an ASP.NET MVC site hosted in Chicago. A client in Hong
Kong SAR reports that the site is slow.",
               new TextboxAnswer[3]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"What tool should you use to determine how long each item is taking to load?",
                       Explanation = @"The Internet Explorer F12 tool can tell you how long each asset is taking to load.
Remember that only six items can be pulled from a host at a time. Depending on the
time required for downloading, you have to figure out how to resolve the situation."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"To make the site load faster, which assets can you modify, and how?",
                       Explanation = @"You can minify and bundle JavaScript and CSS assets so they can be loaded more
quickly in a browser."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"You believe a CDN will improve performance. What should you check?",
                       Explanation = @"Although CDNs can help, make sure that you know where the CDN servers are located
as well as where the clients are placed. The closer the client is to the server, the shorter
the distance the data has to travel."
                   }
               });


            ////////////////// Section 4 - Troubleshoot and debug web applications /////////////////////

            // Section 4.1
            CurrentBackground = @"Prevent and troubleshoot runtime issues";


            SC("4.1 (1)", @"Your application has an intermittent issue, based on the user’s path through the application,
in which the application seems to stop running. Even when running in debug
mode, the application calls a web service and then stops. The application locks and the
call never returns, thus the user’s request is never completed and eventually times out.
What performance or profiling tool will provide the most pertinent information about
your application?",
                  new CheckboxAnswer[5]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"CPU sampling in the Performance Wizard",
                        IsCorrect = false,
                        Explanation = @"CPU sampling will not provide assistance for the main problem of the
application locking up."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Memory allocation in the Performance Wizard",
                        IsCorrect = false,
                        Explanation = @"Memory analysis will not provide assistance for the main problem of
the application locking up."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Resource contention data in the Performance Wizard",
                        IsCorrect = true,
                        Explanation = @"Thread and resource profiling will give some understanding of what kind
of actions are taking place that will cause a resource or thread to be blocked."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Tracing from the System.Diagnostics namespace that logs the times of web service
call and web service return",
                        IsCorrect = false,
                        Explanation = @"Tracing information regarding when a web service starts and stops
would be interesting. However, the problem is that the web service is called and
never returns, so this logging information would not be complete or useful."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The Health Monitoring tool, for capturing security information related to the interaction
with the web service",
                        IsCorrect = false,
                        Explanation = @"A problem with security information would likely not cause the service
to stop responding."
                    }
                });


            MC("4.1 (2)", @"You inherited a working application that began as a proof of concept but was eventually adopted as a production application without being refactored. Many new requirements need to be added. As part of your initial analysis, you notice a lot of problems with bad data. Which solutions will help remediate this issue?",
                  new CheckboxAnswer[5]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Running the Performance Wizard to sample CPU usage",
                        IsCorrect = false,
                        Explanation = @"The Performance Wizard does not support management or identify
data issues."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Adding code contracts to ensure that the input parameters have expected values",
                        IsCorrect = true,
                        Explanation = @"Preconditional code contracts ensure that incorrect information is not
submitted to a method. This helps eliminate the possibility of bad data."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Adding code contracts to ensure that the return values meet specific criteria",
                        IsCorrect = true,
                        Explanation = @"Postconditional code contracts ensure that incorrect information is not
returned from a method. This helps eliminate the possibility of bad data."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Adding code contracts to ensure that objects do not become invalid during process",
                        IsCorrect = true,
                        Explanation = @"Invariant code contracts ensure that objects do not get to an invalid state.
This helps eliminate the possibility of bad data."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Running the Visual Studio profiler to analyze application flow",
                        IsCorrect = false,
                        Explanation = @"The Visual Studio profiler does not provide any support for managing
or identifying data issues."
                    }
                });


            MC("4.1 (3)", @"You are helping a client estimate the effort involved in adding comprehensive monitoring to an enterprise-level ASP.NET MVC application. Which of the following are useful considerations while building the estimate?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Adding health monitoring involves many choices. An evaluation will have to be performed to determine which monitors will be of real use.",
                        IsCorrect = true,
                        Explanation = @"Adding health monitors is an easy task. Determining which options provide
the information you need will be time-consuming because of the number of
choices."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Adding generic logging can be done as part of the rework. After the logging pattern has been established, you can add logging as part of any new work and as part of the refactoring process.",
                        IsCorrect = true,
                        Explanation = @"Generic logging is a useful addition and can be added as items are refactored;
it is not an all-or-nothing type of work."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Adding tracing must be done completely and comprehensively before it can be of any use. There is no point in implementing tracing in a single part of the application if you are not going to refactor the application.",
                        IsCorrect = false,
                        Explanation = @"You can perform tracing on an ad-hoc basis as needed."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Creating a data collection set in Performance Monitor will provide all the needed information and can be set up in a few minutes.",
                        IsCorrect = false,
                        Explanation = @"Data collection in Performance Monitor is relatively simple. However,
determining the most critical items to monitor can be time-consuming because
there are hundreds of counters."
                    }
                });


            MT("4.1",
               @"Understanding data issues",
               @"Maintaining data integrity is important to the success of any ASP.NET MVC 4 application.
It is especially critical when working on a data-intensive application, like a
point-of-sales application.",
               new TextboxAnswer[2]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"What are some tools that help you prevent data corruption?",
                       Explanation = @"Several tools can help you maintain the integrity of your data. The first is parameter
checking, in which you throw an ArgumentException. Another is the use of code contracts
to perform a check on the values going into and coming out of the application."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What are some tools you can use to diagnose whether data corruption has
occurred?",
                       Explanation = @"Several tools can give you an idea of what is happening with your data. You can’t depend
solely on errors for the information you need because the data might be wrong
without throwing errors. You can add helpful information to logging messages, as well
as details on the parameters coming into a method and the return values out of the
method. This information helps you understand problems in the system. You can also
take the same approach with tracing."
                   }
               });


            // Section 4.2
            CurrentBackground = @"Design an exception handling strategy";


            SC("4.2 (1)", @"You are re-creating an application that was originally built with ASP 2.0. You need to break the monolithic application into a traditional three-tier application. One of the requirements is that database errors must be displayed in the UI. Which solution will present enough information to the user so they can notify the appropriate person of a problem, but not allow the user to gain information about the design of the database?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add a first chance exception handler and log the information in the error.",
                        IsCorrect = false,
                        Explanation = @"Although logging information is an important consideration, it does not
provide additional information to the users."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add a generic database-layer exception handler to the business layer, and pass appropriate error information to the UI layer for display in a custom error page.",
                        IsCorrect = true,
                        Explanation = @"The business layer is the appropriate place to manage data layer exceptions.
By repackaging it and sending it up to the user layer, you can maintain
separation of concerns and meet the requirement to display information to the
user. This will also enable you to log the information as necessary to support debugging."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add specific database exception handlers in the business layer and log the information into the database for further review.",
                        IsCorrect = false,
                        Explanation = @"You are having the business layer handle the database exception correctly,
but you are not meeting the requirement to provide information about the
error to the user."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Let the errors pass through the business layer to the UI layer where they can be handled as specific errors and presented as appropriate in the UI.",
                        IsCorrect = false,
                        Explanation = @"By not having the business layer capture and handle the error, you are
building an improper relationship between the UI and the database."
                    }
                });


            SC("4.2 (2)", @"What is an advantage to using first chance exception notification?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The ability to capture and handle all exceptions that occur within the application in one place",
                        IsCorrect = false,
                        Explanation = @"First chance exception is a notification only. It does not let you do anything
other than observe the exception."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The ability to log an exception after it is handled by its appropriate error handling code",
                        IsCorrect = false,
                        Explanation = @"The first chance exception handler gets the exception before it has
been issued to its appropriate error management code. You cannot see what happens
to the exception as it is being handled."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The ability to log an exception before it is touched by any other error handler",
                        IsCorrect = true,
                        Explanation = @"The first chance exception handler enables you to examine an exception
and take some action before it is touched by any other handler."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The ability to forward an exception to an error handler based on the type of exception that was thrown",
                        IsCorrect = false,
                        Explanation = @"The first chance exception handler can examine the exception, but cannot
forward or handle the exception."
                    }
                });


            SCC("4.2 (3)", @"Using custom error pages provides a lot of flexibility to an application because it allows for a consistent user experience even when the application has a problem. To take full advantage of this flexibility, you need to be able to create the pages and configure the application to use these pages. What code will provide custom error pages for 404 errors and general exceptions?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"<system.web>
    <customErrors mode=”RemoteOnly”
        defaultRedirect=”Error/GeneralException”>
            <error statusCode=”404” redirect=”ErrorController.Status404” />
    </customErrors>
</system.web>",
                        IsCorrect = false,
                        Explanation = @"This shows only the <system.web> part of the configuration. There is
also a need to have an entry in the <system.webServer> node."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"<system.web>
    <customErrors mode=”RemoteOnly”
        defaultRedirect=”Error/GeneralException”>
            <error statusCode=”404” redirect=”Error/Status404” />
    </customErrors>
</system.web>
<system.webServer>
    <httpErrors errorMode=”Detailed” />
</system.webServer>",
                        IsCorrect = true,
                        Explanation = @"This example shows the 404 error status and it sets the default error handler
to the server error action."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"<system.web>
    <customErrors mode=”RemoteOnly”>
        <error statusCode=”404” redirect=”Error/Status404” />
        <error statusCode=”GeneralException”
            redirect=”Error/GeneralException” />
    </customErrors>
</system.web>
<system.webServer>
    <httpErrors errorMode=”Detailed” />
</system.webServer>",
                        IsCorrect = false,
                        Explanation = @"This example shows the general exception handler using a status code
of GeneralException. The status codes relates to the type of HTTP error that will be
handled."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"<system.web>
    <customErrors mode=”LocalOnly”
        defaultRedirect=”Error/GeneralException”>
            <error statusCode=”404” redirect=”Error/Status404” />
    </customErrors>
</system.web>
<system.webServer>
    <httpErrors errorMode=”Detailed” />
</system.webServer>",
                        IsCorrect = false,
                        Explanation = @"The custom error mode is set to mode=”LocalOnly”, which means that
users will not see the actual error pages."
                    }
                });


            MT("4.2",
               @"Implementing error handling",
               @"You are writing an application to be used by your company’s support staff to manage
the configuration of many custom applications used throughout the enterprise.
Your users are power users who will provide tier 1 support for the application you
are building.",
               new TextboxAnswer[2]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"How should you treat error handling, considering the application is for support
staff?",
                       Explanation = @"You should treat exception handling as robustly as possible, regardless of the audience.
It is a best practice to try to eliminate and/or remediate issues as soon as they happen."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Should you include as much information as possible about each error, including
tables with values and other information? Why or why not?",
                       Explanation = @"No. None of the information should be displayed in an error page outside of the development
environment. Although these users are support staff for multiple applications,
it is possible they are not fully accredited administrators across all the applications with
which you are interfacing. You should treat logging and information display the same
as if the users were not support staff. However, although you should not display the
information in the UI to the user, you need to capture the necessary logging information
to help technical staff troubleshoot issues."
                   }
               });


            // Section 4.3
            CurrentBackground = @"Test a web application";


            SC("4.3 (1)", @"You are under contract with a large company that is starting to implement a TDD approach. As part of the long-term support for this effort, the company needs you to complete several unit tests. As you review the current code base, you find good tests for the model, but no tests for anything other than the model. Which of the following is the best approach to complete the unit tests?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a new directory in the unit test project for the controller- and action-specific tests. Create a unit test file for each controller. Inside that file, have one or more tests for only the controller action methods.",
                        IsCorrect = false,
                        Explanation = @"There might very well be nonaction methods in a controller. Those
methods should be tested as well."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Add a new file to the unit test project called ControllerTests. Put all tests for all the controllers and the actions in the file.",
                        IsCorrect = false,
                        Explanation = @"You should provide much more separation of your tests than using a
single file for every unit test that applies to a controller."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a new directory in the unit test project for the controller- and action-specific tests. Create a unit test file for each action you are going to test.",
                        IsCorrect = false,
                        Explanation = @"This is too much of a breakdown. The best relationship between controllers
and the applicable unit tests is usually 1 to 1. However, a controller with a
large number of methods working within the controller will not meet the 1-to-1
ratio. You should test nonaction methods in the controller as well."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a new directory in the unit test project for the controller- and action-specific tests. Create a unit test file for each controller. Inside that file have one or more tests for all methods in the controller, regardless of whether they are an action or not.",
                        IsCorrect = true,
                        Explanation = @"This solution provides for testing actions and nonactions as well as a
good split of the tests per file."
                    }
                });

            SC("4.3 (2)", @"Your ASP.NET MVC web application has just been released to a group of pilot users. The users are reporting periods of extreme performance degradation. You did not encounter performance issues during development or the quality assurance phase. What can you do in your development environment to understand what is occurring in the production environment?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a set of unit tests that repeatedly test certain parts of the application. Run them continuously over a period of time to ensure that the application works as expected.",
                        IsCorrect = false,
                        Explanation = @"Unit tests are designed to ensure that the functionality and logic of
the application are correct. They do not work well when you need a performancebased
analysis."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a set of web tests that exercise the application. Set a run load of 50 percent of your pilot users and run them in a constant load testing process to validate the application’s behavior.",
                        IsCorrect = false,
                        Explanation = @"Although this test will provide some useful and interesting information,
running a constant load of 50 percent might not give you the information you
need. More users could use the application at any point in time."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a set of web tests that exercise the application. Using a step approach, start with a minimal number of users and increase to the total number of users in the pilot program.",
                        IsCorrect = true,
                        Explanation = @"Starting from a midlevel count of users and then increasing to the total
number of possible numbers should give you an idea of what is happening during
the day-to-day running of the application in production."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create a set of web tests that exercise the application. Using a goal-based approach, set the process to run to 75 percent CPU utilization. When you reach that point, compare the results with the number of users in the pilot program.",
                        IsCorrect = false,
                        Explanation = @"Understanding the number of users required to reach 75 percent
utilization of the CPU might be interesting, but it does not help you understand
the users’ issue. They might be experiencing slowness due to memory utilization or
threading contention that this approach will not be able to detect."
                    }
                });


            SC("4.3 (3)", @"You support hardware purchasing for an ASP.NET MVC application in your company. The application is finished, and the development team knows the number of intended users. Which approach will give the team the best understanding of the application’s hardware needs?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create several web tests that exercise all parts of the application, including all static pages. Run these tests in a constant load at various levels to see the effect on performance.",
                        IsCorrect = false,
                        Explanation = @"Running at a constant load, even at various levels of user count, will not
efficiently give them the information that they need."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create several web tests that exercise all parts of the application, including all static pages. Run these tests in a step approach up to the maximum expected number of users. This will give you an idea of the load one server can handle.",
                        IsCorrect = false,
                        Explanation = @"Although this approach will give them interesting information and an
understanding of the capacity of a server, it is based less on statistical information
than on a subjective analysis of performance."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create several web tests that exercise all parts of the application, including all static pages. Run these tests in a goal-based approach in which the percent of CPU usage and percent of memory usage metrics are set to the company standard maximums. Evaluate how many users it takes to reach the maximum levels.",
                        IsCorrect = true,
                        Explanation = @"This approach will give them an objective analysis of the amount of users
a server can manage at a particular level of memory or CPU usage."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Create several web tests that exercise only the dynamic parts of the application, ignoring all static content. Run these tests in a goal-based approach in which the percent of CPU usage and percent of memory usage metrics are set to the company standard maximums. Evaluate how many users it takes to reach these maximum levels.",
                        IsCorrect = false,
                        Explanation = @"This approach leaves out static pages. For a true test, you should exercise
all parts of the application."
                    }
                });


            MTC("4.3",
               @"Testing an ASP.NET MVC 4 application",
               @"You work with a team that is upgrading an ASP.NET Web Forms application to an ASP.NET MVC 4 application and adding enhancements. The look, feel, and navigation of the application will remain the same, but the underlying structure must be modified in anticipation of future features.

The business rules that are part of the application are poorly documented and complicated. The team is starting to run behind schedule. They have stopped working on new functionality, but they are continuing the migration.",
               new TextboxAnswer[2]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"To add testing support, where would it make the most sense for you to start?",
                       Explanation = @"Because you are not knowledgeable about the business rules, and because the user
flow and navigation will remain the same, you can quickly bring the most value by
starting web tests. Unit tests validate the software one method at a time. However,
web tests enable you to evaluate the correctness of whole business flows at a time.
If web tests fail, you can use unit tests to help you find where logic issues might be
occurring."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Can you create valid unit tests without a complete understanding of the business
rules being tested?",
                       Explanation = @"Conceptually, you can create a unit test without understanding the business requirements.
However, you are assuming that the method performs all the logic appropriately.
This means the unit tests are meant to ensure that other changes in the software
do not affect the methods being tested instead of just ensuring that the application is
correct. Both have their places, however, especially if the application has already been
tested for correctness through other means. In that case, assuming that the functionality
is correct is not the same risk as joining an application in progress."
                   }
               });


            // Section 4.4
            CurrentBackground = @"Debug a Windows Azure application";


            MC("4.4 (1)", @"You will be deploying your application in Windows Azure. You know that you need to include logging, but you have some concerns. What diagnostic capabilities can Windows Azure support?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The capability to turn on and off different aspects of the Windows Azure diagnostics through configuration changes",
                        IsCorrect = true,
                        Explanation = @"You can control the diagnostics setting through configuration within your
Windows Azure–hosted application."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The capability to retain logs from your custom logging solution on a Windows Azure storage account",
                        IsCorrect = true,
                        Explanation = @"Although the Windows Azure VMs get recycled when the role restarts,
saving the logs to the Windows Azure storage system outside of your VM would
ensure that they are available."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The capability to retain logs from your custom logging solution when written to a local store",
                        IsCorrect = false,
                        Explanation = @"Because the Windows Azure VMs are entirely virtual and recycle whenever
a role restarts, there is a significant chance you will lose logging information."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The capability to send entries to the Windows Azure diagnostics system from within your application",
                        IsCorrect = true,
                        Explanation = @"You can treat the Windows Azure diagnostics system like a traditional
logging tool by writing messages to the system."
                    }
                });


            MC("4.4 (2)", @"What do you need to do to use IntelliTrace from within Windows Azure?",
                  new CheckboxAnswer[5]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Publish the solution from any version of Visual Studio Professional 2012 or higher.",
                        IsCorrect = false,
                        Explanation = @"You can enable IntelliTrace only through Visual Studio Ultimate 2012
edition."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Select the Enable IntelliTrace check box before publishing the solution.",
                        IsCorrect = true,
                        Explanation = @"When publishing the projects, you can configure the process to deploy in
debug mode with IntelliTrace enabled."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Ensure that you made all configuration changes in the Web.config file that will turn on IntelliTrace.",
                        IsCorrect = false,
                        Explanation = @"There are no Web.config changes required to run the system with IntelliTrace
turned on."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Download and view the IntelliTrace logs through a web browser.",
                        IsCorrect = false,
                        Explanation = @"You cannot download and run the IntelliTrace logs through a web
browser. Running them requires Visual Studio Ultimate 2012."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Download and view the IntelliTrace logs through Visual Studio Ultimate 2012.",
                        IsCorrect = true,
                        Explanation = @"You can view the IntelliTrace logs through Visual Studio after download."
                    }
                });


            SC("4.4 (3)", @"You want to configure Windows Azure diagnostics. Where do you configure the specific Performance Monitors you want to have run in support of your application?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"ServiceDefinition.csdef",
                        IsCorrect = false,
                        Explanation = @"You include diagnostics in a project using the ServiceDefinition.csdef
file. The file does not configure which monitors will be included when your application
is running."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"ServiceConfiguration.cscfg",
                        IsCorrect = false,
                        Explanation = @"You configure log storage locations in the ServiceConfiguration.cscfg
file. The file does not configure which monitors will be included when your application
is running."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Diagnostics.wadcfg",
                        IsCorrect = true,
                        Explanation = @"You configure specific monitors that will run in your application in the
Diagnostics.wadcfg file."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Web.config",
                        IsCorrect = false,
                        Explanation = @"The Web.config file is not used to include Windows Azure diagnostics in
an application."
                    }
                });


            MT("4.4",
               @"Running an application in Windows Azure",
               @"Your company wants to run a test project for hosting an ASP.NET MVC application
in the cloud. Management is not sure whether it is the appropriate solution, so they
want to ease into it and see how it goes. You have been asked to make your application
work when hosted in a Windows Azure Virtual Machine (VM). Your application
depends on other sources, such as supplier systems, for information used within the
system, which results in a lot of logging and other transactional tracing within the
application. Although your application must work in Windows Azure, you are not
supposed to eliminate the ability to work outside of Windows Azure.",
               new TextboxAnswer[2]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"Your application uses a third-party logging tool to write to logs that are stored
on the local file system. Most methods in the application reference this tool. It
will not work when deployed on Windows Azure. What do you need to change?",
                       Explanation = @"The use of third-party logging tools is complicated by the virtual nature of the Windows
Azure VM. However, you can take certain approaches to mitigate the risk of
losing logging data. One approach is to configure your logging subsystem to write the
logging data to files that are external to your VM, such as in a Windows Azure storage
account or in one of the Windows Azure database providers. If that is not possible,
create a log transference job in which a process moves the logs or writes them into
another storage option regularly, such as every 10 minutes. All this work can be done
outside of the application and will not affect the application’s capability to run in a
non-Windows Azure system."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What Windows Azure diagnostic tools can you include in your application
without having to make any code changes that would affect the application’s
capability to run in a non-Windows Azure environment?",
                       Explanation = @"You can include many of the Windows Azure diagnostic tools in the application without
affecting the code base. The entire diagnostics framework, for example, can be set
up by adding some additional files that would be meaningless to applications stored
in a non-Windows Azure system. The ability to run remote IntelliTrace is also allowed
without having to create any changes in the application’s source code. You can also use
RDP to help you debug in Windows Azure."
                   }
               });


            //////////////// Section 5 - Design and implement security ////////////////

            // Section 5.1
            CurrentBackground = @"Configure authentication";


            SC("5.1 (1)", @"What type of authentication accepts login credentials that will be checked against the
domain or local server and are sent in a hashed format?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Basic authentication",
                        IsCorrect = false,
                        Explanation = @"Basic authentication is transmitted from the client to the server in
Base64 encoding and is not encrypted."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Digest authentication",
                        IsCorrect = true,
                        Explanation = @"Digest authentication is a challenge-response–based authentication
method to ensure that user credentials are not sent over the network in clear text."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Forms authentication",
                        IsCorrect = false,
                        Explanation = @"Forms authentication does not require login credentials to be sent in
an encrypted form, and it does not automatically check against the domain and/or
local server."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Windows authentication",
                        IsCorrect = false,
                        Explanation = @"Windows authentication does not send user names and passwords from
the client to the server."
                    }
                });


            SCC("5.1 (2)", @"Forms authentication enables you to write code to validate user credentials. After it is complete, you can register the authentication cookies for use throughout the user’s visit by using which of the following?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"FormsAuthentication.SetAuthCookie",
                        IsCorrect = true,
                        Explanation = @"FormsAuthentication.SetAuthCookie registers the authentication token in
the cookie for use in future requests."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"FormsAuthenticationCookie = new FormsAuthenicationCookie();",
                        IsCorrect = false,
                        Explanation = @"You have created a new FormsAuthenticationCookie, but it has no values,
so it would not be useful in future requests."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"FormsAuthentication.ClearAuthCookie",
                        IsCorrect = false,
                        Explanation = @"FormsAuthentication.ClearAuthCookie removes the authentication
token."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"MembershipProvider.User =",
                        IsCorrect = false,
                        Explanation = @"MembershipProvider.User = does not maintain information between
requests."
                    }
                });


            MC("5.1 (3)", @"What default attributes or inline checks would you use to create an accepted-list
scenario in ASP.NET MVC 4?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Authorize attribute",
                        IsCorrect = true,
                        Explanation = @"AuthorizeAttribute can be put on controllers and actions to require authorization."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"RequireHttps attribute",
                        IsCorrect = false,
                        Explanation = @"RequireHttps will ensure that the communication between client and
server is encrypted, but it does not do anything to help ensure an accepted-list
scenario."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"WebSecurity.IsAuthenticated",
                        IsCorrect = false,
                        Explanation = @"WebSecurity.IsAuthenticated verifies that a user is authenticated, but it
does not do it in a way that supports an accepted-list scenario."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"AllowAnonymous attribute",
                        IsCorrect = true,
                        Explanation = @"AllowAnonymous can be put on controllers and actions to enable unauthorized
users access."
                    }
                });

            MC("5.1 (4)", @"What interfaces or classes should be implemented or inherited when creating custom authentication that is based on a non-Windows, third-party provider?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"ActiveDirectoryMembershipProvider",
                        IsCorrect = false,
                        Explanation = @"ActiveDirectoryMembershipProvider is a Windows-specific authentication
provider. It would not provide assistance in this case."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"IIdentity",
                        IsCorrect = true,
                        Explanation = @"IIdentity describes the user that was authenticated."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"SqlMembershipProvider",
                        IsCorrect = false,
                        Explanation = @"SqlMembershipProvider provides assistance in getting membership data
from SQL databases. It would not help get information from a third-party solution."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"IPrincipal",
                        IsCorrect = true,
                        Explanation = @"IPrincipal provides the security context of the user on whose behalf the
code is running."
                    }
                });


            MC("5.1 (5)", @"What kind of helper methods does WebSecurity provide?",
                  new CheckboxAnswer[5]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Login",
                        IsCorrect = true,
                        Explanation = @"Login is one of the methods provided by the WebSecurity helper."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"ResetPassword",
                        IsCorrect = true,
                        Explanation = @"ResetPassword is one of the methods provided by the WebSecurity helper."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"CreateAccount",
                        IsCorrect = true,
                        Explanation = @"CreateAccount is one of the methods provided by the WebSecurity helper."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"ChangePassword",
                        IsCorrect = true,
                        Explanation = @"ChangePassword is one of the methods provided by the WebSecurity helper."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"DeleteAccount",
                        IsCorrect = false,
                        Explanation = @"DeleteAccount is not supported through the WebSecurity helper."
                    }
                });


            MT("5.1",
               @"Configuring authorization in a multisystem environment",
               @"One of your clients is in the environmental sciences. The client runs its labora-tory information management (LIM) system on a mainframe computer with dumb terminal access. The client wants to build an intranet site and migrate some business processes to computers running Windows Server. Currently, the user management system runs within the LIM. The client will either migrate the LIM last, or keep the LIM on the mainframe and access the user management system from Windows as an ordinary database.",
               new TextboxAnswer[3]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"What is the easiest way to solve the business problem of authenticating against the LIM?",
                       Explanation = @"There are several options that could be used to solve this problem. The primary, and
likely easiest-to-implement, option is to create a custom authentication provider that
implements IIdentity and IPrincipal. By overriding the base methods with the custom
code that accesses the database, you can use the same methods to determine authenticity
as you would with a .NET-delivered authentication provider."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Does the possibility that the client might never change from LIM authentication affect your solution choice? Why or why not?",
                       Explanation = @"It should not, as long as you develop your solution using IIdentity and IPrincipal
interfaces rather than the implementation. If the authentication mechanism changes,
managing that in your application could be as simple as switching to a supported
implementation."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What kind of problems do you anticipate if the LIM is left on the mainframe?",
                       Explanation = @"There are many problems that could occur that you should try to predict and resolve.
If the LIM remains on the mainframe, system access can be complicated and will not
fit any of the common providers. You will have to create a custom provider and ensure
that validation occurs with every release."
                   }
               });


            // Section 5.2
            CurrentBackground = @"Configure and apply authorization";


            SCC("5.2 (1)", @"What attribute or code snippet within a controller enables a role named Admin to access actions or code blocks after the check-in code?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"RoleProvider.GetRolesForUser(“Admin”)",
                        IsCorrect = false,
                        Explanation = @"RoleProvider.GetRolesForUser(“Admin”) gets the list of roles for the user
that is passed in as a parameter. In this case, it uses a hard-coded value of “Admin.”"
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"[Authorize(Roles=”Admin”)]",
                        IsCorrect = true,
                        Explanation = @"The Authorize attribute handles authorization on a controller and/or
action basis by using the Roles= qualifier."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"RoleProvider.IsUserInRole(User.Name)",
                        IsCorrect = false,
                        Explanation = @"RoleProvider.IsUserInRole(User.Name) does a check to see whether the
currently logged in user is within a role that is passed in as a parameter to the
function. In this case, the code will be looking for a role that matches the user’s
name."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"[AuthorizeAttribute(Roles=”Admin”)]",
                        IsCorrect = false,
                        Explanation = @"Although AuthorizeAttribute is the correct class, the proper way to use it
in attribution is through the Authorize keyword."
                    }
                });


            MC("5.2 (2)", @"Which methods help the RoleProvider determine whether a user is assigned a role or set of roles?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"GetRoles",
                        IsCorrect = false,
                        Explanation = @"The GetRoles method provides a list of all roles for an ApplicationName."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"GetRolesForUser",
                        IsCorrect = true,
                        Explanation = @"GetRolesForUser gets a list of roles for a user."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"IsUserInRole",
                        IsCorrect = true,
                        Explanation = @"IsUserInRole returns a Boolean on whether a particular user has a role."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"FindUsersInRole",
                        IsCorrect = false,
                        Explanation = @"FindUsersInRole returns a list of users that have the applicable role."
                    }
                });


            MC("5.2 (3)", @"Why should you create a custom role provider?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"To use a data source not regularly supported",
                        IsCorrect = true,
                        Explanation = @"Because traditional providers work only on SQL Server, accessing a different
data provider such as MySQL would require a custom provider."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"To use the SimpleRoleProvider",
                        IsCorrect = false,
                        Explanation = @"Using the SimpleRole provider would not require you to create a custom
provider."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"To use a database design different than .NET provides",
                        IsCorrect = true,
                        Explanation = @"Using a different database design would require that you create a custom
provider."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"To provide a special configuration file entry",
                        IsCorrect = false,
                        Explanation = @"Creating a custom provider would require a special configuration file
entry, but that would not be a reason to create the custom provider in the first place."
                    }
                });


            MT("5.2",
               @"Defining detailed roles",
               @"You work for a large manufacturing firm that wants to create a just-in-time (JIT) inventory
management system. The firm purchased an off-the-shelf enterprise system
about five years ago, but the firm’s manufacturing process is so highly customized
that the software does not meet their needs. The project owner said general roles
are not sufficient, and that he wants to define each major item and provide access
to create, read, update, and delete individually. As you evaluate the system, you realize
hundreds of roles must be created, which will make maintenance a challenge.",
               new TextboxAnswer[2]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"What are some advantages of creating this many roles?",
                       Explanation = @"There are several significant advantages to this approach. This is a classic enterprise
approach to determining rights and responsibilities because it gives you control of all
aspects of the use of the system. Companies evolve over time, so the definition of “who
does what” changes as well as personnel and business needs. In the real world, we
often discover situations in which a role named Director needs access to information,
342 CHAPTER 5 Design and implement security
but it is actually an administrative assistant who compiles the information. Should you
assign the Director role to the administrative assistant’s user account as well? In a large
organization, these issues will arise repeatedly. By defining access at this low level,
these kinds of situations can be easily resolved."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What can you do to mitigate maintenance challenges?",
                       Explanation = @"There are several ways to mitigate maintenance difficulties. The first is to create a subset
of “groups” that have standard sets of functionality. Users are assigned to a group
that gives the users the most appropriate access. If a person’s needs change, the group
assignment can change as well. Users would not see the mechanics of how the group
is actually defined in the system. Another way to mitigate maintenance challenges is
by using templates. Templates let an administrator group a set of roles. This approach
provides many of the flexible features of traditional roles but with the capability to
customize individual access without having to create a new role."
                   }
               });


            // Section 5.3
            CurrentBackground = @"Design and implement claims-based authentication across federated identity stores";

            SC("5.3 (1)", @"Windows Azure ACS allows for federated identification from outside agencies such as a Microsoft account or Facebook. Which entity issues the final token that your application will accept for access?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The outside agency",
                        IsCorrect = false,
                        Explanation = @"The token provided by a Microsoft account or Facebook is read by ACS,
which then creates the token that your application uses."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Your application",
                        IsCorrect = false,
                        Explanation = @"An application does not read the token; it reads the token provided by
ACS."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Your network firewall",
                        IsCorrect = false,
                        Explanation = @"A network firewall does not provide a security token."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Windows Azure ACS",
                        IsCorrect = true,
                        Explanation = @"ACS reads the token provided by the identity providers and creates a
token that your application understands and accepts."
                    }
                });


            SC("5.3 (2)", @"WIF enables you to create a custom token. To be able to use the token, you must create a custom token handler by overriding which of the following?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"SecurityToken",
                        IsCorrect = false,
                        Explanation = @"SecurityToken is the .NET Framework class that identifies the token."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"SecurityTokenHandler",
                        IsCorrect = true,
                        Explanation = @"SecurityTokenHandler is the appropriate class to override for creating a
custom token handler."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"SWTToken",
                        IsCorrect = false,
                        Explanation = @"SWTToken is a type of common token."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Saml2SecurityTokenHandler",
                        IsCorrect = false,
                        Explanation = @"Saml2SecurityTokenHandler is a specific handler for a specific type of
token."
                    }
                });


            MC("5.3 (3)", @"WIF adds the concepts of claims to which of the following?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"IIdentity",
                        IsCorrect = true,
                        Explanation = @"The class implementing the IIdentity interface contains claims
information."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"IPrincipal",
                        IsCorrect = true,
                        Explanation = @"The class implementing the IPrincipal interface contains claims information."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"User",
                        IsCorrect = false,
                        Explanation = @"User does not carry any claims information."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"MembershipProvider",
                        IsCorrect = false,
                        Explanation = @"MembershipProvider does not contain claims information."
                    }
                });


            MT("5.3",
               @"Creating an application that supports multiple authentication methods",
               @"You are working on a new internal site for your employer that will act as a companyspecific
social networking site. Your product owner has requested that this site be
able to be authenticated from your internal Active Directory for your employees,
but you also need to support logins from your customers, using their preferred
authentication mechanisms. They have a mixture of authentication processes that
your application will need to interact with.",
               new TextboxAnswer[2]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"What are examples of the kind of claims you would want to get from the client’s
authentication mechanism?",
                       Explanation = @"You should gather information such as name, email address, and employer, at a minimum,
to properly populate a social networking site. Other claim information (such as a
phone number or other personally identifiable information) might also be useful. It is
important to gather a consistent set of information from your clients."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What would implementing Windows Azure ACS mean to the work that you have
to do in your application’s code?",
                       Explanation = @"The main consideration is that it enables you to build your application to use a single
source of authentication: the Windows Azure ACS. Your application will not have to
be modified when new clients are brought into the system with their own authentication
points. You simply have to configure ACS to ensure the proper redirection and
authentication."
                   }
               });


            // Section 5.4
            CurrentBackground = @"Manage data integrity";


            SC("5.4 (1)", @"What class handles the actual data encryption",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"RijndaelManaged",
                        IsCorrect = false,
                        Explanation = @"RijndaelManaged is a method of encryption and does not create a
stream."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"RSACryptoServiceProvider",
                        IsCorrect = false,
                        Explanation = @"RSACryptoServiceProvider does not provide a stream."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"CryptoStream",
                        IsCorrect = true,
                        Explanation = @"CryptoStream provides access to an encrypted stream."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"RSAKeyInfo",
                        IsCorrect = false,
                        Explanation = @"RSAKeyInfo contains information for managing an RSA key and does
manage encryption."
                    }
                });


            MC("5.4 (2)", @"What are digital signatures used for?",
                  new CheckboxAnswer[5]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Encryption",
                        IsCorrect = false,
                        Explanation = @"Encryption is not a valid reason for digital signing."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Authentication",
                        IsCorrect = true,
                        Explanation = @"Authentication is a reason for digital signing because it uniquely defines
the party signing the data."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Nonrepudiation",
                        IsCorrect = true,
                        Explanation = @"Nonrepudiation is a reason for digital signing because it uniquely defines
the party signing the data and shows that it had to be the original party that submitted
the data."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Authorization",
                        IsCorrect = true,
                        Explanation = @"Authorization can be determined after the individual is uniquely defined."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Hashing",
                        IsCorrect = false,
                        Explanation = @"Hashing is not a valid reason for digital signing."
                    }
                });


            SCC("5.4 (3)", @"How do you encrypt the <connectionStrings> section of the Web.config file?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"aspnet_regiis -pe “ConnectionStrings” -app “/MachineDPAPI” -prov “RsaProtectedConfigurationProvider”",
                        IsCorrect = true,
                        Explanation = @"This is the proper way to encrypt the <connectionStrings> setting of the
Web.config file."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"aspnet_regiis -pe “Web.Config”-app “/MachineDPAPI” -prov “RsaProtectedConfigurationProvider”",
                        IsCorrect = false,
                        Explanation = @"This command encrypts more than the <connectionStrings> section of
the Web.config file."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"aspnet_regiis -pd “ConnectionStrings” -app “/MachineDPAPI” -prov “RsaProtectedConfigurationProvider”",
                        IsCorrect = false,
                        Explanation = @"This command decrypts the <connectionStrings> section of the Web.
config file."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"aspnet_regiis -pd “Web.Config” -app “/MachineDPAPI” -prov “RsaProtectedConfigurationProvider”",
                        IsCorrect = false,
                        Explanation = @"This command decrypts rather than encrypts information, and more
information than just the <connectionStrings> section."
                    }
                });


            MT("5.4",
               @"Salting and hashing",
               @"You have been asked to perform an evaluation of an application your company
wants to implement. The application’s developers claim to have field-level encryption
of every point of data in the database. As you investigate this claim, you realize
that every column in the database, other than keys, is encrypted prior to being
saved in the database using a symmetric key unique to each user. The application
guarantees this uniqueness by creating a randomized salt that is hashed and then
used as the key for that unique user.",
               new TextboxAnswer[3]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"What is the primary advantage of this approach?",
                       Explanation = @"The most significant advantage of this approach is that even if the database is hacked,
the data inside has additional security because of the encryption. As long as the
implementation is competent and secure, even if the database is copied out of your
network, getting useful information out of the system will be exponentially more complicated
than if the data was not encrypted."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What are some disadvantages of this approach?",
                       Explanation = @"There are several disadvantages of this approach. The first is that you lose the advantage
of strong-typing in your database because all data will need to be stored
as CHAR, VARCHAR, or a blob type. Another disadvantage is reduced performance
because each piece of information you access will need to be decrypted before presentation
and encrypted before saving. Finally, although this solution offers enhanced
security, it also makes other kinds of data access more complicated. Imagine trying to
add a third-party reporting tool, or scripting data changes in this environment. Because
all data will be encrypted, you will have to insert code to decrypt the data before
being able to do any reporting or scripting."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Do you think the advantage outweighs the disadvantages? Why or why not?",
                       Explanation = @"Although the concept of encrypting every field in the database is intriguing, the
drawbacks are too extreme to make it practical. If any other tools need to access the
database, they would need some way to decrypt the data as well before being able to
do anything with the information. It would be difficult to recommend using a solution
in which every piece of data is encrypted for such an important piece of the business
process."
                   }
               });


            // Section 5.5
            CurrentBackground = @"Implement a secure site with ASP.NET";


            MC("5.5 (1)", @"What roles does a CA play in a PKI?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Serves the certificate for the web server",
                        IsCorrect = false,
                        Explanation = @"The CA does not serve the certificate; the CA validates and signs it."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Validates the company or individual purchasing the certificate",
                        IsCorrect = true,
                        Explanation = @"The CA validates the company or individual."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Signs the certificate as authentic",
                        IsCorrect = true,
                        Explanation = @"The CA signs the certificate."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Provides the Domain Name System (DNS) connections between the client and server",
                        IsCorrect = false,
                        Explanation = @"The CA does not do anything with DNS."
                    }
                });

            MC("5.5 (2)", @"What are the primary differences between the AntiXSS Library and the default .NET Framework?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The AntiXSS Library takes a blocked-list approach, whereas the .NET Framework takes an accepted-list approach.",
                        IsCorrect = false,
                        Explanation = @"AntiXSS takes an accepted-list approach, which means only those characters
that have been approved are allowed through."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The AntiXSS Library has be modified to realize performance gains.",
                        IsCorrect = true,
                        Explanation = @"The AntiXSS Library is more performant than the traditional .NET
Framework."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The AntiXSS Library takes an accepted-list approach, whereas the .NET Framework takes a blocked-list approach.",
                        IsCorrect = true,
                        Explanation = @"AntiXSS takes an accepted-list approach, which means only those characters
that have been approved are allowed through."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"The AntiXSS Library offers enhanced globalization capabilities.",
                        IsCorrect = true,
                        Explanation = @"AntiXSS was designed to allow additional support for globalization."
                    }
                });

            MC("5.5 (3)", @"A SQL injection attack occurs when an application allows input submitted by the client to be run as part of a SQL command. What actions should a developer take to ensure that this doesn’t happen?",
                  new CheckboxAnswer[4]
                  {
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use Entity SQL because it does not suffer from the same risk.",
                        IsCorrect = false,
                        Explanation = @"Entity SQL is vulnerable to SQL injection attacks, so you would have to
take the same steps to protect an application using it for the data layer."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use SQLParameters to submit the parameters.",
                        IsCorrect = true,
                        Explanation = @"Using SQLParameters is the best way to manage this risk."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Use Linq-to-Entities to access the database,",
                        IsCorrect = true,
                        Explanation = @"Linq-to-Entities uses an object model as access into the data layer, so the
risk is mitigated."
                    },
                    new CheckboxAnswer()
                    { 
                        AnswerText = @"Filter out keywords and symbols.",
                        IsCorrect = false,
                        Explanation = @"This approach is dangerous because you might end up filtering out
parts of the content. It is also a blocked-list approach; an accepted list is more
secure."
                    }
                });


            MT("5.5",
               @"Storing salts",
               @"You are working with a company that recently had their database for their custom intranet system cracked by hackers. The company has determined that its 10-year-old system had several flaws that allowed the breach to occur. One of the primary vulnerabilities the company discovered was that the application stores users’ passwords in clear text. The company is refactoring its application to eliminate the vulnerability. One of its requests is for the application to use salted and hashed passwords. Although company decision makers know salted and hashed passwords are necessary, they cannot agree where to keep the salt. One party feels that keep-ing the salts in a separate column of the Users table is sufficient, whereas another party feels that it should be kept in a second, differently credentialed database that is used only for saving and retrieving salts.",
               new TextboxAnswer[3]
               {
                   new TextboxAnswer()
                   {
                       AnswerText = @"What are the advantages of storing the salts in a separate database?",
                       Explanation = @"Getting access to the salts stored in a separate database requires hackers to hack two
different databases to get access to the salt and the salted password. Storing them in
the same table as the password, or even another table of the same database, would
mean that when hackers gain access to the database, they will have access to both the
salt and the password hash. Because security includes the process of making hacking
into the system too expensive or time-consuming to be worth it, doubling the amount
of access a hacker would have to gain should make the system more secure."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"What are the advantages of storing the salts in the same database?",
                       Explanation = @"Ease of use is the primary reason for keeping the salts in the same database as the
hashed passwords. You would not have to ensure that two databases are always available
at the same time, and always in sync. The advantage of having a salt is minimal if
each user has a randomized salt because although it might make discovery of an individual’s
password easier, the amount of force necessary to crack the passwords of the
system overall will be high. In this level of discussion, that is really what the expectation
is: to protect the passwords. If the hackers have acquired a copy of the database, your
application data is already compromised. At this point, the issue is to mitigate users’
risks because of the potential of shared passwords."
                   },
                   new TextboxAnswer()
                   {
                       AnswerText = @"Which would you recommend?",
                       Explanation = @"The requirement of maintaining two separate linked, databases is extensive. Granted, it
adds the perception of security, but the only advantage that it gives is that it protects
a password, a single element of data. If every field in the database were individually
encrypted, and this same salt was used for that, it would make more sense to store it
separately from the data because the basic security of your system is enhanced."
                   }
               });


        }
    }
}