﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Revision.Models
{
    /// <summary>
    /// One possile answer to the question
    /// </summary>
    public interface IPossibleAnswer
    {
        /// <summary>
        /// The possible answer
        /// </summary>
        string AnswerText { get; set; }

        /// <summary>
        /// Does the answer contain source code?
        /// (which must be formatted as-is)
        /// </summary>
        bool IsSourceCode { get; set; }

        //////////////// Marking ///////////

        /// <summary>
        /// Is this possible answer markable? Yes for checkboxes, No for textboxes
        /// </summary>
        bool IsMarkable { get; }

        /// <summary>
        /// Did the user get this answer correct? Only valid if IsMarkable == true
        /// </summary>
        bool IsAnsweredCorrectly { get; }

        /// <summary>
        /// Did the user answer the question, at all?
        /// </summary>
        bool IsAnswered { get; }

        //////////////// Correct Answer (if any) ///////////

        /// <summary>
        /// Is this one of the correct answers? Fixed, unchangeable.
        /// </summary>
        bool IsCorrect { get; }

        /// <summary>
        /// Checkbox value, if IsMarkable
        /// </summary>
        bool ChosenAnswer { get; set; }

        /// <summary>
        /// User's text as an answer. Not IsMarkable
        /// </summary>
        string ChosenText { get; set; }

        /// <summary>
        /// Explanation for this answer being right/wrong
        /// </summary>
        string Explanation { get; set; }

        //////////////// Admin ///////////////

        /// <summary>
        /// Resets the answer to this question
        /// </summary>
        void Clear();

        /// <summary>
        /// Should all text be redacted (to protect copyright?)
        /// </summary>
        bool IsTextRedacted { get; set; }
    }
}
